unit Afeier;

{$ifdef fpc}
  {$mode objfpc}{$H+}
{$endif}

interface

uses
  {$ifndef Unix}
    Windows,
  {$endif}
  {$ifdef fpc}
    LConvEncoding,
    LCLType,
    LResources,
  {$endif}
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  IniFiles,
  Common,
  Grids,
  Zusatz,
  DelTime,
  WolfUtil;

const
  ColModus         = 'm';
  ColListe         = 'l';

type

  { TAutoHolidayForm }

  TAutoHolidayForm   = class(TForm)
    Feiertag         : TStringGrid;
    ComboBox         : TComboBox;
    CheckBox         : TCheckBox;
    BtnOk            : TButton;
    BtnCancel        : TButton;
    LabelHolidayList : TLabel;
    HolidayList      : TEdit;
    procedure          FormCreate(Sender: TObject);
    procedure          FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure          FormDestroy(Sender: TObject);
    procedure          FeiertagDrawCell(Sender: TObject; ACol, ARow: Integer;
                                        Rect: TRect; State: TGridDrawState);
    procedure          FeiertagTopLeftChanged(Sender: TObject);
    procedure          CheckBoxMouseUp(Sender: TObject; Button: TMouseButton;
                                       Shift: TShiftState; X, Y: Integer);
    procedure          CheckBoxKeyUp(Sender: TObject; var Key: Word;
                                     Shift: TShiftState);
    procedure          ComboBoxEnter(Sender: TObject);
    procedure          CheckBoxEnter(Sender: TObject);
    procedure          BtnOkClick(Sender: TObject);
    procedure          BtnCancelClick(Sender: TObject);
    procedure          FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure          CheckBoxClick(Sender: TObject);
  private
    IsDirty          : Boolean;  
    function           ModusName(s : String) : String;
    procedure          TopRow(s : String);
  public
    function           IsHolidayList(Index     : Integer;
                                     Jahr      : Integer;
                                     Var JDay  : tJulianDay;
                                     var hName : String) : Boolean;
    function           IsHoliday(JulianDay    : tJulianDay;
                                 var sHoliday : String) : Boolean;
    procedure          Load;
  end;

var
  AutoHolidayForm : TAutoHolidayForm;

implementation

{$R *.dfm}

function TAutoHolidayForm.IsHoliday(JulianDay    : tJulianDay;
                                    var sHoliday : String) : Boolean;
  var    i, k, e,
         Jahr    : Integer;
         m       : Word;
         d       : Double;
         s, t    : String;
         j       : tJulianDay;
         c       : tLanguageList;
         l       : tLanguage;
         b       : Boolean;
  begin
    IsHoliday := False;
    sHoliday  := '';
    Jahr      := 0;
    m         := 0;
    d         := 0;
    try
      c       := tLanguageList.Create;
    except
      c       := Nil;
    end;
    if c = Nil then
      MsgBox('cannot create language list', Caption, mb_IconHand)
    else
      try
        LocalJulianDate(JulianDay, Jahr, m, d);
        for k := 1 to Feiertag.RowCount - 1 do
          if Feiertag.Objects[1, k] is tComboBox then
            begin
              s := (Feiertag.Objects[1, k] as tComboBox).Name;
              Val(Copy(s, 2, length(s)), i, e);
              c.Clear;
              c.Add(SysLocale.PriLangID, SysLocale.SubLangID);
              l.PriLangID := SysLocale.PriLangID;
              l.SubLangID := SysLocale.SubLangID;
              j           := 0;
              b           := GetSpecialDay(tSpecialDay(i), Jahr, j, c);
              if j >= 0 then
                if LocalSameDate(j, JulianDay) then
                  begin
                    t := Feiertag.Cells[0, k];
                    s := (Feiertag.Objects[1, k] as tComboBox).Text;
                    if s = ModusName(IniInvisible) then
                      begin
                        t := '';
                        b := False;
                      end;
                    if (s = ModusName(IniAutomatic)) and (c <> Nil)
                       and Not LanguageInLanguages(l, c)
                    then
                      begin
                        t  := '';
                        b := False;
                      end;
                    if s = ModusName(IniNever) then
                      b := False;
                    if s = ModusName(IniAlways) then
                      b := True;
                    if b then
                      IsHoliday := True;
                    if t > '' then
                      sHoliday := sHoliday + A_LF + t;
                  end;
            end;
      finally
        c.Free;
      end;
    sHoliday := Copy(sHoliday, 2, length(sHoliday));
  end;

function TAutoHolidayForm.IsHolidayList(Index     : Integer;
                                        Jahr      : Integer;
                                        var JDay  : tJulianDay;
                                        var hName : String) : Boolean;
  var    i, e : Integer;
         s    : String;
         c    : tLanguageList;
         l    : tLanguage;
  begin
    IsHolidayList := False;
    hName         := '';
    JDay          := -1;
    try
      c           := tLanguageList.Create;
    except
      c           := Nil;
    end;
    if c = Nil then
      MsgBox('cannot create language list', Caption, mb_IconHand)
    else
      try
        if Feiertag.Objects[2, Index] is tCheckBox then
          begin
            IsHolidayList := (Feiertag.Objects[2, Index] as tCheckBox).Checked;
            s             := (Feiertag.Objects[2, Index] as tCheckBox).Name;
            Val(Copy(s, 2, length(s)), i, e);
            hName := Feiertag.Cells[0, Index];
            c.Clear;
            c.Add(SysLocale.PriLangID, SysLocale.SubLangID);
            l.PriLangID := SysLocale.PriLangID;
            l.SubLangID := SysLocale.SubLangID;
            GetSpecialDay(tSpecialDay(i), Jahr, JDay, c);
            if (JDay < 0) then
              begin
                IsHolidayList := False;
                hName         := '';
              end;
            s := '';
            if Feiertag.Objects[1, Index] is tComboBox then
              s := (Feiertag.Objects[1, Index] as tComboBox).Text;
            if s = ModusName(IniInvisible) then
              begin
                IsHolidayList := False;
                hName         := '';
              end;
            if (s = ModusName(IniAutomatic)) and (c <> Nil)
               and Not LanguageInLanguages(l, c)
            then
              begin
                IsHolidayList := False;
                hName         := '';
              end;
          end;
      finally
        c.Free;
      end;
  end;

function tAutoHolidayForm.ModusName(s : String) : String;
  begin
    case SysLocale.PriLangID of
      LANG_GERMAN:
        if s = IniAutomatic then
          ModusName := 'Automatisch'
        else if s = IniAlways then
          ModusName := 'Immer'
        else if s = IniNever then
          ModusName := 'Nie'
        else
          ModusName := 'Unsichtbar';
    else
      if s = IniAutomatic then
        ModusName := 'Automatic'
      else if s = IniAlways then
        ModusName := 'Always'
      else if s = IniNever then
        ModusName := 'Never'
      else
        ModusName := 'Invisible';
    end;
  end;

procedure TAutoHolidayForm.FormCreate(Sender: TObject);
  var     i        : tSpecialDay;
          j, k, w  : Integer;
          s        : String;
  begin
    IsDirty := False;
    LoadFormFromIni(Self);
    Feiertag.RowCount := 1;
    case SysLocale.PriLangID of
      LANG_GERMAN:
        begin
          Feiertag.Cells[0, 0] := 'Feiertag';
          Feiertag.Cells[1, 0] := 'Einstellung';
          Feiertag.Cells[2, 0] := CP1252toSystem('bersicht Liste');
        end;
    else
      begin
        Caption                  := 'Automatic holidays';
        LabelHolidayList.Caption := 'Holiday list name';
        BtnCancel.Caption        := '&Cancel';
        Feiertag.Cells[0, 0]     := 'Holiday';
        Feiertag.Cells[1, 0]     := 'Setting';
        Feiertag.Cells[2, 0]     := 'Outline list';
      end;
    end;
    w := Canvas.TextWidth(Feiertag.Cells[0, 0]);
    for i := succ(low(tSpecialDay)) to high(tSpecialDay) do
      begin
        Feiertag.RowCount := Feiertag.RowCount + 1;
        s := SpecialDayName[i];
        j := 1;
        k := 1;
        repeat
          if j + 1 = Feiertag.RowCount then
            k := 0
          else
            if Feiertag.Cells[0, j] < s then
              inc(j)
            else
              begin
                for k := Feiertag.RowCount - 1 downto j + 1 do
                  begin
                    Feiertag.Cells[0,   k] := Feiertag.Cells[0,   k - 1];
                    Feiertag.Objects[1, k] := Feiertag.Objects[1, k - 1];
                    Feiertag.Objects[2, k] := Feiertag.Objects[2, k - 1];
                  end;
                k := 0;
              end;
        until k = 0;
        Feiertag.Cells[0, j]   := s;
        Feiertag.Objects[1, j] := tComboBox.Create(Self);
        Feiertag.Objects[2, j] := tCheckBox.Create(Self);
        k := Canvas.TextWidth(s);
        if k > w then
          w := k;
        if Feiertag.Objects[1, j] is tComboBox then
          begin
            (Feiertag.Objects[1, j] as tComboBox).Visible := False;
            (Feiertag.Objects[1, j] as tComboBox).Parent  := Feiertag;
            (Feiertag.Objects[1, j] as tComboBox).Name    :=
              ColModus + IntToStr(ord(i));
            (Feiertag.Objects[1, j] as tComboBox).Style   := csDropDownList;
            (Feiertag.Objects[1, j] as tComboBox).Items.Add(
              ModusName(IniAutomatic));
            (Feiertag.Objects[1, j] as tComboBox).Items.Add(
              ModusName(IniNever));
            (Feiertag.Objects[1, j] as tComboBox).Items.Add(
              ModusName(IniAlways));
            (Feiertag.Objects[1, j] as tComboBox).Items.Add(
              ModusName(IniInvisible));
            case SysLocale.PriLangID of
              LANG_GERMAN:
                (Feiertag.Objects[1, j] as tComboBox).Hint :=
                   'Automatisch = Sprachenspezifisch immer, '
                                  + 'nie oder unsichtbar' + A_LF
                   + 'Immer = benannt und hervorgehoben' + A_LF
                   + 'Nie = benannt, aber nicht hervorgehoben' + A_LF
                   + 'Unsichtbar = weder benannt, noch hervorgehoben';
            else
              (Feiertag.Objects[1, j] as tComboBox).Hint :=
                 'Automatic = Always, never or invisible corresponding '
                              + 'to the language' + A_LF
                 + 'Always = named and marked' + A_LF
                 + 'Never = named, but not marked' + A_LF
                 + 'Invisible = neither named, nor marked';
            end;
            (Feiertag.Objects[1, j] as tComboBox).ShowHint  := True;
            (Feiertag.Objects[1, j] as tComboBox).TabStop   := True;
            (Feiertag.Objects[1, j] as tComboBox).Height    := 200;
            {$ifdef fpc}
              (Feiertag.Objects[1, j] as tComboBox).OnEnter    := @ComboBoxEnter;
            {$else}
              (Feiertag.Objects[1, j] as tComboBox).OnEnter    := ComboBoxEnter;
            {$endif}
          end;
        if Feiertag.Objects[2, j] is tCheckBox then
          begin
            (Feiertag.Objects[2, j] as tCheckBox).Visible     := False;
            (Feiertag.Objects[2, j] as tCheckBox).Parent      := Feiertag;
            (Feiertag.Objects[2, j] as tCheckBox).TabStop     := True;
            {$ifdef fpc}
              (Feiertag.Objects[2, j] as tCheckBox).OnEnter   := @CheckBoxEnter;
              (Feiertag.Objects[2, j] as tCheckBox).OnClick   := @CheckBoxClick;
            {$else}
              (Feiertag.Objects[2, j] as tCheckBox).OnEnter   := CheckBoxEnter;
              (Feiertag.Objects[2, j] as tCheckBox).OnMouseUp := CheckBoxMouseUp;
              (Feiertag.Objects[2, j] as tCheckBox).OnKeyUp   := CheckBoxKeyUp;
            {$endif}
            (Feiertag.Objects[2, j] as tCheckBox).Name        :=
              ColListe + IntToStr(ord(i));
            (Feiertag.Objects[2, j] as tCheckBox).Caption     := '';
          end;
      end;
    Feiertag.FixedRows    := 1;
    Feiertag.ColWidths[0] := w + 4 + Feiertag.GridLineWidth;
    w := Canvas.TextWidth(ModusName(IniInvisible));
    k := Canvas.TextWidth(ModusName(IniNever));
    if k > w then
      w := k;
    k := Canvas.TextWidth(ModusName(IniAlways));
    if k > w then
      w := k;
    k := Canvas.TextWidth(ModusName(IniAutomatic));
    if k > w then
      w := k;
    ComboBox.Width := w + ComboBox.Width;
    w := ComboBox.Width;
    k := Canvas.TextWidth(Feiertag.Cells[1, 0]) + 4 + Feiertag.GridLineWidth;
    if k > w then
      w := k;
    Feiertag.ColWidths[1] := w;
    Feiertag.ColWidths[2] := Canvas.TextWidth(Feiertag.Cells[2, 0])
                             + 4 + Feiertag.GridLineWidth;
    Feiertag.ClientWidth  := Feiertag.ColWidths[0]
                             + Feiertag.ColWidths[1]
                             + Feiertag.ColWidths[2]
                             + 2 + Feiertag.GridLineWidth;
    ClientWidth  := Feiertag.Width;
    Feiertag.Row := 1;
    Feiertag.DefaultRowHeight := ComboBox.Height;
    For k := 1 to Feiertag.RowCount - 1 do
      begin
        if Feiertag.Objects[1, k] is tComboBox then
          (Feiertag.Objects[1, k] as tComboBox).TabOrder := 2 * k + 6;
        if Feiertag.Objects[2, k] is tCheckBox then
          (Feiertag.Objects[2, k] as tCheckBox).TabOrder := 2 * k + 7;
      end;
    Load;
  end;

procedure TAutoHolidayForm.FormClose(Sender: TObject;
                                     var CloseAction: TCloseAction);
  begin
    if CloseAction = caFree then;
    SaveFormToIni(Self);
    Load;
  end;

procedure TAutoHolidayForm.FormDestroy(Sender: TObject);
  var i : Integer;
  begin
    for i := 1 to Feiertag.RowCount - 1 do
      begin
        Feiertag.Objects[1, i].Free;
        Feiertag.Objects[1, i] := Nil;
        Feiertag.Objects[2, i].Free;
        Feiertag.Objects[2, i] := Nil;
      end;
  end;

procedure TAutoHolidayForm.FeiertagDrawCell(
            Sender: TObject; ACol, ARow: Integer;
            Rect: TRect; State: TGridDrawState);
  begin
    if State = [] then;
    if Feiertag.Objects[ACol, ARow] is tCheckbox then
      begin
        (Feiertag.Objects[ACol, ARow] as tCheckbox).SetBounds(
          Rect.Left + (Rect.Right - Rect.Left - CheckBox.Width) div 2,
          Rect.Top,
          CheckBox.Width,
          Rect.Bottom - Rect.Top);
        (Feiertag.Objects[ACol, ARow] as tCheckbox).Visible := True;
      end
    else if Feiertag.Objects[ACol, ARow] is tComboBox then
      begin
        (Feiertag.Objects[ACol, ARow] as tComboBox).SetBounds(
          Rect.Left,
          Rect.Top,
          Rect.Right  - Rect.Left,
          Rect.Bottom - Rect.Top);
        (Feiertag.Objects[ACol, ARow] as tComboBox).Visible := True;
      end;
  end;

procedure TAutoHolidayForm.FeiertagTopLeftChanged(Sender: TObject);
  var i : Integer;
  begin
    for i := 1 to Feiertag.RowCount - 1 do
      begin
        if Feiertag.Objects[1, i] is tComboBox then
          (Feiertag.Objects[1, i] as tComboBox).Visible := False;
        if Feiertag.Objects[2, i] is tCheckBox then
          (Feiertag.Objects[2, i] as tCheckBox).Visible := False;
      end;
  end;

procedure TAutoHolidayForm.CheckBoxMouseUp(
            Sender: TObject; Button: TMouseButton;
            Shift: TShiftState; X, Y: Integer);
  begin
    if Button = mbLeft then;
    if Shift  = []     then;
    if X      = 0      then;
    if Y      = 0      then;
    (Sender as tCheckBox).Checked := not (Sender as tCheckBox).Checked;
    IsDirty := True;
  end;

procedure TAutoHolidayForm.CheckBoxKeyUp(Sender: TObject; var Key: Word;
                                         Shift: TShiftState);
  begin
    if Shift = [] then;
    if key = ord(' ') then
      begin
        (Sender as tCheckBox).Checked := not (Sender as tCheckBox).Checked;
        IsDirty := True;
      end;
  end;

procedure TAutoHolidayForm.CheckBoxEnter(Sender: TObject);
  begin
    if (Sender as tCheckBox).Top + (Sender as tCheckBox).Height
       > Feiertag.ClientHeight
    then
      TopRow((Sender as tCheckBox).Name);
  end;

procedure TAutoHolidayForm.ComboBoxEnter(Sender: TObject);
  begin
    if (Sender as tComboBox).Top + (Sender as tComboBox).Height
       > Feiertag.ClientHeight
    then
      TopRow((Sender as tComboBox).Name);
    IsDirty := True;
  end;

procedure TAutoHolidayForm.TopRow(s : String);
  var     i, e : Integer;
  begin
    val(Copy(s, 2, length(s)), i, e);
    for e := 1 to Feiertag.RowCount - 1 do
      if Feiertag.Cells[0, e] = SpecialDayName[tSpecialDay(i)] then
        Feiertag.TopRow := e;
  end;

procedure TAutoHolidayForm.BtnOkClick(Sender: TObject);
  var     IniFile : tIniFile;
          s, t    : String;
          k, j, w : Integer;
          i       : tSpecialDay;
          ok,
          Error   : Boolean;
  begin
    Ok      := True;
    Error   := False;
    try
      IniFile := tIniFile.Create(IniName);
    except
      IniFile := Nil;
    end;
    if IniFile = Nil then
      error := True
    else
      begin
        try
          if not HolidayList.ReadOnly Then
            begin
              if HolidayList.Text = '' Then
                begin
                  Ok := False;
                  case SysLocale.PriLangID of
                    Lang_German:
                      MsgBox('Bitte geben Sie eine Bezeichnung ein!',
                             Caption, mb_IconExclamation);
                  else
                    MsgBox('Please enter a description!',
                           Caption, mb_IconExclamation);
                  end;
                end
              else
                begin
                  k := 0;
                  Repeat
                    s := IntToStr(k);
                    t := IniFile.ReadString(IniHolidayList, s, '');
                    if t = HolidayList.Text then
                      begin
                        Ok := False;
                        t  := '';
                      end;
                    inc(k);
                  until t = '';
                  if not ok then
                    case SysLocale.PriLangID of
                      Lang_German:
                        MsgBox('Diese Bezeichnung ist bereits vergeben',
                               Caption, mb_IconExclamation);
                    else
                      MsgBox('This description is already in use',
                             Caption, mb_IconExclamation);
                    end
                  else
                    begin
                      Ok := IsIni(HolidayList.Text);
                      if not ok then
                        MsgBox(HolidayList.Hint, Caption, mb_IconExclamation);
                    end;
                end;
            end;
          If not ok then
            HolidayList.SetFocus
          else
            begin
              for k := 1 to Feiertag.RowCount - 1 do
                begin
                  i := NormalDay;
                  if Feiertag.Objects[1, k] is tComboBox then
                    begin
                      s := (Feiertag.Objects[1, k] as tComboBox).Name;
                      val(copy(s, 2, length(s)), j, w);
                      i := tSpecialDay(j);
                    end;
                  s := IniAutomatic;
                  if Feiertag.Objects[1, k] is tComboBox then
                    begin
                      if (Feiertag.Objects[1, k] as tComboBox).Text
                         = ModusName(IniInvisible)
                      then
                        s := IniInvisible
                      else if (Feiertag.Objects[1, k] as tComboBox).Text
                              = ModusName(IniAlways)
                      then
                        s := IniAlways
                      else if (Feiertag.Objects[1, k] as tComboBox).Text
                              = ModusName(IniNever)
                      then
                        s := IniNever;
                    end;
                  if Feiertag.Objects[2, k] is tCheckBox then
                    if (Feiertag.Objects[2, k] as tCheckBox).Checked then
                      s := s + ':*'
                    else
                      s := s + ':'
                  else
                    s := s + ':*';
                  IniFile.WriteString(Name + ' ' + HolidayList.Text,
                                      SpecialDayID[i], s);
                end;
              IsDirty     := False;
              ModalResult := mrOk;
            end;
        except
          error := True;
        end;
        IniFile.Free;
      end;
    if error then
      case SysLocale.PriLangID of
        Lang_German: MsgBox('kann automatische Feiertage nicht speichern',
                            Caption, mb_IconHand);
      else
        MsgBox('cannot save automatic holidays', Caption, mb_IconHand);
      end;
  end;

procedure TAutoHolidayForm.Load;
  var     IniFile : tIniFile;
          i       : tSpecialDay;
          j, k, w : Integer;
          s, t    : String;
          b       : Boolean;
  begin
    if HolidayList.Text = '' Then
      begin
        HolidayList.Color    := clWindow;
        HolidayList.ReadOnly := False;
        HolidayList.Tabstop  := True;
        HolidayList.ShowHint := True;
        ActiveControl        := HolidayList;
        for k := 1 to Feiertag.RowCount - 1 do
          begin
            if Feiertag.Objects[1, k] is tComboBox then
              begin
                (Feiertag.Objects[1, k] as tComboBox).ItemIndex :=
                (Feiertag.Objects[1, k] as tComboBox).Items.IndexOf(
                   ModusName(IniAutomatic));
              end;
            if Feiertag.Objects[2, k] is tCheckBox then
              (Feiertag.Objects[2, k] as tCheckBox).Checked := True;
          end;
      end
    else
      begin
        HolidayList.Color    := clBtnFace;
        HolidayList.ReadOnly := True;
        HolidayList.Tabstop  := False;
        HolidayList.ShowHint := False;
        ActiveControl        := Feiertag;
        try
          IniFile := tIniFile.Create(IniName);
        except
          IniFile := Nil;
        end;
        if IniFile <> Nil then
          try
            for k := 1 to Feiertag.RowCount - 1 do
              begin
                i := NormalDay;
                b := True;
                if Feiertag.Objects[1, k] is tComboBox then
                  begin
                    s := (Feiertag.Objects[1, k] as tComboBox).Name;
                    val(copy(s, 2, length(s)), j, w);
                    i := tSpecialDay(j);
                  end;
                t := IniFile.ReadString(Name + ' ' + HolidayList.Text,
                                        SpecialDayID[i], IniAutomatic);
                j := Pos(':', t);
                if j > 0 then
                  begin
                    b := Copy(t, j + 1, 1) = '*';
                    t := Copy(t, 1, j - 1);
                  end;
                if Feiertag.Objects[1, k] is tComboBox then
                  (Feiertag.Objects[1, k] as tComboBox).ItemIndex :=
                    (Feiertag.Objects[1, k] as tComboBox).Items.IndexOf(
                      ModusName(t));
                if Feiertag.Objects[2, k] is tCheckBox then
                  (Feiertag.Objects[2, k] as tCheckBox).Checked := b;
              end;
          finally
            IniFile.Free;
          end;
      end;
    IsDirty := False;
  end;

procedure TAutoHolidayForm.BtnCancelClick(Sender: TObject);
  begin
    IsDirty     := False;
    ModalResult := mrCancel;
  end;

procedure TAutoHolidayForm.FormCloseQuery(Sender: TObject;
                                          var CanClose: Boolean);
  var     i : Integer;
  begin
    if IsDirty then
      case SysLocale.PriLangID of
        Lang_German:
          i := MsgBox('Eingaben verwerfen?', Caption,
                      mb_OkCancel or mb_IconQuestion);
      else
        i := MsgBox('Cancel changes?', Caption,
                    mb_OkCancel or mb_IconQuestion);
      end
    else
      i := idOk;
    if i = idOk Then
      CanClose := True
    else
      CanClose := False;
  end;

procedure TAutoHolidayForm.CheckBoxClick(Sender: TObject);
  begin
    IsDirty := True;
  end;

end.
