unit common;

{$ifdef fpc}
  {$mode objfpc}{$H+}
{$endif}

interface

uses
  {$ifndef Unix}
    Windows,
  {$endif}
  {$ifdef fpc}
    LConvEncoding,
  {$endif}
  SysUtils,
  Graphics,
  Zusatz,
  WolfUtil;

Type
  tFontElement        = (eFont,
                         eJahr,
                         eFeiertagListe,
                         eArbeitsTag,
                         eSamstag,
                         eFeiertag,
                         eTermin);

  tLanguage           = (Deutsch, English);

  tFormular           = (FormularEineSeite,
                         FormularMonat,
                         FormularRingbuch,
                         FormularBilder,
                         FormularDatei);

const
  IniHeight           = 'Height';
  IniWidth            = 'Width';
  IniForm             = 'Form';
  IniYear             = 'Year';
  IniLayouts          = 'Layouts';
  IniLayout           = 'Layout';
  IniActual           = 'Actual';
  IniFontName         = 'Font Name';
  IniFontSize         = 'Font Size';
  IniFontStyle        = 'Font Style';
  IniFontColor        = 'Font Color';
  IniFontCharset      = 'Font Charset';
  IniPunches          = 'Punches';
  IniPunch1           = 'Punch1';
  IniPunch2           = 'Punch2';
  IniPunch3           = 'Punch3';
  IniDateLists        = 'DateLists';
  IniList             = 'List';
  IniAutomatic        = 'Automatic';
  IniAlways           = 'Always';
  IniNever            = 'Never';
  IniInvisible        = 'Invisible';
  IniNotFound         = 'Not Found';
  IniHeadLine         = 'HeadLine';
  IniHolidayList      = 'Holiday List';
  IniOrientation      = 'Orientation';
  IniCheckSizes       = 'Check sizes';
  IniWeeks            = 'Weeks';
  IniColumns          = 'Columns';
  IniMarginTop        = 'Top margin';
  IniMarginLeft       = 'Left margin';
  IniMarginRight      = 'Right margin';
  IniMarginBottom     = 'Bottom margin';
  IniFooter           = 'Footer';
  IniDocFrame         = 'Document frame';
  IniMonthFrame       = 'Frame each month';
  IniFrom             = 'from';
  IniUntil            = 'until';
  IniPicture          = 'Picture';
  IniComment          = 'Comment';
  IniStartmenue       = 'StartMenu';
  IniFormBackground   = 'FormBackground';
  IniDayBackground    = 'DayBackground';
  IniBackgroundMode   = 'BackgroundMode';
  IniBkGndStretch     = 'Stretch';
  IniBkGndTile        = 'Tile';
  IniFileTypes        = 'FileTypes';
  IniGraphicImport    = 'GraphicImport';
  IniGraphicExport    = 'GraphicExport';
  IniSaveFile         = 'SaveFile';
  IniDayMonthFormat   = 'DayMonthFormat';
  IniDayMonthAlign    = 'DayMonthAlign';
  IniLeft             = 'left';
  IniRight            = 'right';

  Trenn               = '|';
  Stern               = '*.';
  AllStern            = ';' + Stern;
  Bild                = '-Bild'    + Trenn + Stern;
  Picture             = '-Picture' + Trenn + Stern;
  CalPrint            = 'CalPrint';

  IniFontElement      : Array[eJahr .. eTermin] of String
                      = ('Year',
                         'Holiday List',
                         'Normal',
                         'Saturday',
                         'Holiday',
                         'Date');

  IniIsDate           : Array[Boolean] of String
                      = ('', ' Date');

  MMPenWidth          =    0.2;
  MMLoch              =    3;
  MMTextRand          =   12;

function CalcTermin(Satz : String; Jahr : Integer; var Meldung : String)
         : String;
function FormularName(Formular : tFormular; PriLangID : Word) : String;
function LinkName : String;

implementation

const c_Funktion  = '$';
      c_BeginnArg = '(';
      c_EndArg    = ')';

function LinkName : String;
  begin
    case SysLocale.PriLangID of
      LANG_GERMAN:
        LinkName := 'Kalender Drucker';
    else
        LinkName := 'Calendar printer';
    end;
  end;


function FormularName(Formular : tFormular; PriLangID : Word) : String;
  begin
    FormularName := '';
    case Formular of
      FormularEineSeite:
        case PriLangId of
          LANG_GERMAN : FormularName := CP1252toSystem('bersicht');
        else
          FormularName := 'Overview';
        end;
      FormularMonat:
        case PriLangId of
          LANG_GERMAN : FormularName := 'Monatsplaner';
        else
          FormularName := 'Monthly schedule';
        end;
      FormularRingbuch:
        case PriLangId of
          LANG_GERMAN : FormularName := 'Ringbuch';
        else
          FormularName := 'Appointment book';
        end;
      FormularBilder:
        case PriLangId of
          LANG_GERMAN : FormularName := 'Bilder';
        else
          FormularName := 'Pictures';
        end;
      FormularDatei:
        case PriLangId of
          LANG_GERMAN : FormularName := 'Datei';
        else
          FormularName := 'File';
        end;
    end;
  end;

function CalcFunktion(Funktion, StrArgument : String; Jahr : Integer;
                      var Meldung : String) : String;
  var    Argument,
         Error    : integer;
         s, f     : string;
  begin
    CalcFunktion := c_Funktion + Funktion
                    + c_BeginnArg + StrArgument + c_EndArg;
    f            := UpperCase(Trim(Funktion));
    val(trim(StrArgument), Argument, error);
    if error <> 0 then
      case SysLocale.PriLangID of
        LANG_GERMAN:
          Meldung := CP1252toSystem('Bitte geben Sie ein gltiges Jahr in ')
                     + c_Funktion + Funktion
                     + c_BeginnArg + '...' + c_EndArg + ' an Stelle von '
                     + StrArgument + ' ein!';
      else
        Meldung := 'Please enter a valid year in '
                   + c_Funktion + Funktion
                   + c_BeginnArg + '...' + c_EndArg + ' in place of '
                   + StrArgument + '!';
      end
    else
      begin
        if Jahr * Argument < 0 then
          Argument := abs(Jahr) + abs(Argument) - 1
        else
          Argument := abs(Jahr - Argument);
        str(Argument, s);
        if f = 'JAHRE' then
          if Argument = 1 then
            CalcFunktion := s + ' ' + Copy(Funktion, 1, 4)
          else
            CalcFunktion := s + ' ' + Funktion
        else if f = 'JAHREN' then
          if Argument = 1 then
            CalcFunktion := s + ' ' + Copy(Funktion, 1, 4)
          else
            CalcFunktion := s + ' ' + Funktion
        else if f = 'YEARS' then
          if Argument = 1 then
            CalcFunktion := s + ' ' + Copy(Funktion, 1, 4)
          else
            CalcFunktion := s + ' ' + Funktion
        else if (f = 'ZAHL') or (f = 'NUMBER') then
          CalcFunktion := s
        else if f = 'ORDINAL' then
          CalcFunktion := Ordinal(Argument)
        else
          case SysLocale.PriLangID of
            LANG_GERMAN:
              Meldung := CP1252toSystem('Bitte geben Sie eine gltige Funktion ')
                         + c_Funktion + '...'
                         + c_BeginnArg + StrArgument + c_EndArg + ' an Stelle von '
                         + Funktion + ' an!';
          else
            Meldung := 'Please enter a valid function '
                       + c_Funktion + '...'
                       + c_BeginnArg + StrArgument + c_EndArg + ' in place of '
                       + Funktion + '!';
          end;
      end;
  end;

function CalcTermin(Satz : String; Jahr : Integer; var Meldung : String)
         : String;
  var    Erledigt,
         Rest,
         Funktion : String;
         w        : Word;
  begin
    Meldung  := '';
    Erledigt := '';
    Rest     := Satz;
    while Rest > '' do
      begin
        w := Pos(c_Funktion, Rest);
        if w = 0 then
          begin
            Erledigt := Erledigt + Rest;
            Rest     := '';
          end
        else
          begin
            Erledigt := Erledigt + Copy(Rest, 1, w - 1);
            Rest     := Copy(Rest, w + 1, $ff);
            w        := Pos(c_EndArg, Rest);
            if w = 0 then
              begin
                case SysLocale.PriLangID of
                  LANG_GERMAN:
                    Meldung :=
                      CP1252toSystem('Bitte geben Sie eine gltige Funktion ')
                      + c_Funktion  + '....'
                      + c_BeginnArg + '....' + c_EndArg + ' an!';
                else
                  Meldung := 'Please enter a valid function '
                             + c_Funktion  + '....'
                             + c_BeginnArg + '....' + c_EndArg + '!';
                end;
                Erledigt := Erledigt + c_Funktion + Rest;
                Rest     := '';
              end
            else
              begin
                Funktion := Copy(Rest, 1, w - 1);
                Rest     := Copy(Rest, w + 1, $ff);
                w        := Pos(c_BeginnArg, Funktion);
                if w = 0 then
                  begin
                    case SysLocale.PriLangID of
                      LANG_GERMAN:
                        Meldung :=
                          CP1252toSystem('Bitte geben Sie eine gltige Funktion ')
                          + c_Funktion  + '....'
                          + c_BeginnArg + '....' + c_EndArg + ' an!'
                    else
                      Meldung := 'Please enter a valid function '
                                 + c_Funktion  + '....'
                                 + c_BeginnArg + '....' + c_EndArg + '!';
                    end;
                    Erledigt := Erledigt + c_Funktion + Funktion + c_EndArg;
                  end
                else
                  Erledigt := Erledigt +
                    CalcFunktion(Copy(Funktion, 1, w - 1),
                                 Copy(Funktion, w + 1, $ff),
                                 Jahr, Meldung);
              end;
          end;
      end;
    CalcTermin := Erledigt;
  end;

end.
