unit dates;

{$ifdef fpc}
  {$mode objfpc}{$H+}
{$endif}

interface

uses
  {$ifdef fpc}
    LCLType,
    LCLIntf,
    LResources,
    LConvEncoding,
  {$endif}
  {$ifdef Unix}
    Unix,
  {$else}
    Windows,
  {$endif}
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  Grids,
  IniFiles,
  Common,
  Zusatz,
  Buttons,
  ExtCtrls,
  Menus,
  ClipBrd,
  DelTime,
  WolfUtil,
  SortGrid;

type
  tCols             = (DayCol, MonthCol, YearCol, TextCol);

  { TDatesForm }

  TDatesForm        = class(TForm)
    Liste           : TEdit;
    FindText        : TEdit;
    LabelFind       : TLabel;
    LabelListName   : TLabel;
    LabelExample    : TLabel;
    LabelYear       : TLabel;
    LabelText       : TLabel;
    LabelFYearsOld  : TLabel;
    LabelPrinted    : TLabel;
    LabelYearsOld   : TLabel;
    LabelFYearsAgo  : TLabel;
    LabelYearsAgo   : TLabel;
    Panel           : TPanel;
    GroupBox        : TGroupBox;
    BtnOk           : TBitBtn;
    BtnCancel       : TBitBtn;
    BtnInsert       : TBitBtn;
    BtnDelete       : TBitBtn;
    BtnFind         : TBitBtn;
    BtnSort         : TBitBtn;
    BtnPrint        : TBitBtn;
    MainMenu        : TMainMenu;
    MenuDatei       : TMenuItem;
    MenuOk          : TMenuItem;
    MenuAbbruch     : TMenuItem;
    MenuPrint       : TMenuItem;
    MenuBearbeiten  : TMenuItem;
    MenuInsert      : TMenuItem;
    MenuDelete      : TMenuItem;
    MenuSort        : TMenuItem;
    MenuFind        : TMenuItem;
    LabelFBirthday  : TLabel;
    LabelBirthday   : TLabel;
    LabelLeeresJahr : TLabel;
    BtnCopy         : TBitBtn;
    BtnPaste        : TBitBtn;
    MenuClipboard   : TMenuItem;
    MenuPaste       : TMenuItem;
    MenuCopy        : TMenuItem;
    Grid            : tSortGrid;
    procedure         BtnOkClick(Sender: TObject);
    procedure         DelRow(Sender: TObject);
    procedure         InsertRow(Sender: TObject);
    procedure         BtnSortClick(Sender: TObject);
    procedure         FormResize(Sender: TObject);
    procedure         FormCreate(Sender: TObject);
    procedure         BtnPrintClick(Sender: TObject);
    procedure         Abbruch(Sender: TObject);
    procedure         CheckFind(Sender: TObject);
    procedure         Find(Sender: TObject);
    procedure         FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure         BtnCopyClick(Sender: TObject);
    procedure         BtnPasteClick(Sender: TObject);
    procedure         FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure         GridKeyPress(Sender: TObject; var Key: Char);
    procedure         GridMouseUp(Sender: TObject; Button: TMouseButton;
                                  Shift: TShiftState; X, Y: Integer);
  private
    IsDirty         : Boolean;  
    function          Check     : Boolean;
    function          Sortieren : Boolean;
    procedure         EnableAll(MakeEnabled : Boolean);
    procedure         NewSize;
  public
    procedure         Load;
    function          IsTermin(Day, Month, Year : Integer) : String;
  end;

var
  DatesForm : TDatesForm;

implementation

{$R *.dfm}

uses main;

procedure TDatesForm.EnableAll(MakeEnabled : Boolean);
  begin
    if MakeEnabled then
      Screen.Cursor        := crDefault
    else
      Screen.Cursor        := crHourGlass;
    MenuDatei.enabled      := MakeEnabled;
    MenuBearbeiten.enabled := MakeEnabled;
    MenuClipboard.enabled  := MakeEnabled;
    MenuFind.enabled       := MakeEnabled and (FindText.Text > '');
    BtnOk.enabled          := MakeEnabled;
    BtnCancel.enabled      := MakeEnabled;
    BtnInsert.enabled      := MakeEnabled;
    BtnDelete.enabled      := MakeEnabled and (Grid.RowCount > 2);
    BtnCopy.enabled        := MakeEnabled;
    BtnPaste.enabled       := MakeEnabled;
    FindText.enabled       := MakeEnabled;
    BtnFind.enabled        := MakeEnabled and (FindText.Text > '');
    BtnSort.enabled        := MakeEnabled;
    BtnPrint.enabled       := MakeEnabled;
    Liste.enabled          := MakeEnabled;
    Grid.enabled           := MakeEnabled;
  end;

procedure TDatesForm.Load;
  var     i       : LongInt;
          p, j    : Integer;
          s, t    : String;
          IniFile : tIniFile;
          error   : Boolean;
  begin
    for i := 0 to Grid.ColCount - 1 do
      Grid.Cols[i].Clear;
    Grid.RowCount := 2;
    case SysLocale.PriLangID of
      Lang_German:
        begin
          Grid.Cells[ord(DayCol),   0] := 'Tag';
          Grid.Cells[ord(MonthCol), 0] := 'Monat';
          Grid.Cells[ord(YearCol),  0] := 'Jahr';
          Grid.Cells[ord(TextCol),  0] := 'Text';
        end;
    else
      Grid.Cells[ord(DayCol),   0] := 'Day';
      Grid.Cells[ord(MonthCol), 0] := 'Month';
      Grid.Cells[ord(YearCol),  0] := 'Year';
      Grid.Cells[ord(TextCol),  0] := 'Text';
    end;
    if Liste.Text = '' Then
      begin
        Liste.Color    := clWindow;
        Liste.ReadOnly := False;
        Liste.Tabstop  := True;
        Liste.ShowHint := True;
        if Liste.enabled then
          ActiveControl := Liste;
      end
    else
      begin
        error          := False;
        Liste.Color    := clBtnFace;
        Liste.ReadOnly := True;
        Liste.Tabstop  := False;
        Liste.ShowHint := False;
        if Grid.enabled then
          ActiveControl := Grid;
        try
          IniFile := tIniFile.Create(IniName);
        except
          IniFile := Nil;
        end;
        if IniFile = Nil then
          error := True
        else
          begin
            try
              i := 1;
              with Grid do
                repeat
                  s := IniFile.ReadString(IniList + ' ' + Liste.Text,
                                          IntToDez(i, 3, False), '');
                  if s > '' then
                    begin
                      inc(i);
                      RowCount := i;
                      p := Pos('.', s);
                      if p > 0 then
                        begin
                          t := Copy(s, 1, p - 1);
                          s := Copy(s, p + 1, $ff);
                          Cells[ord(DayCol), RowCount - 1] :=
                            IntToStr(StrToInt(t));
                        end;
                      p := Pos('.', s);
                      if p > 0 then
                        begin
                          t := Copy(s, 1, p - 1);
                          s := Copy(s, p + 1, $ff);
                          Cells[ord(MonthCol), RowCount - 1] :=
                            IntToStr(StrToInt(t));
                        end;
                      if Copy(s, 1, 1) <> ' ' then
                        begin
                          val(s, j, p);
                          if p = 0 then
                            begin
                              Cells[ord(YearCol), RowCount - 1] := s;
                              s := '';
                            end
                          else
                            if p > 1 then
                              begin
                                Cells[ord(YearCol), RowCount - 1] :=
                                  Copy(s, 1, p - 1);
                                s := Copy(s, p + 1, $ff);
                              end;
                          p := j;
                        end;
                      Cells[ord(TextCol), RowCount - 1] := Trim(s);
                      MinWidthCell(ord(TextCol), RowCount - 1);
                    end;
                until s = '';
            except
              error := True;
            end;
            IniFile.Free;
          end;
        if error then
          case SysLocale.PriLangID of
            Lang_German: MsgBox('kann Termine nicht laden', Caption, mb_IconHand);
          else
            MsgBox('cannot load dates', Caption, mb_IconHand);
          end;
      end;
    NewSize;
    BtnDelete.enabled := Grid.RowCount > 2;
    IsDirty := False;
  end;

procedure TDatesForm.BtnOkClick(Sender: TObject);
  var     i       : Longint;
          IniFile : tIniFile;
          error   : Boolean;
  begin
    EnableAll(False);
    if Sortieren then
      begin
        error := False;
        try
          IniFile := tIniFile.Create(IniName);
        except
          IniFile := Nil;
        end;
        if IniFile = Nil then
          error := True
        else
          begin
            try
              IniFile.EraseSection(IniList + ' ' + Liste.Text);
              for i := 1 to Grid.RowCount - 1 do
                IniFile.WriteString(IniList + ' ' + Liste.Text,
                                    IntToDez(i, 3, False),
                                    Grid.Cells[ord(DayCol), i]     + '.'
                                    + Grid.Cells[ord(MonthCol), i] + '.'
                                    + Grid.Cells[ord(YearCol),  i] + ' '
                                    + Grid.Cells[ord(TextCol),  i]);
            except
              error := True;
            end;
            IniFile.Free;
          end;
        if error then
          case SysLocale.PriLangID of
            Lang_German: MsgBox('kann Termine nicht speichern',
                                Caption, mb_IconHand);
          else
            MsgBox('cannot save dates', Caption, mb_IconHand);
          end;
        IsDirty     := False;
        ModalResult := mrOk;
      end;
    EnableAll(True);
  end;

procedure TDatesForm.DelRow(Sender: TObject);
  begin
    Grid.DeleteRows(Grid.Selection.Top, Grid.Selection.Bottom);
    Grid.SetFocus;
    BtnDelete.enabled := Grid.RowCount > 2;
  end;

procedure TDatesForm.InsertRow(Sender: TObject);
  begin
    Grid.InsertRows(Grid.Row, 1);
    Grid.SetFocus;
    BtnDelete.enabled := True;
  end;

function TDatesForm.Check : Boolean;
  var    i        : LongInt;
         error,
         day,
         month,
         year     : Integer;
         s, t     : String;
         ok, ex   : Boolean;
         IniFile  : tIniFile;
  begin
    ok        := True;
    ex        := False;
    try
      IniFile := tIniFile.Create(IniName);
    except
      IniFile := Nil;
    end;
    if IniFile = Nil then
      ex := True
    else
      begin
        try
          if not Liste.ReadOnly Then
            if Liste.Text = '' Then
              begin
                ok := False;
                case SysLocale.PriLangID of
                  Lang_German:
                    MsgBox('Bitte geben Sie eine Bezeichnung ein!',
                           Caption,  mb_IconExclamation);
                else
                  MsgBox('Please enter a description!',
                         Caption,  mb_IconExclamation);
                end;
              end
            else
              begin
                i := 0;
                repeat
                  t := IntToStr(i);
                  s := IniFile.ReadString(IniDateLists, t, '');
                  if s = Liste.Text then
                    begin
                      ok := False;
                      s  := '';
                    end;
                  inc(i);
                until s = '';
                if not ok then
                  case SysLocale.PriLangID of
                    Lang_German:
                      MsgBox('Diese Bezeichnung ist bereits vergeben',
                             Caption,  mb_IconExclamation);
                  else
                    MsgBox('This description is already in use',
                           Caption,  mb_IconExclamation);
                  end
                else
                  begin
                    ok := IsIni(Liste.Text);
                    if not ok then
                      MsgBox(Liste.Hint, Caption, mb_IconExclamation);
                  end;
              end;
        except
          ex := True;
        end;
        IniFile.Free;
      end;
    if ex then
      begin
        case SysLocale.PriLangID of
          Lang_German: MsgBox(CP1252ToSystem('kann Termine nicht prfen'),
                              Caption, mb_IconHand);
        else
          MsgBox('cannot check dates', Caption, mb_IconHand);
        end;
        ok := False;
      end;
    if not ok Then
      begin
        if Liste.enabled then
          Liste.SetFocus;
      end
    else
      begin
        Ok := True;
        i  := 1;
        with Grid do
          while Ok and (i < RowCount) do
            begin
              Month := 0;
              Col   := ord(DayCol);
              val(Cells[Col, i], day, error);
              if (Error <> 0) or (day < 1) or (day > 31) Then
                begin
                  Row := i;
                  case SysLocale.PriLangID of
                    Lang_German:
                      MsgBox(CP1252ToSystem(
                        'Bitte geben Sie einen gltigen Tag ein!'),
                        Caption, mb_IconExclamation)
                  else
                    MsgBox('Please enter a valid day!',
                           Caption, mb_IconExclamation);
                  end;
                  Ok := False;
                end;
              if Ok then
                begin
                  Cells[ord(DayCol), i] := IntToStr(Day);
                  Col := ord(MonthCol);
                  val(Cells[Col, i], month, error);
                  if (Error <> 0) or (month < 1) or (month > 12) Then
                    begin
                      Row := i;
                      case SysLocale.PriLangID of
                        Lang_German:
                          MsgBox(CP1252ToSystem(
                            'Bitte geben Sie einen gltigen Monat ein!'),
                            Caption, mb_IconExclamation);
                      else
                        MsgBox('Please enter a valid month!',
                               Caption, mb_IconExclamation);
                      end;
                      Ok := False;
                    end;
                end;
              if Ok then
                begin
                  Cells[ord(MonthCol), i] := IntToStr(Month);
                  if (month in [4, 6, 9, 11]) and (day > 30) or
                     (month = 2) and (day > 29) then
                    begin
                      Row := i;
                      Col := ord(DayCol);
                      case SysLocale.PriLangID of
                        Lang_German:
                          MsgBox(CP1252ToSystem(
                            'Bitte geben Sie ein gltiges Datum ein!'),
                            Caption, mb_IconExclamation)
                      else
                        MsgBox('Please enter a valid date!',
                               Caption, mb_IconExclamation);
                      end;
                      Ok := False;
                    end;
                end;
              if Ok then
                begin
                  Col := ord(YearCol);
                  Cells[Col, i] := trim(Cells[Col, i]);
                  if Cells[Col, i] > '' then
                    begin
                      val(Cells[Col, i], year, error);
                      if (Error <> 0) or (year < MinYear) Then
                        begin
                          Row := i;
                          case SysLocale.PriLangID of
                            Lang_German:
                              MsgBox(CP1252ToSystem(
                                'Bitte geben Sie ein gltiges Jahr ein!'),
                                Caption, mb_IconExclamation);
                          else
                            MsgBox('Please enter a valid year!',
                                   Caption, mb_IconExclamation);
                          end;
                          Ok := False;
                        end;
                      if Ok then
                        begin
                          s := IntToStr(Year);
                          Cells[Col, i] := s;
                        end;
                    end;
                end;
              if Ok then
                begin
                  Col := ord(TextCol);
                  if Cells[Col, i] = '' then
                    begin
                      Row := i;
                      case SysLocale.PriLangID of
                        Lang_German:
                          MsgBox('Bitte geben Sie einen Text ein!',
                                 Caption, mb_IconExclamation);
                      else
                        MsgBox('Please enter a text!',
                               Caption, mb_IconExclamation);
                      end;
                      Ok := False;
                    end;
                end;
              if Ok then
                begin
                  Col := ord(TextCol);
                  CalcTermin(Cells[Col, i], 0, s);
                  if s > '' then
                    begin
                      Row := i;
                      MsgBox(s, Caption, mb_IconExclamation);
                      Ok := False;
                    end;
                end;
              Inc(i);
            end;
        if not Ok Then
          if Grid.enabled then
            Grid.SetFocus;
      end;
    Check := Ok;
  end;

function TDatesForm.Sortieren : Boolean;
  begin
    Sortieren := False;
    if Check Then
      begin
        Sortieren := True;
        Grid.DoSort(1, Grid.RowCount - 1);
      end;
  end;

procedure TDatesForm.BtnSortClick(Sender: TObject);
  begin
    EnableAll(False);
    Sortieren;
    EnableAll(True);
    Grid.SetFocus;
  end;

function TDatesForm.IsTermin(Day, Month, Year : Integer) : String;
  var    sTag,
         sMonat,
         sJahr,
         s       : String;
         i       : LongInt;
  begin
    s := '';
    if Liste.Text > '' then
      begin
        sTag   := IntToStr(Day);
        sMonat := IntToStr(Month);
        sJahr  := IntToStr(Year);
        with Grid do
          for i := 1 to RowCount - 1 do
            if (Cells[ord(DayCol), i] = sTag)
               and (Cells[ord(MonthCol), i] = sMonat)
            Then
              if (Cells[ord(YearCol), i] = '')
                 or (Cells[ord(YearCol), i] = sJahr)
              Then
                s := s + Cells[ord(TextCol), i] + A_LF;
      end;
    IsTermin := s;
  end;

procedure TDatesForm.FormResize(Sender: TObject);
  begin
    NewSize;
    SaveFormToIni(Self);
  end;

procedure TDatesForm.NewSize;
  begin
    Grid.ExactWidthCell(ord(DayCol), 0);
    Grid.ExactWidthCell(ord(MonthCol), 0);
    Grid.ColWidths[ord(YearCol)] := Canvas.TextWidth('-####')
      + 2 + Grid.GridLineWidth;
    Grid.ExactWidthCell(ord(TextCol), 0);
    Grid.MinWidthCol(ord(TextCol));
    Grid.FillWidth(ord(TextCol));
  end;

procedure TDatesForm.FormCreate(Sender: TObject);
  begin
    IsDirty                        := False;
    Grid.ColCount                  := ord(High(tCols)) + 1;
    Grid.FormatType[ord(DayCol)]   := sgfInteger;
    Grid.FormatType[ord(MonthCol)] := sgfInteger;
    Grid.FormatType[ord(YearCol)]  := sgfInteger;
    Grid.FormatType[ord(TextCol)]  := sgfString;
    Grid.Alignment[ord(DayCol)]    := taRightJustify;
    Grid.Alignment[ord(MonthCol)]  := taRightJustify;
    Grid.Alignment[ord(YearCol)]   := taRightJustify;
    Grid.AddSortColRow(ord(MonthCol));
    Grid.AddSortColRow(ord(DayCol));
    Grid.AddSortColRow(ord(YearCol));
    Grid.AddSortColRow(ord(TextCol));
    LoadFormFromIni(Self);
    case SysLocale.PriLangID of
      Lang_German:
    else
      begin
        Caption                 := 'Merged dates';
        LabelFind.Caption       := 'Find item';
        LabelListName.Caption   := 'Dates list name';
        LabelExample.Caption    := 'Example:';
        LabelYear.Caption       := 'A Calendar is printed to the year 1999.';
        LabelText.Caption       := 'The text:';
        LabelFBirthday.Caption  := '$number(1990)th birthday';
        LabelPrinted.Caption    := 'will be printed as:';
        LabelBirthday.Caption   := '9th birthday';
        LabelFYearsOld.Caption  := '$years(1998)';
        LabelYearsOld.Caption   := '1 year';
        LabelFYearsAgo.Caption  := '$ordinal(1998) birthday';
        LabelYearsAgo.Caption   := '1st birthday';
        LabelLeeresJahr.Caption := 'empty year = every year';
        Liste.hint              := 'only letters, digits, blanks and _ allowed';
        BtnCancel.Hint          := 'Cancel';
        BtnInsert.Hint          := 'Insert a line';
        BtnDelete.Hint          := 'Delete lines range';
        BtnFind.Hint            := 'Find';
        BtnSort.Hint            := 'Sort';
        BtnPrint.Hint           := 'Print';
        BtnCopy.Hint            := 'Copy';
        BtnPaste.Hint           := 'Paste';
        MenuDatei.Caption       := '&File';
        MenuOk.Caption          := '&Ok';
        MenuAbbruch.Caption     := '&Cancel';
        MenuPrint.Caption       := '&Print';
        MenuBearbeiten.Caption  := '&Edit';
        MenuInsert.Caption      := '&Insert a line';
        MenuDelete.Caption      := '&Delete lines range';
        MenuSort.Caption        := '&Sort';
        MenuFind.Caption        := '&Find';
        MenuClipboard.Caption   := 'Clip&board';
        MenuCopy.Caption        := '&Copy';
        MenuPaste.Caption       := '&Paste'; 
      end;
    end;
  end;

procedure TDatesForm.BtnPrintClick(Sender: TObject);
  begin
    EnableAll(False);
    if Sortieren then
      MainForm.PrintDates;
    EnableAll(True);
  end;

procedure TDatesForm.Abbruch(Sender: TObject);
  begin
    IsDirty     := False;
    ModalResult := mrCancel;
  end;

procedure TDatesForm.CheckFind(Sender: TObject);
  begin
    if FindText.Text > '' then
      begin
        BtnFind.enabled  := True;
        BtnFind.default  := True;
        BtnOk.default    := False;
        MenuFind.enabled := True;
      end
    else
      begin
        BtnFind.enabled  := False;
        BtnFind.default  := False;
        BtnOk.default    := True;
        MenuFind.enabled := False;
      end;
  end;

procedure TDatesForm.Find(Sender: TObject);
  var     r, c,
          StartCol,
          StartRow : LongInt;
          found    : Boolean;
          FindS    : String;
  begin
    found := False;
    FindS := AnsiUpperCase(FindText.Text);
    with Grid do
      begin
        StartRow := Row;
        StartCol := Col;
        r        := StartRow;
        c        := StartCol;
        repeat
          inc(c);
          if c >= ColCount then
            begin
              inc(r);
              c := 0;
              if r >= RowCount then
                r := 1;
            end;
          if pos(FindS, AnsiUpperCase(Cells[c, r])) > 0 then
            begin
              found := True;
              Row   := r;
              Col   := c;
              r     := StartRow;
              c     := StartCol;
              SetFocus;
            end;
        until (r = StartRow) and (c = StartCol);
      end;
    if not found then
      case SysLocale.PriLangID of
        Lang_German: MsgBox(FindText.Text + ' nicht gefunden', Caption, mb_Ok);
      else
        MsgBox(FindText.Text + ' not found', Caption, mb_Ok);
      end;
  end;

procedure TDatesForm.FormClose(Sender: TObject; var CloseAction: TCloseAction);
  begin
    if CloseAction = caFree then;
    SaveFormToIni(Self);
    Load;
  end;

procedure TDatesForm.BtnCopyClick(Sender: TObject);
  var     s    : String;
          i, j : Integer;
  begin
    s := '';
    for i := Grid.Selection.Top to Grid.Selection.Bottom do
      begin
        for j := Grid.Selection.Left to Grid.Selection.Right do
          s := s + Grid.Cells[j, i] + A_HT;
        s := Copy(s, 1, length(s) - 1) + A_CR + A_LF;
      end;
    Clipboard.AsText := Copy(s, 1, length(s) - 2);
  end;

procedure TDatesForm.BtnPasteClick(Sender: TObject);
  var     s,   t   : String;
          Col, Row,
          c,   r   : Integer;
  begin
    if not Grid.EditorMode then
      begin
        s   := ClipBoard.asText;
        Col := Grid.Col;
        Row := Grid.Row;
        while s > '' do
          begin
            c := Col;
            r := Row;
            t := GetTable(s, Col, Row);
            if c < Grid.ColCount then
              Grid.Cells[c, r] := t
            else
              Grid.Cells[Grid.ColCount - 1, r] :=
                Grid.Cells[Grid.ColCount - 1, r] + ' ' + t;
            if Col = 0 then
              Col := Grid.Col;
            if Row >= Grid.RowCount then
              Grid.InsertRows(Grid.RowCount, Row - Grid.RowCount + 1);
          end;
      end;
  end;

procedure TDatesForm.FormCloseQuery(Sender: TObject;
                                    var CanClose: Boolean);
  var     i : Integer;
  begin
    if IsDirty then
      case SysLocale.PriLangID of
        Lang_German:
          i := MsgBox('Eingaben verwerfen?', Caption,
                      mb_OkCancel or mb_IconQuestion);
      else
        i := MsgBox('Cancel changes?', Caption,
                    mb_OkCancel or mb_IconQuestion);
      end
    else
      i := idOk;
    if i = idOk Then
      CanClose := True
    else
      CanClose := False;
  end;

procedure TDatesForm.GridKeyPress(Sender: TObject; var Key: Char);
  begin
    IsDirty := True;
  end;

procedure TDatesForm.GridMouseUp(Sender: TObject; Button: TMouseButton;
                                 Shift: TShiftState; X, Y: Integer);
  begin
    IsDirty := True;
  end;

end.
