unit holidays;

{$ifdef fpc}
  {$mode objfpc}{$H+}
{$endif}

interface

uses
  {$ifdef fpc}
    LCLType,
    LCLIntf,
    LResources,
    LConvEncoding,
  {$endif}
  {$ifdef Unix}
    Unix,
  {$else}
    Windows,
  {$endif}
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  Grids,
  IniFiles,
  Common,
  Zusatz,
  Buttons,
  ExtCtrls,
  Menus,
  DelTime,
  SortGrid,
  WolfUtil;

const
  ColWeekDay         = 'w';
  ColHoliday         = 'h';
  ColIstFeiertag     = 'f';
  ColListe           = 'l';

type
  tCol               = (ColTagNr,
                        ColMonatNr,
                        ColJahrNr,
                        ColWeekDayNr,
                        ColWocheNr,
                        ColHolidayNr,
                        ColIstFeiertagNr,
                        ColListeNr,
                        ColTextNr);

  { TUserHolidaysForm }

  TUserHolidaysForm  = class(TForm)
    Panel            : TPanel;
    BtnOk            : TBitBtn;
    BtnCancel        : TBitBtn;
    BtnInsert        : TBitBtn;
    BtnDelete        : TBitBtn;
    BtnSort          : TBitBtn;
    MainMenu         : TMainMenu;
    MenuDatei        : TMenuItem;
    MenuOk           : TMenuItem;
    MenuCancel       : TMenuItem;
    MenuBearbeiten   : TMenuItem;
    MenuInsert       : TMenuItem;
    MenuDelete       : TMenuItem;
    MenuSort         : TMenuItem;
    LabelJahr        : TLabel;
    LabelOstern      : TLabel;
    LabelWochentag   : TLabel;
    CheckBox         : TCheckBox;
    WeekDays         : TComboBox;
    Holidays         : TComboBox;
    LabelWoche       : TLabel;
    LabelHolidayList : TLabel;
    HolidayList      : TEdit;
    Grid             : tSortGrid;
    procedure           BtnDeleteClick(Sender: TObject);
    procedure           BtnInsertClick(Sender: TObject);
    procedure           FormResize(Sender: TObject);
    procedure           FormCreate(Sender: TObject);
    procedure           FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure           BtnOkClick(Sender: TObject);
    procedure           FormDestroy(Sender: TObject);
    procedure           BtnSortClick(Sender: TObject);
    procedure           CheckBoxKeyUp(Sender: TObject; var Key: Word;
                                      Shift: TShiftState);
    procedure           CheckBoxMouseUp(Sender: TObject; Button: TMouseButton;
                                        Shift: TShiftState; X, Y: Integer);
    procedure           GridTopLeftChanged(Sender: TObject);
    procedure           GridDrawCell(Sender: TObject; ACol, ARow: Integer;
                                     Rect: TRect; State: TGridDrawState);
    procedure           GridSelectCell(Sender: TObject; ACol, ARow: Integer;
                                       var CanSelect: Boolean);
    procedure           MenuCancelClick(Sender: TObject);
    procedure           ComboBoxEnter(Sender: TObject);
    procedure           CheckBoxEnter(Sender: TObject);
    procedure           GridCompareCells(Sender: TObject;
                                         ACol, ARow, BCol, BRow: Integer;
                                         var Result: Integer);
    procedure           BtnCancelClick(Sender: TObject);
    procedure           FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure           GridKeyPress(Sender: TObject; var Key: Char);
    procedure           GridMouseUp(Sender: TObject; Button: TMouseButton;
                                    Shift: TShiftState; X, Y: Integer);
    procedure           CheckBoxClick(Sender: TObject);
  private
    IsDirty           : Boolean;                   
    function            Check(Jahr : Integer) : Boolean;
    function            Sortieren : Boolean;
    procedure           FreeObjects;
    function            GridRowCount : LongInt;
    procedure           NewCheckBox(col, row : Integer; cbName : String);
    procedure           NewComboBox(col, row : Integer; cbName : String);
    procedure           NewObjects(row : LongInt);
    procedure           GetHolidayOnHoliday(GridDay, GridYear : Integer;
                                            GridHoliday       : tSpecialDay;
                                            var jDay          : tJulianDay;
                                            var ErrorCol      : Integer;
                                            var Error         : String);
    procedure           GetHolidayOnDate(GridDay, GridMonth,
                                         GridYear          : Integer;
                                         var jDay          : tJulianDay;
                                         var ErrorCol      : Integer;
                                         var Error         : String);
    procedure           GetHolidayOnWeek(GridMonth, GridYear,
                                         GridWeek,  GridWeekDay : Integer;
                                         var jDay               : tJulianDay;
                                         var ErrorCol           : Integer;
                                         var Error              : String);
    procedure           GetHoliday(i            : LongInt;
                                   Jahr         : Integer;
                                   var jDay     : tJulianDay;
                                   var hText    : String;
                                   var ErrorCol : Integer;
                                   var Error    : String);
    procedure           GridReset;
    procedure           RenameComboBox(Col, Row : Integer; cbName : String);
    procedure           RenameCheckBox(Col, Row : Integer; cbName : String);
    procedure           RenameObjects;
    function            GetComboIndex(Col, Row : LongInt) : Integer;
    procedure           SetComboIndex(Col, Row, Index : LongInt);
    function            GetChecked(Col, Row : LongInt) : Boolean;
    procedure           SetChecked(Col, Row : LongInt; c : Boolean);
    procedure           NewSize;
  public
    function            IsHoliday(JulianDay : tJulianDay;
                                  var sHolidays : String) : Boolean;
    function            IsHolidayList(Index     : Integer;
                                      Jahr      : Integer;
                                      Var JDay  : tJulianDay;
                                      var hName : String) : Boolean;
    procedure           Load;
  end;

var
  UserHolidaysForm : TUserHolidaysForm;

implementation

uses Main;

{$R *.dfm}

function TUserHolidaysForm.GetComboIndex(Col, Row : LongInt) : Integer;
  begin
    if Grid.Objects[Col, Row] is tComboBox then
      GetComboIndex := (Grid.Objects[Col, Row] as tComboBox).ItemIndex
    else
      GetComboIndex := 0;
  end;

procedure TUserHolidaysForm.SetComboIndex(Col, Row, Index : LongInt);
  begin
    if Grid.Objects[Col, Row] is tComboBox then
      (Grid.Objects[Col, Row] as tComboBox).ItemIndex := Index;
  end;

function TUserHolidaysForm.GetChecked(Col, Row : LongInt) : Boolean;
  begin
    if Grid.Objects[Col, Row] is tCheckBox then
      GetChecked := (Grid.Objects[Col, Row] as tCheckBox).Checked
    else
      GetChecked := False;
  end;

procedure TUserHolidaysForm.SetChecked(Col, Row : LongInt; c : Boolean);
  begin
    if Grid.Objects[Col, Row] is tCheckBox then
      (Grid.Objects[Col, Row] as tCheckBox).Checked := c;
  end;

procedure tUserHolidaysForm.RenameComboBox(Col, Row : Integer; cbName : String);
  begin
    if Grid.Objects[ord(Col), Row] is tComboBox then
      (Grid.Objects[ord(Col), Row] as tComboBox).Name := cbName + IntToStr(Row);
  end;

procedure tUserHolidaysForm.RenameCheckBox(Col, Row : Integer; cbName : String);
  begin
    if Grid.Objects[ord(Col), Row] is tCheckBox then
      (Grid.Objects[ord(Col), Row] as tCheckBox).Name := cbName + IntToStr(Row);
  end;

procedure TUserHolidaysForm.BtnDeleteClick(Sender: TObject);
  var i : Integer;
  begin
    if Grid.RowCount > 2 then
      begin
        for i := 0 to Grid.ColCount - 1 do
          if Grid.Objects[i, Grid.Row] <> Nil then
            begin
              Grid.Objects[i, Grid.Row].Free;
              Grid.Objects[i, Grid.Row] := Nil;
            end;
        Grid.DeleteRows(Grid.Row, Grid.Row);
      end;
    if Grid.RowCount <= 2 then
      for i := 0 to Grid.ColCount - 1 do
        begin
          SetComboIndex(i, Grid.RowCount - 1, 0);
          SetChecked(i, Grid.RowCount - 1, False);
          Grid.Cells[i, Grid.RowCount - 1] := '';
        end;
    Grid.SetFocus;
    RenameObjects;
    BtnDelete.enabled := GridRowCount > 1;
  end;

procedure TUserHolidaysForm.BtnInsertClick(Sender: TObject);
  begin
    Grid.InsertRows(Grid.Row, 1);
    NewObjects(Grid.Row);
    Grid.SetFocus;
    RenameObjects;
    BtnDelete.enabled := True;
  end;

function tUserHolidaysForm.GridRowCount : LongInt;
  var    c, i : LongInt;
         s    : String;
  begin
    c  := Grid.RowCount;
    if c <= 2 then
      begin
        s := '';
        for i := 0 to Grid.ColCount - 1 do
          begin
            s := s + Trim(Grid.Cells[i, Grid.RowCount - 1]);
            if GetComboIndex(i, Grid.RowCount - 1) > 0 then
              s := '!'
            else
              if GetChecked(i, Grid.RowCount - 1) then
                s := '!';
          end;
        if s = '' then
          c := 1;
      end;
    GridRowCount := c;
  end;

procedure TUserHolidaysForm.GetHolidayOnWeek(GridMonth, GridYear,
                                             GridWeek,  GridWeekDay : Integer;
                                             var jDay               : tJulianDay;
                                             var ErrorCol           : Integer;
                                             var Error              : String);
  var     TestYear  : Integer;
          TestMonth : Word;
          TestDay   : tJulianDay;
  begin
    if GridYear < 0 then
      inc(GridYear);
    if GridMonth > 0 then
      begin
        if (GridWeek < -5) or (GridWeek = 0) or (GridWeek > 5) then
          begin
            ErrorCol := ord(ColWocheNr);
            case SysLocale.PriLangID of
              Lang_German:
                Error := 'Bitte geben Sie eine Woche von +/-1 bis +/-5 ein!';
            else
              Error := 'Please enter a week from +/-1 to +/-5!';
            end;
          end;
        if GridWeek > 0 then
          begin
            jDay := LocalJulianDay(GridYear, GridMonth, 1.5);
            jDay := jDay + (GridWeekDay + 7 - LocalJulDayOfWeek(jDay)) mod 7
                         + (GridWeek - 1) * 7;
            TestMonth := 0;
            TestDay   := 0;
            LocalJulianDate(jDay, GridYear, TestMonth, TestDay);
            while TestMonth <> GridMonth do
              begin
                jDay     := jDay - 7;
                TestYear := 0;
                LocalJulianDate(jDay, TestYear, TestMonth, TestDay);
              end;
          end
        else
          begin
            jDay := LocalJulianDay(GridYear, GridMonth, 31.5);
            jDay := jDay + (GridWeekDay + 7 - LocalJulDayOfWeek(jDay)) mod 7;
            LocalJulianDate(jDay, GridYear, TestMonth, TestDay);
            while TestMonth <> GridMonth do
              begin
                jDay := jDay - 7;
                LocalJulianDate(jDay, TestYear, TestMonth, TestDay);
              end;
            jDay := jDay - (- GridWeek - 1) * 7;
            LocalJulianDate(jDay, GridYear, TestMonth, TestDay);
            while TestMonth <> GridMonth do
              begin
                jDay := jDay + 7;
                LocalJulianDate(jDay, TestYear, TestMonth, TestDay);
              end;
          end;
      end
    else
      if GridWeek >= 0 then
        jDay := Week0(GridYear) + 7 * GridWeek + (5 + GridWeekDay) mod 7
      else
        begin
          jDay := LocalJulianDay(GridYear, 12, 31.5);
          jDay := jDay + (GridWeekDay + 7 - LocalJulDayOfWeek(jDay)) mod 7;
          LocalJulianDate(jDay, GridYear, TestMonth, TestDay);
          while TestYear <> GridYear do
            begin
              jDay := jDay - 7;
              LocalJulianDate(jDay, TestYear, TestMonth, TestDay);
            end;
          jDay := jDay - (- GridWeek - 1) * 7;
          LocalJulianDate(jDay, GridYear, TestMonth, TestDay);
          while TestYear <> GridYear do
            begin
              jDay := jDay + 7;
              LocalJulianDate(jDay, TestYear, TestMonth, TestDay);
            end;
        end;
  end;

procedure TUserHolidaysForm.GetHolidayOnHoliday(GridDay, GridYear : Integer;
                                                GridHoliday       : tSpecialDay;
                                                var jDay          : tJulianDay;
                                                var ErrorCol      : Integer;
                                                var Error         : String);
  var     TestYear  : Integer;
          TestMonth : Word;
          TestDay   : tJulianDay;
          l         : tLanguageList;
  begin
    try
      l := tLanguageList.Create;
    except
      l := Nil;
    end;
    if l = Nil then
      MsgBox('cannot create language list', Caption, mb_IconHand)
    else
      try
        l.Add(SysLocale.PriLangID, SysLocale.SubLangID);
        GetSpecialDay(GridHoliday, GridYear, JDay, l);
        if JDay >= 0 then
          begin
            JDay      := JDay + GridDay;
            TestYear  := 0;
            TestMonth := 0;
            TestDay   := 0;
            LocalJulianDate(JDay, TestYear, TestMonth, TestDay);
            if TestYear <> GridYear then
              begin
                ErrorCol := ord(ColTagNr);
                case SysLocale.PriLangID of
                  Lang_German:
                    Error := CP1252ToSystem('Bitte geben Sie einen gltigen ')
                             + 'Abstand vom Feiertag ein!';
                else
                  Error := 'Please enter a valid shift by days from the holiday!';
                end;
              end;
          end;
      finally
        l.Free;
      end;
  end;

procedure TUserHolidaysForm.GetHolidayOnDate(GridDay,
                                             GridMonth,
                                             GridYear     : Integer;
                                             var jDay     : tJulianDay;
                                             var ErrorCol : Integer;
                                             var Error    : String);
  var     TestYear  : Integer;
          TestMonth : Word;
          TestDay   : tJulianDay;
  begin
    if (GridDay < 1) or (GridDay > 31) then
      begin
        ErrorCol := ord(ColTagNr);
        case SysLocale.PriLangID of
          Lang_German:
            Error := CP1252ToSystem('Bitte geben Sie einen gltigen Tag ein!');
        else
          Error := 'Please enter a valid day!';
        end;
      end;
    if (GridMonth < 1) or (GridMonth > 12) then
      begin
        ErrorCol := ord(ColMonatNr);
        case SysLocale.PriLangID of
          Lang_German:
            Error := 'Bitte geben Sie einen Monat ein!';
        else
          Error := 'Please enter a month!';
        end;
      end;
    if   (GridDay > 30) and (GridMonth in [4, 6, 9, 11])
      or (GridDay > 29) and (GridMonth = 2)
    then
      begin
        ErrorCol := ord(ColTagNr);
        case SysLocale.PriLangID of
          Lang_German:
            Error := CP1252ToSystem('Bitte geben Sie ein gltiges Datum ein!');
        else
          Error := 'Please enter a valid date!';
        end;
      end;
    if GridYear < 0 then
      inc(GridYear);
    jDay      := LocalJulianDay(GridYear, GridMonth, GridDay + 0.5);
    TestYear  := 0;
    TestMonth := 0;
    TestDay   := 0;
    LocalJulianDate(jDay, TestYear, TestMonth, TestDay);
    if   (GridDay   <> trunc(TestDay))
      or (GridMonth <> TestMonth)
      or (GridYear  <> TestYear)
    Then
      { 29.2. ohne Schaltjahr }
      jDay := -1;
  end;

procedure TUserHolidaysForm.GetHoliday(i            : LongInt;
                                       Jahr         : Integer;
                                       var jDay     : tJulianDay;
                                       var hText    : String;
                                       var ErrorCol : Integer;
                                       var Error    : String);
  var    e,
         GridDay,
         GridMonth,
         GridYear,
         GridWeek,
         GridWeekDay : Integer;
         s,
         GridHoliday : tSpecialDay;
  begin
    Grid.Cells[ord(ColTagNr),   i] := Trim(Grid.Cells[ord(ColTagNr),   i]);
    Grid.Cells[ord(ColMonatNr), i] := Trim(Grid.Cells[ord(ColMonatNr), i]);
    Grid.Cells[ord(ColJahrNr),  i] := Trim(Grid.Cells[ord(ColJahrNr),  i]);
    Grid.Cells[ord(ColWocheNr), i] := Trim(Grid.Cells[ord(ColWocheNr), i]);
    Grid.Cells[ord(ColTextNr),  i] := Trim(Grid.Cells[ord(ColTextNr),  i]);
    jDay        := -1;
    hText       := Grid.Cells[ord(ColTextNr), i];
    ErrorCol    :=  0;
    Error       := '';
    GridWeekDay := GetComboIndex(ord(ColWeekDayNr), i);
    GridHoliday := NormalDay;
    if Grid.Objects[ord(ColHolidayNr), i] is tComboBox then
      for s := Low(tSpecialDay) to High(tSpecialDay) do
        if (Grid.Objects[ord(ColHolidayNr), i] as tComboBox).Text
           = SpecialDayName[s]
        then
          GridHoliday := s;
    GridDay := 0;
    if Grid.Cells[ord(ColTagNr), i] > '' then
      begin
        val(Grid.Cells[ord(ColTagNr), i], GridDay, e);
        if e = 0 then
          Grid.Cells[ord(ColTagNr), i] := IntToStr(GridDay)
        else
          begin
            ErrorCol := ord(ColTagNr);
            case SysLocale.PriLangID of
              Lang_German:
                Error :=
                  CP1252ToSystem('Bitte geben Sie einen gltigen Tag ein!');
            else
              Error := 'Please enter a valid day!';
            end;
          end;
      end;
    GridMonth := 0;
    if Grid.Cells[ord(ColMonatNr), i] > '' then
      begin
        val(Grid.Cells[ord(ColMonatNr), i], GridMonth, e);
        if (e = 0) and (1 <= GridMonth) and (GridMonth <= 12) then
          Grid.Cells[ord(ColMonatNr), i] := IntToStr(GridMonth)
        else
          begin
            ErrorCol := ord(ColMonatNr);
            case SysLocale.PriLangID of
              Lang_German:
                Error :=
                  CP1252ToSystem('Bitte geben Sie einen gltigen Monat ein!');
            else
              Error := 'Please enter a valid month!';
            end;
          end;
      end;
    if Grid.Cells[ord(ColJahrNr), i] = '' then
      GridYear := Jahr
    else
      begin
        val(Grid.Cells[ord(ColJahrNr), i], GridYear, e);
        if (e = 0) and (GridYear >= MinYear) and (GridYear <> 0) then
          begin
            Grid.Cells[ord(ColJahrNr), i] := IntToStr(GridYear);
            if GridYear < 0 then
              inc(GridYear);
          end
        else
          begin
            Grid.Col := ord(ColJahrNr);
            case SysLocale.PriLangID of
              Lang_German:
                Error := 'Bitte geben Sie ein Jahr, nicht 0 und '
                         + CP1252ToSystem('nicht frher als ')
                         + IntToStr(MinYear) + ' ein!';
            else
              Error := 'Please enter a year, '
                     + 'not 0 and not before ' + IntToStr(MinYear) + '!';
            end;
          end;
      end;
    GridWeek := 0;
    if Grid.Cells[ord(ColWocheNr), i] > '' then
      begin
        val(Grid.Cells[ord(ColWocheNr), i], GridWeek, e);
        if (e = 0) and (-53 <= GridWeek) and (GridWeek <= 53) then
          Grid.Cells[ord(ColWocheNr), i] := IntToStr(GridWeek)
        else
          begin
            ErrorCol := ord(ColWocheNr);
            case SysLocale.PriLangID of
              Lang_German:
                Error :=
                  CP1252ToSystem('Bitte geben Sie eine gltige Woche ein!');
            else
              Error := 'Please enter a valid week!';
            end;
          end;
      end;

    if Error = '' then
      begin
        if GridWeekDay > 0 then
          begin
            if Grid.Cells[ord(ColWocheNr), i] = '' then
              begin
                ErrorCol := ord(ColWocheNr);
                case SysLocale.PriLangID of
                  Lang_German:
                    Error := 'Bitte geben Sie eine Woche ein!';
                else
                  Error := 'Please enter a week!';
                end;
              end;
            if Grid.Cells[ord(ColTagNr), i] > '' then
              begin
                ErrorCol := ord(ColTagNr);
                case SysLocale.PriLangID of
                  Lang_German:
                    Error := 'Wochentag kann nicht mit Tag kombiniert werden!';
                else
                  Error := 'Day of week cannot be combined with day!';
                end;
              end;
            if GridHoliday <> NormalDay then
              begin
                ErrorCol := ord(ColHolidayNr);
                case SysLocale.PriLangID of
                  Lang_German:
                    Error := 'Wochentag kann nicht mit Feiertag kombiniert werden!';
                else
                  Error := 'Day of week cannot be combined with holiday!';
                end;
              end;
            GetHolidayOnWeek(GridMonth, GridYear, GridWeek, GridWeekDay,
                             jDay, ErrorCol, Error);
          end
        else
          if GridHoliday <> NormalDay then
            begin
              if Grid.Cells[ord(ColTagNr), i] = '' then
                begin
                  ErrorCol := ord(ColTagNr);
                  case SysLocale.PriLangID of
                    Lang_German:
                      Error := 'Bitte geben Sie einen Abstand in Tagen ein!';
                  else
                    Error := 'Please enter a shift by days!';
                  end;
                end;
              if Grid.Cells[ord(ColWocheNr), i] > '' then
                begin
                  ErrorCol := ord(ColWocheNr);
                  case SysLocale.PriLangID of
                    Lang_German:
                      Error := 'Feiertag kann nicht mit Woche kombiniert werden!';
                  else
                    Error := 'Holiday cannot be combined with week!';
                  end;
                end;
              if GridWeekDay > 0 then
                begin
                  ErrorCol := ord(ColWeekDayNr);
                  case SysLocale.PriLangID of
                    Lang_German:
                      Error := 'Feiertag kann nicht mit '
                               + 'Wochentag kombiniert werden!';
                  else
                    Error := 'Holiday cannot be combined with day of week!';
                  end;
                end;
              if GridMonth > 0 then
                begin
                  ErrorCol := ord(ColMonatNr);
                  case SysLocale.PriLangID of
                    Lang_German:
                      Error := 'Feiertag kann nicht mit Monat kombiniert werden!';
                  else
                    Error := 'Holiday cannot be combined with month!';
                  end;
                end;
              GetHolidayOnHoliday(GridDay, GridYear, GridHoliday,
                                  jDay, ErrorCol, Error);
            end
          else
            if    (Grid.Cells[ord(ColTagNr),   i] = '')
              and (Grid.Cells[ord(ColMonatNr), i] = '')
            then
              begin
                ErrorCol := ord(ColTagNr);
                case SysLocale.PriLangID of
                  Lang_German:
                    Error := 'Bitte geben Sie Datum, Wochentag, oder '
                             + 'Bezug auf Feiertag ein!';
                else
                  Error := 'Please enter date, day of week or '
                           + 'reference to holiday!';
                end;
              end
            else
              begin
                if Grid.Cells[ord(ColWocheNr), i] > '' then
                  begin
                    ErrorCol := ord(ColWocheNr);
                    case SysLocale.PriLangID of
                      Lang_German:
                        Error := 'Datum kann nicht mit Woche kombiniert werden!';
                    else
                      Error := 'Date cannot be combined with week!';
                    end;
                  end;
                GetHolidayOnDate(GridDay, GridMonth, GridYear,
                                 jDay, ErrorCol, Error);
              end;
      end;
    if hText = '' then
      begin
        ErrorCol := ord(ColTextNr);
        case SysLocale.PriLangID of
          Lang_German:
            Error := 'Bitte geben Sie einen Text ein!';
        else
          Error := 'Please enter a text!';
        end;
      end;
  end;

function TUserHolidaysForm.IsHolidayList(Index     : Integer;
                                         Jahr      : Integer;
                                         Var JDay  : tJulianDay;
                                         var hName : String) : Boolean;
  var    ErrorCol : Integer;
         Error    : String;
  begin
    IsHolidayList := False;
    ErrorCol      := 0;
    Error         := '';
    GetHoliday(Index, Jahr, jDay, hName, ErrorCol, Error);
    if jDay >= 0 then
      IsHolidayList := GetChecked(ord(ColListeNr), Index);
  end;

function TUserHolidaysForm.Check(Jahr : Integer) : Boolean;
  var    i        : LongInt;
         ok       : Boolean;
         jDay     : tJulianDay;
         hName,
         s1, s2,
         Error    : String;
         ErrorCol : Integer;
         IniFile  : tIniFile;
  begin
    Ok := True;
    if not HolidayList.ReadOnly Then
      begin
        try
          IniFile := tIniFile.Create(IniName);
        except
          IniFile := Nil;
        end;
        if IniFile <> Nil then
          try
            if HolidayList.Text = '' Then
              begin
                Ok := False;
                case SysLocale.PriLangID of
                  Lang_German:
                    MsgBox('Bitte geben Sie eine Bezeichnung ein!',
                           Caption, mb_IconExclamation);
                else
                  MsgBox('Please enter a description!',
                         Caption, mb_IconExclamation);
                end;
              end
            else
              begin
                i := 0;
                Repeat
                  s1 := IntToStr(i);
                  s2 := IniFile.ReadString(IniHolidayList, s1, '');
                  if s2 = HolidayList.Text then
                    begin
                      Ok := False;
                      s2 := '';
                    end;
                  inc(i);
                until s2 = '';
                if not ok then
                  case SysLocale.PriLangID of
                    Lang_German:
                      MsgBox('Diese Bezeichnung ist bereits vergeben',
                             Caption, mb_IconExclamation);
                  else
                    MsgBox('This description is already in use',
                           Caption, mb_IconExclamation);
                  end
                else
                  begin
                    Ok := IsIni(HolidayList.Text);
                    if not ok then
                      MsgBox(HolidayList.Hint, Caption, mb_IconExclamation);
                  end;
              end;
          finally
            IniFile.Free;
          end;
      end;
    If not ok then
      HolidayList.SetFocus;
    i := 0;
    while Ok and (i + 1 < GridRowCount) do
      begin
        inc(i);
        jDay     := 0;
        hName    := '';
        ErrorCol := 0;
        Error    := '';
        GetHoliday(i, Jahr, jDay, hName, ErrorCol, Error);
        if Error > '' then
          begin
            if Grid.Objects[ErrorCol, i] is tComboBox then
              (Grid.Objects[ErrorCol, i] as tComboBox).SetFocus
            else
              begin
                Grid.Col := ErrorCol;
                Grid.Row := i;
                Grid.SetFocus;
              end;
            ok := False;
            MsgBox(Error, Caption, mb_IconHand);
          end;
      end;
    Check := Ok;
  end;

function TUserHolidaysForm.IsHoliday(JulianDay : tJulianDay;
                                     var sHolidays : String) : Boolean;
  var    i        : LongInt;
         jDay     : tJulianDay;
         Txt,
         Error    : String;
         j        : Integer;
         m        : Word;
         t        : double;
         ErrorCol : Integer;
  begin
    sHolidays := '';
    IsHoliday := False;
    j         := 0;
    m         := 0;
    t         := 0;
    LocalJulianDate(JulianDay, j, m, t);
    for i := 1 to GridRowCount - 1 do
      begin
        jDay     := 0;
        Txt      := '';
        ErrorCol := 0;
        Error    := '';
        GetHoliday(i, j, jDay, Txt, ErrorCol, Error);
        if jDay >= 0 then
          if LocalSameDate(jDay, JulianDay) then
            begin
              sHolidays := sHolidays + A_LF + Txt;
              if GetChecked(ord(ColIstFeiertagNr), i) then
                IsHoliday := True;
            end;
      end;
    sHolidays := Copy(sHolidays, 2, length(sHolidays));
  end;

procedure TUserHolidaysForm.FormResize(Sender: TObject);
  begin
    NewSize;
    SaveFormToIni(Self);
  end;

procedure TUserHolidaysForm.NewSize;
  begin
    Grid.ColWidths[ord(ColTagNr)] :=
      Canvas.TextWidth('-###')
      + Grid.GridLineWidth + 2;
    Grid.ExactWidthCell(ord(ColMonatNr), 0);
    Grid.ColWidths[ord(ColJahrNr)] :=
      Canvas.TextWidth('####')
      + Grid.GridLineWidth + 2;
    Grid.ColWidths[ord(ColWeekDayNr)] :=
      WeekDays.Width
      + Grid.GridLineWidth + 2;
    Grid.ExactWidthCell(ord(ColWocheNr), 0);
    Grid.ColWidths[ord(ColHolidayNr)] :=
      Holidays.Width
      + Grid.GridLineWidth + 2;
    Grid.ExactWidthCell(ord(ColIstFeiertagNr), 0);
    Grid.ExactWidthCell(ord(ColListeNr), 0);
    Grid.ExactWidthCell(ord(ColTextNr), 0);
    Grid.MinWidthCol(ord(ColTextNr));
    Grid.FillWidth(ord(ColTextNr));
  end;

procedure TUserHolidaysForm.FormCreate(Sender: TObject);
  var     h      : tSpecialDay;
          i, w,
          Breite : Integer;
          s      : String;
  begin
    IsDirty       := False;
    Grid.ColCount := ord(High(tCol)) + 1;
    Grid.AddSortColRow(ord(ColHolidayNr));
    Grid.AddSortColRow(ord(ColWeekDayNr));
    Grid.AddSortColRow(ord(ColWocheNr));
    Grid.AddSortColRow(ord(ColMonatNr));
    Grid.AddSortColRow(ord(ColTagNr));
    Grid.AddSortColRow(ord(ColJahrNr));
    Grid.AddSortColRow(ord(ColIstFeiertagNr));
    Grid.AddSortColRow(ord(ColListeNr));
    Grid.AddSortColRow(ord(ColTextNr));
    Grid.FormatType[ord(ColWocheNr)] := sgfInteger;
    Grid.FormatType[ord(ColMonatNr)] := sgfInteger;
    Grid.FormatType[ord(ColTagNr)]   := sgfInteger;
    Grid.FormatType[ord(ColJahrNr)]  := sgfInteger;
    Grid.FormatType[ord(ColTextNr)]  := sgfString;
    Grid.Alignment[ord(ColWocheNr)]  := taRightJustify;
    Grid.Alignment[ord(ColMonatNr)]  := taRightJustify;
    Grid.Alignment[ord(ColTagNr)]    := taRightJustify;
    Grid.Alignment[ord(ColJahrNr)]   := taRightJustify;
    case SysLocale.PriLangID of
      Lang_German:
        ;
    else
      begin
        LabelHolidayList.Caption := 'Holiday list name';
        LabelWochentag.Caption   := 'When day of week, then with week of year '
                                    + 'or with month and week';
        LabelWoche.Caption       := 'When week negative, '
                                    + 'then before end of month or year';
        LabelOstern.Caption      := 'When references holiday, '
                                    + 'then with shift by day';
        LabelJahr.Caption        := 'Empty year = every year';
        Caption                  := 'Individual holidays';
        BtnCancel.Hint           := 'Cancel';
        BtnInsert.Hint           := 'Insert';
        BtnDelete.Hint           := 'Delete';
        BtnSort.Hint             := 'Sort';
        MenuDatei.Caption        := '&File';
        MenuOk.Caption           := '&Ok';
        MenuCancel.Caption       := '&Cancel';
        MenuBearbeiten.Caption   := '&Edit';
        MenuInsert.Caption       := '&Insert';
        MenuDelete.Caption       := '&Delete';
        MenuSort.Caption         := '&Sort';
      end;
    end;
    GridReset;
    Breite := 0;
    for i := 1 to 7 do
      begin
        s := GetShortDayName(i);
        w := Canvas.TextWidth(s);
        if Breite < w then
          Breite := w;
        WeekDays.Items.Add(s);
      end;
    WeekDays.Width := WeekDays.Width + Breite;
    w := Canvas.TextWidth(Grid.Cells[ord(ColWeekDayNr), 0])
         + 4 + Grid.GridLineWidth;
    if WeekDays.Width < w then
      WeekDays.Width := w;
    Breite := 0;
    for h := succ(low(tSpecialDay)) to High(tSpecialDay) do
      begin
        s := SpecialDayName[h];
        w := Canvas.TextWidth(s);
        if Breite < w then
          Breite := w;
        Holidays.Items.Add(s);
      end;
    Holidays.Width := Holidays.Width + Breite;
    w := Canvas.TextWidth(Grid.Cells[ord(ColHolidayNr), 0])
         + 4 + Grid.GridLineWidth;
    if Holidays.Width < w then
      Holidays.Width := w;
    Grid.DefaultRowHeight := Holidays.Height;
    LoadFormFromIni(Self);
  end;

procedure TUserHolidaysForm.FormClose(Sender: TObject;
                                      var CloseAction: TCloseAction);
  begin
    if CloseAction = caFree then;
    SaveFormToIni(Self);
    Load;
  end;

procedure TUserHolidaysForm.BtnOkClick(Sender: TObject);
  var     i         : Longint;
          IniFile   : tIniFile;
          WeekDay,
          Holiday,
          sHoliday,
          Liste     : String;
          s         : tSpecialDay;
          error     : Boolean;
  begin
    if Sortieren then
      begin
        error := False;
        try
          IniFile := tIniFile.Create(IniName);
        except
          IniFile := Nil;
        end;
        if IniFile = Nil then
          error := True
        else
          begin
            try
              IniFile.EraseSection(Name + ' ' + HolidayList.Text);
              for i := 1 to GridRowCount - 1 do
                begin
                  Holiday  := SpecialDayId[NormalDay];
                  sHoliday := '';
                  Liste    := '';
                  WeekDay  := IntToStr(GetComboIndex(ord(ColWeekDayNr), i));
                  if Grid.Objects[ord(ColHolidayNr), i] is tComboBox then
                    begin
                      for s := Low(tSpecialDay) to High(tSpecialDay) do
                        if (Grid.Objects[ord(ColHolidayNr), i] as tComboBox).Text
                           = SpecialDayName[s]
                        then
                          Holiday := SpecialDayId[s];
                    end;
                  if GetChecked(ord(ColIstFeiertagNr), i) then
                    sHoliday := '*';
                  if GetChecked(ord(ColListeNr), i) then
                    Liste := '*';
                  IniFile.WriteString(Name + ' ' + HolidayList.Text,
                                      IntToDez(i, 3, False),
                                      Grid.Cells[ord(ColTagNr),        i] + '.'
                                      + Grid.Cells[ord(ColMonatNr),    i] + '.'
                                      + Grid.Cells[ord(ColJahrNr),     i] + '.'
                                      + WeekDay                           + '.'
                                      + Grid.Cells[ord(ColWocheNr),    i] + '.'
                                      + Holiday                           + '.'
                                      + sHoliday                          + '.'
                                      + Liste                             + '.'
                                      + Grid.Cells[ord(ColTextNr), i]);
                end;
            except
              error := True;
            end;
            IniFile.Free;
          end;
        if error then
          case SysLocale.PriLangID of
            Lang_German :
              MsgBox('kann individuelle Feiertage nicht speichern',
                     Caption, mb_IconHand);
          else
            MsgBox('cannot save individual holidays', Caption, mb_IconHand);
          end;
        IsDirty     := False;
        ModalResult := mrOk;
      end;
  end;

procedure TUserHolidaysForm.FormDestroy(Sender: TObject);
  begin
    FreeObjects;
  end;

procedure TUserHolidaysForm.FreeObjects;
  var     i, j : Integer;
  begin
    for j := 0 to Grid.ColCount - 1 do
      for i := 1 to Grid.RowCount - 1 do
        if Grid.Objects[j, i] <> Nil then
          begin
            Grid.Objects[j, i].Free;
            Grid.Objects[j, i] := Nil;
          end;
  end;

procedure tUserHolidaysForm.NewCheckBox(col, row : Integer; cbName : String);
  begin
    Grid.Objects[Col, Row] := tCheckBox.Create(Self);
    if Grid.Objects[Col, Row] is tCheckBox then
      begin
        SetChecked(Col, Row, False);
        RenameCheckBox(Col, Row, '__' + cbName);
        (Grid.Objects[Col, Row] as tCheckBox).Parent    := Grid;
        (Grid.Objects[Col, Row] as tCheckBox).Caption   := '';
        (Grid.Objects[Col, Row] as tCheckBox).TabStop   := True;
        {$ifdef fpc}
          (Grid.Objects[Col, Row] as tCheckBox).OnEnter   := @CheckBoxEnter;
          (Grid.Objects[Col, Row] as tCheckBox).OnClick   := @CheckBoxClick;
        {$else}
          (Grid.Objects[Col, Row] as tCheckBox).OnEnter   := CheckBoxEnter;
          (Grid.Objects[Col, Row] as tCheckBox).OnKeyUp   := CheckBoxKeyUp;
          (Grid.Objects[Col, Row] as tCheckBox).OnMouseUp := CheckBoxMouseUp;
        {$endif}
        (Grid.Objects[Col, Row] as tCheckBox).Visible := False;
      end;
  end;

procedure tUserHolidaysForm.NewComboBox(col, row : Integer; cbName : String);
  begin
    Grid.Objects[Col, Row] := tComboBox.Create(Self);
    if Grid.Objects[Col, Row] is tComboBox then
      begin
        RenameComboBox(Col, Row, '__' + cbName);
        (Grid.Objects[Col, Row] as tComboBox).Parent   := Grid;
        (Grid.Objects[Col, Row] as tComboBox).Style    := csDropDownList;
        (Grid.Objects[Col, Row] as tComboBox).ShowHint := False;
        (Grid.Objects[Col, Row] as tComboBox).TabStop  := True;
        {$ifdef fpc}
          (Grid.Objects[Col, Row] as tComboBox).OnEnter  := @ComboBoxEnter;
        {$else}
          (Grid.Objects[Col, Row] as tComboBox).OnEnter  := ComboBoxEnter;
        {$endif}
        (Grid.Objects[Col, Row] as tComboBox).Height   := 200;
        (Grid.Objects[Col, Row] as tComboBox).Visible  := False;
      end;
  end;

procedure TUserHolidaysForm.NewObjects(row: LongInt);
  begin
    NewComboBox(ord(ColWeekDayNr), row, ColWeekDay);
    if Grid.Objects[ord(ColWeekDayNr), Row] is tComboBox then
      begin
        (Grid.Objects[ord(ColWeekDayNr), Row] as tComboBox).Items :=
          WeekDays.Items;
        (Grid.Objects[ord(ColWeekDayNr), Row] as tComboBox).Width :=
          WeekDays.Width;
        SetComboIndex(ord(ColWeekDayNr), Row, 0);
      end;
    NewComboBox(ord(ColHolidayNr), row, ColHoliday);
    if Grid.Objects[ord(ColHolidayNr), Row] is tComboBox then
      begin
        (Grid.Objects[ord(ColHolidayNr), Row] as tComboBox).Items :=
            Holidays.Items;
        (Grid.Objects[ord(ColHolidayNr), Row] as tComboBox).Width :=
          Holidays.Width;
        SetComboIndex(ord(ColHolidayNr), Row, 0);
      end;
    NewCheckBox(ord(ColIstFeiertagNr), row, ColIstFeiertag);
    NewCheckBox(ord(ColListeNr),       row, ColListe);
  end;

procedure TUserHolidaysForm.RenameObjects;
  var     i, j : Integer;
  begin
    for i := 0 to Grid.RowCount - 1 do
      begin
        RenameComboBox(ord(ColWeekDayNr),     i, '_' + ColWeekDay);
        RenameComboBox(ord(ColHolidayNr),     i, '_' + ColHoliday);
        RenameCheckBox(ord(ColIstFeiertagNr), i, '_' + ColIstFeiertag);
        RenameCheckBox(ord(ColListeNr),       i, '_' + ColListe);
      end;
    for i := 0 to Grid.RowCount - 1 do
      begin
        RenameComboBox(ord(ColWeekDayNr),     i, ColWeekDay);
        RenameComboBox(ord(ColHolidayNr),     i, ColHoliday);
        RenameCheckBox(ord(ColIstFeiertagNr), i, ColIstFeiertag);
        RenameCheckBox(ord(ColListeNr),       i, ColListe);
      end;
    for i := 0 to Grid.RowCount - 1 do
      for j := 0 to Grid.ColCount - 1 do
        if Grid.Objects[j, i] is tCheckBox then
          (Grid.Objects[j, i] as tCheckBox).TabOrder :=
            i * Grid.ColCount + j + 6
        else
          if Grid.Objects[j, i] is tComboBox then
            (Grid.Objects[j, i] as tComboBox).TabOrder :=
              i * Grid.ColCount + j + 6;
  end;

function TUserHolidaysForm.Sortieren : Boolean;
  begin
    Sortieren := False;
    if Check(MainForm.Jahr.Position) then
      begin
        Sortieren        := True;
        BtnSort.enabled  := False;
        MenuSort.enabled := False;
        Grid.DoSort(1, Grid.RowCount - 1);
        BtnSort.enabled  := True;
        MenuSort.enabled := True;
        RenameObjects;
      end;
  end;

procedure TUserHolidaysForm.BtnSortClick(Sender: TObject);
  begin
    Sortieren;
    Grid.SetFocus;
  end;

procedure TUserHolidaysForm.CheckBoxKeyUp(Sender: TObject; var Key: Word;
                                          Shift: TShiftState);
  begin
    if Shift = [] then;
    if Key = ord(' ') then
      begin
        (Sender as tCheckBox).Checked := Not (Sender as tCheckBox).Checked;
        IsDirty := True;
      end;
  end;

procedure TUserHolidaysForm.CheckBoxMouseUp(Sender: TObject;
                                            Button: TMouseButton;
                                            Shift : TShiftState; X, Y: Integer);
  begin
    if Button = mbLeft then;
    if Shift  = []     then;
    if X      = 0      then;
    if Y      = 0      then;
    (Sender as tCheckBox).Checked := Not (Sender as tCheckBox).Checked;
    IsDirty := True;
  end;

procedure TUserHolidaysForm.GridTopLeftChanged(Sender: TObject);
  var     j, i : Integer;
  begin
    for j := 0 to Grid.ColCount - 1 do
      for i := 1 to Grid.RowCount - 1 do
        begin
          if Grid.Objects[j, i] is tComboBox then
            (Grid.Objects[j, i] as tComboBox).Visible := False;
          if Grid.Objects[j, i] is tCheckBox then
            (Grid.Objects[j, i] as tCheckBox).Visible := False;
        end;
  end;

procedure TUserHolidaysForm.GridDrawCell(Sender: TObject; ACol, ARow: Integer;
                                         Rect: TRect; State: TGridDrawState);
  begin
    if State = [] then;
    if Grid.Objects[aCol, aRow] is tCheckBox then
      begin
        (Grid.Objects[aCol, aRow] as tCheckBox).SetBounds(
          Rect.Left + (Rect.Right - Rect.Left - CheckBox.Width) div 2,
          Rect.Top,
          CheckBox.Width,
          Rect.Bottom - Rect.Top);
        (Grid.Objects[aCol, aRow] as tCheckBox).Visible := True;
      end
    else
      if Grid.Objects[aCol, ARow] is tComboBox then
        begin
          (Grid.Objects[aCol, ARow] as tComboBox).SetBounds(
            Rect.Left,
            Rect.Top,
            Rect.Right  - Rect.Left,
            Rect.Bottom - Rect.Top);
          (Grid.Objects[aCol, ARow] as tComboBox).Visible := True;
        end;
  end;

procedure TUserHolidaysForm.GridSelectCell(
            Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
  begin
    if ARow = 0 then;
    CanSelect :=      (aCol <> ord(ColWeekDayNr))
                  and (aCol <> ord(ColHolidayNr))
                  and (aCol <> ord(ColIstFeiertagNr))
                  and (aCol <> ord(ColListeNr));
  end;

procedure TUserHolidaysForm.MenuCancelClick(Sender: TObject);
  begin
    ModalResult := mrCancel;
  end;

procedure TUserHolidaysForm.GridReset;
  var     p : Integer;
  begin
    FreeObjects;
    for p := 0 to Grid.ColCount - 1 do
      Grid.Cols[p].Clear;
    Grid.RowCount := 1;
    case SysLocale.PriLangID of
      Lang_German:
        begin
          Grid.Cells[ord(ColTagNr),         0] := 'Tag';
          Grid.Cells[ord(ColMonatNr),       0] := 'Monat';
          Grid.Cells[ord(ColJahrNr),        0] := 'Jahr';
          Grid.Cells[ord(ColWeekDayNr),     0] := 'Wochentag';
          Grid.Cells[ord(ColWocheNr),       0] := 'Woche';
          Grid.Cells[ord(ColHolidayNr),     0] := 'Bezug auf Feiertag';
          Grid.Cells[ord(ColIstFeiertagNr), 0] := 'Hervorgehoben';
          Grid.Cells[ord(ColListeNr),       0] :=
            CP1252toSystem('bersicht Liste');
          Grid.Cells[ord(ColTextNr),        0] := 'Text';
        end;
    else
      begin
        Grid.Cells[ord(ColTagNr),         0] := 'Day';
        Grid.Cells[ord(ColMonatNr),       0] := 'Month';
        Grid.Cells[ord(ColJahrNr),        0] := 'Year';
        Grid.Cells[ord(ColWeekDayNr),     0] := 'Day of week';
        Grid.Cells[ord(ColWocheNr),       0] := 'Week';
        Grid.Cells[ord(ColHolidayNr),     0] := 'References holiday';
        Grid.Cells[ord(ColIstFeiertagNr), 0] := 'Marked';
        Grid.Cells[ord(ColListeNr),       0] := 'Outline list';
        Grid.Cells[ord(ColTextNr),        0] := 'Text';
      end;
    end;
  end;

procedure TUserHolidaysForm.Load;
  var     p, e,
          WeekDay   : Integer;
          s, t      : String;
          bHoliday,
          error,
          Liste     : Boolean;
          IniFile   : tIniFile;
          h,
          Holiday   : tSpecialDay;
  begin
    GridReset;
    if HolidayList.Text = '' Then
      begin
        HolidayList.Color    := clWindow;
        HolidayList.ReadOnly := False;
        HolidayList.Tabstop  := True;
        HolidayList.ShowHint := True;
        ActiveControl        := HolidayList;
      end
    else
      begin
        HolidayList.Color    := clBtnFace;
        HolidayList.ReadOnly := True;
        HolidayList.Tabstop  := False;
        HolidayList.ShowHint := False;
        ActiveControl        := Grid;
        error                := False;
        try
          IniFile := tIniFile.Create(IniName);
        except
          IniFile := Nil;
        end;
        if IniFile = Nil then
          error := True
        else
          begin
            try
              repeat
                s := IniFile.ReadString(Name + ' ' + HolidayList.Text,
                                        IntToDez(Grid.RowCount, 3, False), '');
                if s > '' then
                  begin
                    Grid.RowCount := Grid.RowCount + 1;
                    WeekDay       := 0;
                    Holiday       := NormalDay;
                    bHoliday      := False;
                    Liste         := False;
                    p             := Pos('.', s);
                    if p > 0 then
                      begin
                        Grid.Cells[ord(ColTagNr), Grid.RowCount - 1] :=
                          Copy(s, 1, p - 1);
                        s := Copy(s, p + 1, length(s));
                      end;
                    p := Pos('.', s);
                    if p > 0 then
                      begin
                        Grid.Cells[ord(ColMonatNr), Grid.RowCount - 1] :=
                          Copy(s, 1, p - 1);
                        s := Copy(s, p + 1, length(s));
                      end;
                    p := Pos('.', s);
                    if p > 0 then
                      begin
                        Grid.Cells[ord(ColJahrNr), Grid.RowCount - 1] :=
                          Copy(s, 1, p - 1);
                        s := Copy(s, p + 1, length(s));
                      end;
                    p := Pos('.', s);
                    if p > 0 then
                      begin
                        val(Copy(s, 1, p - 1), WeekDay, e);
                        s := Copy(s, p + 1, length(s));
                      end;
                    p := Pos('.', s);
                    if p > 0 then
                      begin
                        Grid.Cells[ord(ColWocheNr), Grid.RowCount - 1] :=
                          Copy(s, 1, p - 1);
                        s := Copy(s, p + 1, length(s));
                      end;
                    p := Pos('.', s);
                    if p > 0 then
                      begin
                        t := Copy(s, 1, p - 1);
                        for h := low(tSpecialDay) to High(tSpecialDay) do
                          if t = SpecialDayID[h] then
                            Holiday := h;
                        s := Copy(s, p + 1, length(s));
                      end;
                    p := Pos('.', s);
                    if p > 0 then
                      begin
                        if Copy(s, 1, p - 1) = '*' then
                          bHoliday := True;
                        s := Copy(s, p + 1, length(s));
                      end;
                    p := Pos('.', s);
                    if p > 0 then
                      begin
                        if Copy(s, 1, p - 1) = '*' then
                          Liste := True;
                        s := Copy(s, p + 1, length(s));
                      end;
                    Grid.Cells[ord(ColTextNr), Grid.RowCount - 1] := s;
                    Grid.MinWidthCell(ord(ColTextNr), Grid.RowCount - 1);
                    NewObjects(Grid.RowCount - 1);
                    SetComboIndex(ord(ColWeekDayNr), Grid.RowCount - 1, WeekDay);
                    if Grid.Objects[ord(ColHolidayNr), Grid.RowCount - 1]
                       is tComboBox
                    then
                      (Grid.Objects[ord(ColHolidayNr), Grid.RowCount - 1]
                        as tComboBox).ItemIndex :=
                      (Grid.Objects[ord(ColHolidayNr), Grid.RowCount - 1]
                        as tComboBox).Items.IndexOf(SpecialDayName[Holiday]);
                    SetChecked(ord(ColIstFeiertagNr), Grid.RowCount - 1, bHoliday);
                    SetChecked(ord(ColListeNr),       Grid.RowCount - 1, Liste);
                  end;
              until s = '';
            except
              error := True;
            end;
            IniFile.Free;
          end;
        if error then
          case SysLocale.PriLangID of
            Lang_German : MsgBox('kann individuelle Feiertage nicht laden',
                                 Caption, mb_IconHand);
          else
            MsgBox('cannot load individual holidays', Caption, mb_IconHand);
          end;
      end;
    if Grid.RowCount < 2 then
      begin
        Grid.RowCount := 2;
        NewObjects(1);
      end;
    Grid.Row          := 1;
    Grid.FixedRows    := 1;
    BtnDelete.enabled := GridRowCount > 1;
    IsDirty           := False;
    RenameObjects;
    NewSize;
  end;

procedure TUserHolidaysForm.CheckBoxEnter(Sender: TObject);
  var     i, e : Integer;
          s    : String;
  begin
    s := (Sender as tCheckBox).Name;
    val(Copy(s, 2, length(s)), i, e);
    Grid.Row := i;
    if (Sender as tCheckBox).Top + (Sender as tCheckBox).Height
      > Grid.ClientHeight
    then
      Grid.TopRow := i;
  end;

procedure TUserHolidaysForm.ComboBoxEnter(Sender: TObject);
  var     i, e : Integer;
          s    : String;
  begin
    s := (Sender as tComboBox).Name;
    val(Copy(s, 2, length(s)), i, e);
    Grid.Row := i;
    if (Sender as tComboBox).Top + (Sender as tComboBox).Height
       > Grid.ClientHeight
    then
      Grid.TopRow := i;
    IsDirty := True;
  end;

procedure TUserHolidaysForm.GridCompareCells(Sender: TObject;
                                             ACol, ARow, BCol, BRow: Integer;
                                             var Result: Integer);
  begin
    if ACol <> BCol then
      Result := Grid.DefaultCompareCells(ACol, ARow, BCol, BRow)
    else
      case ACol of
        ord(ColHolidayNr):
          Result := GetComboIndex(ACol, ARow) - GetComboIndex(BCol, BRow);
        ord(ColWeekDayNr):
          Result := GetComboIndex(ACol, ARow) - GetComboIndex(BCol, BRow);
        ord(ColIstFeiertagNr):
          Result := ord(GetChecked(ACol, ARow)) - ord(GetChecked(BCol, BRow));
        ord(ColListeNr):
          Result := ord(GetChecked(ACol, ARow)) - ord(GetChecked(BCol, BRow));
      else
        Result := Grid.DefaultCompareCells(ACol, ARow, BCol, BRow);
      end;
  end;

procedure TUserHolidaysForm.BtnCancelClick(Sender: TObject);
  begin
    IsDirty     := False;
    ModalResult := mrCancel;
  end;

procedure TUserHolidaysForm.FormCloseQuery(Sender: TObject;
                                           var CanClose: Boolean);
  var     i : Integer;
  begin
    if IsDirty then
      case SysLocale.PriLangID of
        Lang_German:
          i := MsgBox('Eingaben verwerfen?', Caption,
                      mb_OkCancel or mb_IconQuestion);
      else
        i := MsgBox('Cancel changes?', Caption,
                    mb_OkCancel or mb_IconQuestion);
      end
    else
      i := idOk;
    if i = idOk Then
      CanClose := True
    else
      CanClose := False;
  end;

procedure TUserHolidaysForm.GridKeyPress(Sender: TObject; var Key: Char);
  begin
    IsDirty := True;
  end;

procedure TUserHolidaysForm.GridMouseUp(Sender: TObject;
            Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  begin
    IsDirty := True;
  end;

procedure TUserHolidaysForm.CheckBoxClick(Sender: TObject);
  begin
    IsDirty := True;
  end;

end.

