unit link;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFnDEF FPC}
  {$ifdef Ver330}
    Vcl.ExtCtrls,
    Vcl.Dialogs,
  {$else}
    ExtCtrls,
    Dialogs,
  {$endif}
{$ELSE}
  LCLIntf,
  LCLType,
  LMessages,
  FileCtrl,
  ExtCtrls,
  Dialogs,
{$ENDIF}
  Windows,
  SysUtils,
  Classes,
  Controls,
  Forms,
  StdCtrls,
  Registry,
  Zusatz,
  WolfUtil,
  Common;

type
  TLinkForm                = class(TForm)
    CheckBoxDesktop        : TCheckBox;
    BtnSave                : TButton;
    Benutzer               : TRadioGroup;
    CheckBoxStartMenue     : TCheckBox;
    SaveDialog             : TSaveDialog;
    procedure                FormCreate(Sender: TObject);
    procedure                FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure                BenutzerChange(Sender: TObject);
    procedure                BtnSaveClick(Sender: TObject);
    procedure                FormShow(Sender: TObject);
  private
    OldStartMenue,
    OldStartMenueDir,
    NewStartmenue,
    NewStartMenueDir,
    OldDesktop,
    OldDesktopDir,
    NewDesktop,
    NewDesktopDir          : String;
    Start                  : Boolean;
    function                 MakeLink(Pfad, Arbeitsordner: String) : Boolean;
    procedure                Search(StartDir : String;
                                    var OldLnk, OldWorkDir: String;
                                    CheckBox : tCheckBox);
  end;

var
  LinkForm : TLinkForm;

implementation

{$R *.dfm}

function TLinkForm.MakeLink(Pfad, Arbeitsordner: String) : Boolean;
  var    s : String;
  begin
    MakeLink := True;
    s        := LinkName;
    s        := MkLink(Application.ExeName, s, Arbeitsordner, Pfad);
    if s > '' then
      begin
        MsgBox(s, Caption, mb_IconHand);
        MakeLink := False;
      end;
  end;

procedure TLinkForm.FormCreate(Sender: TObject);
  begin
    Start := True;
    case SysLocale.PriLangID of
      LANG_GERMAN: ;
    else
      Caption                       := 'shortcuts';
      Benutzer.Caption              := 'user';
      Benutzer.Items[0]             := 'al&l';
      Benutzer.Items[1]             := '&current';
      CheckBoxStartmenue.Caption    := 'in the selected folder of the &start menu';
      CheckBoxDesktop.Caption       := 'on the &desktop';
      BtnSave.Caption               := '&edit shortcut';
    end;
    LoadFormFromIni(Self);
    BenutzerChange(Sender);
    if not CheckBoxStartMenue.Checked  and not CheckBoxDesktop.Checked then
      ShowModal;
  end;

procedure TLinkForm.FormClose(Sender: TObject; var CloseAction: TCloseAction);
  begin
    SaveFormToIni(Self);
  end;

procedure TLinkForm.Search(StartDir : String; var OldLnk, OldWorkDir: String;
                           CheckBox : tCheckBox);
  var     Exe, Beschreibung,
          Lnk, WorkDir       : String;
          SearchRec          : tSearchRec;
  begin
    Lnk     := LinkName + LinkExt;
    WorkDir := OldWorkDir;
    if FindFirst(MakePath(StartDir, '*'), faAnyFile, SearchRec) = 0 then
      repeat
        if Copy(SearchRec.Name, 1, 1) <> '.' then
          if ((SearchRec.Attr and faDirectory) <> 0) then
            Search(MakePath(StartDir, SearchRec.Name), OldLnk, OldWorkDir,
                            CheckBox)
          else
            begin
              if GetLink(MakePath(StartDir, lnk),
                      Exe, Beschreibung, WorkDir) = '' then
                begin
                  OldLnk           := StartDir;
                  OldWorkDir       := WorkDir;
                  CheckBox.Checked := True;
                end;
            end;
      until FindNext(SearchRec) <> 0;
      FindClose(SearchRec);
  end;

procedure TLinkForm.BenutzerChange(Sender: TObject);
  var     Registry : tRegistry;
          s        : String;
  begin
    CheckBoxStartMenue.Checked := False;
    OldStartMenue              := '';
    OldStartMenueDir           := ExtractFilePath(Application.ExeName);
    CheckBoxDesktop.Checked    := False;
    OldDesktop                 := '';
    OldDesktopDir              := '';
    try
      Registry := tRegistry.Create;
    except
      Registry := Nil;
    end;
    if Registry <> Nil then
      begin
        s := LinkName + LinkExt;
        if Benutzer.ItemIndex = 0 then
          Registry.RootKey := HKEY_Local_Machine
        else
          Registry.RootKey := HKEY_Current_User;
        if Registry.openkeyReadOnly(ShellFolders) then
          begin
            try
              if Benutzer.ItemIndex = 0 then
                NewStartMenue := Registry.ReadString(CommonProgramsFolder)
              else
                NewStartMenue := Registry.ReadString(ProgramsFolder);
              if NewStartMenue > '' then
                Search(NewStartMenue, OldStartMenue, OldStartMenueDir,
                       CheckBoxStartMenue);
            except
            end;
            try
              if Benutzer.ItemIndex = 0 then
                begin
                  NewDesktop    := Registry.ReadString(CommonDesktopFolder);
                  NewDesktopDir := Registry.ReadString(CommonDocumentsFolder);
                end
              else
                begin
                  NewDesktop    := Registry.ReadString(DesktopFolder);
                  NewDesktopDir := Registry.ReadString(PersonalFolder);
                end;
              if NewDesktop > '' then
                Search(NewDesktop, OldDesktop, OldDesktopDir, CheckBoxDesktop);
            except
            end;
            Registry.CloseKey;
          end;
        if OldStartMenue = '' then
          OldStartMenue := NewStartMenue;
        if OldDesktop = '' then
          OldDesktop := NewDesktop;
        if OldDesktopDir = '' then
          OldDesktopDir := NewDesktopDir;
        Registry.Free;
      end;
  end;

procedure TLinkForm.BtnSaveClick(Sender: TObject);
  var     s, t, u, v,
          UserDir,
          AllDir     : String;
          Registry   : tRegistry;
  begin
    if Benutzer.ItemIndex = 0 then
      case SysLocale.PriLangID of
        LANG_GERMAN : SaveDialog.Title := 'fr alle';
        else
          SaveDialog.Title := 'for all users';
      end
    else
      case SysLocale.PriLangID of
        LANG_GERMAN : SaveDialog.Title := 'fr aktuellen Benutzer';
        else
          SaveDialog.Title := 'for current user';
      end;
    try
      Registry := tRegistry.Create;
    except
      Registry := Nil;
    end;
    if Registry <> Nil then
      begin
        if CheckBoxStartmenue.Checked then
          begin
            s := LinkName;
            t := MakePath(OldStartMenue, s);
            SaveDialog.InitialDir := OldStartMenue;
            SaveDialog.FileName   := t;
            if SaveDialog.Execute() then
              begin
                if Benutzer.ItemIndex = 0 then
                  Registry.RootKey := HKEY_Local_Machine
                else
                  Registry.RootKey := HKEY_Current_User;
                if Registry.openkeyreadonly(ShellFolders) then
                  begin
                    try
                      NewStartMenue := ExtractFilePath(SaveDialog.FileName);
                      u             := MakePath(NewStartMenue, s);
                      v := '';
                      if Benutzer.ItemIndex = 0 then
                        v := Registry.ReadString(CommonStartMenuFolder)
                      else
                        v := Registry.ReadString(StartMenuFolder);
                      if Copy(u, 1, length(v)) <> v then
                        begin
                          case SysLocale.PriLangID of
                            Lang_German : MsgBox(
                              'Unzulssiger Ordner fr das Startmenue',
                              Caption, mb_IconHand);
                          else
                            MsgBox('Invalid folder for start menu',
                                   Caption, mb_IconHand);
                          end;
                          NewStartMenue := '';
                          NewDesktop    := '';
                          OldStartMenue := '';
                          OldDesktop    := '';
                        end
                    except
                    end;
                    Registry.CloseKey;
                  end;
                if OldStartMenue > '' then
                  begin
                    s := GetMyName + IniExt;
                    t := MakePath(OldStartMenueDir, s);
                    SaveDialog.InitialDir := OldStartMenueDir;
                    SaveDialog.FileName   := t;
                    UserDir               := '';
                    AllDir                := '';
                    if SaveDialog.Execute() then
                      begin
                        NewStartMenueDir := ExtractFilePath(SaveDialog.FileName);
                        u                := MakePath(NewStartMenueDir, s);
                        Registry.RootKey := HKEY_Local_Machine;
                        if Registry.openkeyreadonly(ShellFolders) then
                          begin
                            try
                              AllDir  := Registry.ReadString(CommonDocumentsFolder)
                            except
                            end;
                            Registry.CloseKey;
                          end;
                        if Benutzer.ItemIndex > 0 then
                          begin
                            Registry.RootKey := HKEY_Current_User;
                            if Registry.openkeyreadonly(ShellFolders) then
                              begin
                                try
                                  UserDir := Registry.ReadString(PersonalFolder);
                                except
                                end;
                                Registry.CloseKey;
                              end;
                          end;
                        if UserDir = '' then
                          UserDir := AllDir;
                        if AllDir > '' then
                          if     (Copy(u, 1, length(UserDir)) <> UserDir)
                             and (Copy(u, 1, length(AllDir))  <> AllDir)
                          then
                            begin
                              NewStartmenueDir := '';
                              case SysLocale.PriLangID of
                                Lang_German : MsgBox(
                                  'Unzulssiger Arbeitsordner fr das Startmenue',
                                  Caption, mb_IconHand);
                              else
                                MsgBox('Invalid working folder for start menu',
                                       Caption, mb_IconHand);
                              end;
                            end;
                      end
                    else
                      begin
                        NewStartMenue := '';
                        NewDesktop    := '';
                        OldStartMenue := '';
                        OldDesktop    := '';
                      end;
                  end;
              end
            else
              begin
                NewStartMenue := '';
                NewDesktop    := '';
                OldStartMenue := '';
                OldDesktop    := '';
              end;
            if CheckBoxDesktop.Checked and (OldDesktop > '') then
              begin
                s := LinkName;
                t := MakePath(OldDesktop, s);
                SaveDialog.InitialDir := OldDesktop;
                SaveDialog.FileName   := t;
                if SaveDialog.Execute() then
                  begin
                    NewDesktop := ExtractFilePath(SaveDialog.FileName);
                    u          := MakePath(NewDesktop, s);
                    v := '';
                    try
                      if Benutzer.ItemIndex = 0 then
                        v := Registry.ReadString(CommonDesktopFolder)
                      else
                        v := Registry.ReadString(DesktopFolder);
                      if Copy(u, 1, length(v)) <> v then
                        begin
                          case SysLocale.PriLangID of
                            Lang_German : MsgBox('Unzulssiger Ordner fr den Desktop',
                                                 Caption, mb_IconHand);
                          else
                            MsgBox('Invalid folder for desktop', Caption, mb_IconHand);
                          end;
                          NewStartMenue := '';
                          NewDesktop    := '';
                          OldStartMenue := '';
                          OldDesktop    := '';
                        end
                    except
                    end;
                  end
                else
                  begin
                    NewStartMenue := '';
                    NewDesktop    := '';
                    OldStartMenue := '';
                    OldDesktop    := '';
                  end;
                if OldDesktop > '' then
                  begin
                    s := GetMyName + IniExt;
                    t := MakePath(OldDesktopDir, s);
                    SaveDialog.InitialDir := OldDesktopDir;
                    SaveDialog.FileName   := t;
                    UserDir               := '';
                    AllDir                := '';
                    if SaveDialog.Execute() then
                      begin
                        NewDesktopDir := ExtractFilePath(SaveDialog.FileName);
                        u             := MakePath(NewDesktopDir, s);
                        Registry.RootKey := HKEY_Local_Machine;
                        if Registry.openkeyreadonly(ShellFolders) then
                          begin
                            try
                              AllDir  := Registry.ReadString(CommonDocumentsFolder)
                            except
                            end;
                            Registry.CloseKey;
                          end;
                        if Benutzer.ItemIndex > 0 then
                          begin
                            Registry.RootKey := HKEY_Current_User;
                            if Registry.openkeyreadonly(ShellFolders) then
                              begin
                                try
                                  UserDir := Registry.ReadString(PersonalFolder);
                                except
                                end;
                                Registry.CloseKey;
                              end;
                          end;
                        if UserDir = '' then
                          UserDir := AllDir;
                        if AllDir > '' then
                          if     (Copy(u, 1, length(UserDir)) <> UserDir)
                             and (Copy(u, 1, length(AllDir))  <> AllDir)
                          then
                            begin
                              NewDesktopDir := '';
                              case SysLocale.PriLangID of
                                Lang_German : MsgBox(
                                  'Unzulssiger Arbeitsordner fr den Desktop',
                                  Caption, mb_IconHand);
                              else
                                MsgBox('Invalid working folder for desktop',
                                       Caption, mb_IconHand);
                                NewStartMenue := '';
                                NewDesktop    := '';
                                OldStartMenue := '';
                                OldDesktop    := '';
                              end;
                            end;
                      end
                    else
                      begin
                        NewStartMenue := '';
                        NewDesktop    := '';
                        OldStartMenue := '';
                        OldDesktop    := '';
                      end;
                  end;
              end;
            Registry.CloseKey;
          end;
        Registry.Free;
      end;
    if CheckBoxStartMenue.Checked and (NewStartMenue > '')
        and (NewStartMenueDir > '')
    then
      begin
        s := GetMyName + IniExt;
        t := MakePath(OldStartMenueDir, s);
        u := MakePath(NewStartMenueDir, s);
        if (OldStartMenueDir > '') and not FileExists(u) then
          {$ifdef Ver330}
            if not MoveFile(PWideChar(t), PWideChar(u)) then
              if CopyFile(PWideChar(u), PWideChar(t), True) then
                DeleteFile(t);
          {$else}
            if not MoveFile(PChar(t), PChar(u)) then
              if CopyFile(PChar(u), PChar(t), True) then
                DeleteFile(t);
          {$endif}
        s := Linkname + LinkExt;
        t := MakePath(OldStartMenue, s);
        u := MakePath(NewStartMenue, s);
        if (OldStartMenue > '') and not FileExists(u) then
          {$ifdef Ver330}
            if not MoveFile(PWideChar(t), PWideChar(u)) then
              if CopyFile(PWideChar(u), PWideChar(t), True) then
                DeleteFile(t);
          {$else}
            if not MoveFile(PChar(t), PChar(u)) then
              if CopyFile(PChar(u), PChar(t), True) then
                DeleteFile(t);
          {$endif}
        MakeLink(u, NewStartMenueDir);
      end
    else
      begin
        if OldStartMenue > '' then
          begin
            s := Linkname + LinkExt;
            t := MakePath(OldStartMenue, s);
            DeleteFile(t);
          end;
      end;
    if CheckBoxDesktop.Checked and (NewDesktop > '')
        and (NewDesktopDir > '')
    then
      begin
        s := GetMyName + IniExt;
        t := MakePath(OldDesktopDir, s);
        u := MakePath(NewDesktopDir, s);
        if (OldDesktopDir > '') and not FileExists(u) then
          {$ifdef Ver330}
            if not MoveFile(PWideChar(t), PWideChar(u)) then
              if CopyFile(PWideChar(u), PWideChar(t), True) then
                DeleteFile(t);
          {$else}
            if not MoveFile(PChar(t), PChar(u)) then
              if CopyFile(PChar(u), PChar(t), True) then
                DeleteFile(t);
          {$endif}
        s := Linkname + LinkExt;
        t := MakePath(OldDesktop, s);
        u := MakePath(NewDesktop, s);
        if (OldDesktop > '') and not FileExists(u) then
          {$ifdef Ver330}
            if not MoveFile(PWideChar(t), PWideChar(u)) then
              if CopyFile(PWideChar(u), PWideChar(t), True) then
                DeleteFile(t);
          {$else}
            if not MoveFile(PChar(t), PChar(u)) then
              if CopyFile(PChar(u), PChar(t), True) then
                DeleteFile(t);
          {$endif}
        MakeLink(u, NewDesktopDir);
      end
    else
      begin
        if OldDesktop > '' then
          begin
            s := Linkname + LinkExt;
            t := MakePath(OldDesktop, s);
            DeleteFile(t);
          end;
      end;
  end;

procedure TLinkForm.FormShow(Sender: TObject);
  begin
    if Start then
      begin
        Start := False;
        CheckBoxStartMenue.SetFocus;
      end;
  end;

end.
