unit extjpeg;

{ http://www.wotsit.org/

  Exif.zip               10.991 Bytes
  "Exif file format.htm" 45.853 Bytes May 6th 2002

  jfif.zip                4.654 Bytes
  Jfif.txt               15.311 Bytes September 9th 1992

  http://www.stefanheymann.de/homegallery/exif-felder.htm
}

{$ifdef fpc}
  {$mode objfpc}{$H+}
{$endif}

interface

uses
  SysUtils,
  WolfUtil;

const
  JPEGMarker                         = $FF;

  JPEGTEM                            = $01; { usually causes a decoding error,
                                              may be ignored }

  JPEGSOF0                           = $c0; { Start Of Frame (baseline JPEG) }
  JPEGSOF1                           = $c1; { dito }
  JPEGSOF2                           = $c2; { usually unsupported  }
  JPEGSOF3                           = $c3; { usually unsupported  }

  JPEGSOF5                           = $c5; { usually unsupported  }
  JPEGSOF6                           = $c6; { usually unsupported  }
  JPEGSOF7                           = $c7; { usually unsupported  }

  JPEGSOF9                           = $c9; { for arithmetic coding,
                                              usually unsupported }
  JPEGSOF10                          = $ca; { usually unsupported  }
  JPEGSOF11                          = $cb; { usually unsupported  }

  JPEGSOF13                          = $cd; { usually unsupported  }
  JPEGSOF14                          = $ce; { usually unsupported  }
  JPEGSOF15                          = $cf; { usually unsupported  }

  JPEGDHT                            = $c4; { Define Huffman Table }
  JPEGJPG                            = $c8; { undefined/reserved
                                              (causes decoding error) }
  JPEGDAC                            = $cc; { Define Arithmetic Table,
                                              usually unsupported }

  JPEGRST0                           = $d0; { RSTn are used for resync,
                                              may be ignored }
  JPEGRST1                           = $d1;
  JPEGRST2                           = $d2;
  JPEGRST3                           = $d3;
  JPEGRST4                           = $d4;
  JPEGRST5                           = $d5;
  JPEGRST6                           = $d6;
  JPEGRST7                           = $d7;

  JPEGSOI                            = $d8; { Start Of Image }
  JPEGEOI                            = $d9; { End Of Image   }
  JPEGSOS                            = $da; { Start Of Scan  }
  JPEGDQT                            = $db; { Define Quantization Table   }
  JPEGDNL                            = $dc; { usually unsupported, ignore }
  JPEGDRI                            = $dd; { Define Restart Interval }
  JPEGDHP                            = $de; { ignore (skip) }
  JPEGEXP                            = $df; { ignore (skip) }

  JPEGAPP0                           = $e0; { JFIF APP0 segment marker }
  JPEGAPP1                           = $e1; { Exif APP1 segment marker }
  JPEGAPP2                           = $e2; { Exif APP2 segment marker }
  JPEGAPP15                          = $ef; { ignore }

  JPEGJPG0                           = $f0; { ignore (skip) }
  JPEGJPG13                          = $fd; { ignore (skip) }
  JPEGCOM                            = $fe; { Comment }

  JpegTypeJFIF                       = 'JFIF';
  JpegTypeExif                       = 'Exif';

  JFIFUnitsNone                      =   0; { no units, x/y-density specify
                                              the aspect ratio instead  }
  JFIFUnitsDPI                       =   1; { x/y-density are dots/inch }
  JFIFUnitsDPCM                      =   2; { x/y-density are dots/cm   }

  TIFFAlignIntel                     = 'II';
  TIFFAlignMotorola                  = 'MM';
  TIFFTagMark                        = $2A;
  TIFFIFD0Offset                     =   8;

  IFDFormatUByte                     =   1;
  IFDFormatASCII                     =   2;
  IFDFormatUShort                    =   3;
  IFDFormatUlong                     =   4;
  IFDFormatURational                 =   5;
  IFDFormatSByte                     =   6;
  IFDFormatUndefined                 =   7;
  IFDFormatSShort                    =   8;
  IFDFormatSLong                     =   9;
  IFDFormatSRational                 =  10;
  IFDFormatSFloat                    =  11;
  IFDFormatDFloat                    =  12;

  IFDGpsDestLongitudeRef             = $0015;
  IFDGpsDestLongitude                = $0016;
  IFDGpsDestBearingkRef              = $0017;
  IFDGpsDestBearing                  = $0018;
  IFDGpsDestDistanceRef              = $0019;
  IFDGpsDestDistance                 = $001A;
  IFDGpsProcessingMode               = $001B;
  IFDGpsAreaInformation              = $001C;
  IFDGpsDateStamp                    = $001D;
  IFDGpsDifferential                 = $001E;
  IFDNikon_ImageAdjustment1          = $0080;
  IFDNikon_ImageAdjustment2          = $0081;
  IFDNikon_Adapter                   = $0082;
  IFDNikon_LensInformation           = $0084;
  IFDNikon_ManualFocusDistance       = $0085;
  IFDNikon_DigitalZoom               = $0086;
  IFDNikon_FocusArea                 = $0088;
  IFDNikon_Mode                      = $0089;
  IFDNikon_ColorMode                 = $008D;
  IFDNikon_SceneMode                 = $008F;
  IFDNikon_HueAdjustment             = $0092;
  IFDNikon_Saturation                = $0094;
  IFDNikon_NoiseReduction            = $0095;
  IFDNewSubfileType                  = $00FE;
  IFDSubfileType                     = $00FF;
  IFDImageWidth                      = $0100;
  IFDImageLength                     = $0101;
  IFDBitsPerSample                   = $0102;
  IFDCompression                     = $0103;
  IFDPhotometricInterpretation       = $0106;
  IFDFillOrder                       = $010A;
  IFDDocumentName                    = $010D;
  IFDImageDescription                = $010E;
  IFDMake                            = $010F;
  IFDModel                           = $0110;
  IFDStripOffsets                    = $0111;
  IFDOrientation                     = $0112;
  IFDSamplesPerPixel                 = $0115;
  IFDRowsPerStrip                    = $0116;
  IFDStripByteConunts                = $0117;
  IFDMinSampleValue                  = $0118;
  IFDMaxSampleValue                  = $0119;
  IFDXResolution                     = $011A;
  IFDYResolution                     = $011B;
  IFDPlanarConfiguration             = $011C;
  IFDPageName                        = $011D;
  IFDXPosition                       = $011E;
  IFDYPosition                       = $011F;
  IFDFreeOffsets                     = $0120;
  IFDFreeByteCounts                  = $0121;
  IFDGrayReponseUnit                 = $0122;
  IFDGrayReponseCurve                = $0123;
  IFDT4Options                       = $0124;
  IFDT6Options                       = $0125;
  IFDResolutionUnit                  = $0128;
  IFDPageNumber                      = $0129;
  IFDTransferFunction                = $012D;
  IFDSoftware                        = $0131;
  IFDDateTime                        = $0132;
  IFDArtist                          = $013B;
  IFDHostComputer                    = $013C;
  IFDPredictor                       = $013D;
  IFDWhitePoint                      = $013E;
  IFDPrimaryChromaticities           = $013F;
  IFDColorMap                        = $0140;
  IFDHalfToneHints                   = $0141;
  IFDTileWidth                       = $0142;
  IFDTileLength                      = $0143;
  IFDTileOffsets                     = $0144;
  IFDTileByteCounts                  = $0145;
  IFDSubIFDs                         = $014A;
  IFDInkSet                          = $014C;
  IFDInkNames                        = $014D;
  IFDNumberOfInks                    = $014E;
  IFDDotRange                        = $0150;
  IFDTargetPrinter                   = $0151;
  IFDExtraSample                     = $0152;
  IFDSampleFormat                    = $0153;
  IFDSMinSampleValue                 = $0154;
  IFDSMaxSampleValue                 = $0155;
  IFDTransferRange                   = $0156;
  IFDClipPath                        = $0157;
  IFDXClipPathUnits                  = $0158;
  IFDYClipPathUnits                  = $0159;
  IFDIndexed                         = $015A;
  IFDJPEGTables                      = $015B;
  IFDopiProxy                        = $015F;
  IFDSpecialMode                     = $0200;
  IFDJpegQual                        = $0201;
  IFDMacro                           = $0202;
  IFDUnknown1                        = $0203;
  IFDDigiZoom                        = $0204;
  IFDUnknown2                        = $0205;
  IFDUnknown3                        = $0206;
  IFDSoftwareRelease                 = $0207;
  IFDPictInfo                        = $0208;
  IFDCameraID                        = $0209;
  IFDEpson_Comments                  = $020A;
  IFDEpson_Width                     = $020B;
  IFDEpson_Height                    = $020C;
  IFDEpson_SoftRelease               = $020D;
  IFDYCbCrCoefficients               = $0211;
  IFDYCbCrSubSampling                = $0212;
  IFDYCbCrPositioning                = $0213;
  IFDReferenceBlackWhite             = $0214;
  IFDExtensibleMetaDataPlatform      = $02BC;
  IFDEpson                           = $0300;
  IFDGamma                           = $0301;
  IFDIccProfileDescriptor            = $0302;
  IFDSrgbRenderingIntent             = $0303;
  IFDImageTitle                      = $0304;
  IFDMinolta_pimdata                 = $0E00;
  IFDDataDump                        = $0F00;
  IFDFuji_Color                      = $1003;
  IFDOlympus_Bracket                 = $1006;
  IFDOlympus_FocusMode               = $100B;
  IFDOlympus_FocusDistance           = $100C;
  IFDOlympus_Zoom                    = $100D;
  IFDOlympus_MacroFocus              = $100E;
  IFDOlympus_Sharpness               = $100F;
  IFDFuji_FlashMode                  = $1010;
  IFDOlympus_BlackLevel              = $1012;
  IFDOlympus_WhiteBalance            = $1015;
  IFDOlympus_RedBias                 = $1017;
  IFDOlympus_BlueBias                = $1018;
  IFDOlympus_SerialNumber            = $101A;
  IFDFuji_Macro                      = $1020;
  IFDFuji_FocusMode                  = $1021;
  IFDOlympus_FlashBias               = $1023;
  IFDOlympus_Contrast                = $1029;
  IFDOlympus_SharpnessFactor         = $102A;
  IFDOlympus_ColorControl            = $102B;
  IFDOlympus_ValidBits               = $102C;
  IFDOlympus_Coring                  = $102D;
  IFDOlympus_FinalWidth              = $102E;
  IFDOlympus_FinalHeight             = $102F;
  IFDFuji_SlowSync                   = $1030;
  IFDFuji_PictureMode                = $1031;
  IFDOlympus_CompressionRatio        = $1034;
  IFDFuji_ContTake_Bracket           = $1100;
  IFDFuji_BlurWarning                = $1300;
  IFDFuji_FocusWarning               = $1301;
  IFDFuji_aeWarning                  = $1302;
  IFDResolutionXUnit                 = $5001;
  IFDResolutionYUnit                 = $5002;
  IFDResolutionXLengthUnit           = $5003;
  IFDResolutionYLengthUnit           = $5004;
  IFDPrintFlags                      = $5005;
  IFDPrintFlagsVersion               = $5006;
  IFDPrintFlagsCrop                  = $5007;
  IFDPrintFlagsBleedWidth            = $5008;
  IFDPrintFlagsBleedWidthScale       = $5009;
  IFDHalfToneLpi                     = $500A;
  IFDHalfToneLpiUnit                 = $500B;
  IFDHalfToneDegree                  = $500C;
  IFDHalfToneShape                   = $500D;
  IFDHalfToneMisc                    = $500E;
  IFDHalfToneScreen                  = $500F;
  IFDJpegQuality                     = $5010;
  IFDGridSize                        = $5011;
  IFDThumbnailFormat                 = $5012;
  IFDThumbnailWidth                  = $5013;
  IFDThumbnailHeight                 = $5014;
  IFDThumbnailColorDepth             = $5015;
  IFDThumbnailPlanes                 = $5016;
  IFDThumbnailRawBytes               = $5017;
  IFDThumbnailSize                   = $5018;
  IFDThumbnailCompressedSize         = $5019;
  IFDColorTransferFunction           = $501A;
  IFDThumbnailData                   = $501B;
  IFDThumbnailImageWidth             = $5020;
  IFDThumbnailImageHeight            = $5021;
  IFDThumbnailBitsPerSample          = $5022;
  IFDThumbnailCompression            = $5023;
  IFDThumbnailPhotoMetricInterp      = $5024;
  IFDThumbnailImageDescription       = $5025;
  IFDThumbnailEquipMake              = $5026;
  IFDThumbnailEquipModel             = $5027;
  IFDThumbnailStripOffsets           = $5028;
  IFDThumbnailOrientation            = $5029;
  IFDThumbnailSamplesPerPixel        = $502A;
  IFDThumbnailRowsPerStrip           = $502B;
  IFDThumbnailStripBytesCount        = $502C;
  IFDThumbnailResolutionX            = $502D;
  IFDThumbnailResolutionY            = $502E;
  IFDThumbnailPlanarConfig           = $502F;
  IFDThumbnailresolutionunit         = $5030;
  IFDThumbnailTransferFunction       = $5031;
  IFDThumbnailSoftwareUsed           = $5032;
  IFDThumbnailDateTime               = $5033;
  IFDThumbnailArtist                 = $5034;
  IFDThumbnailWhitePoint             = $5035;
  IFDThumbnailPrimaryChromaticities  = $5036;
  IFDThumbnailYcbcrCoefficients      = $5037;
  IFDThumbnailYcbcrSubSampling       = $5038;
  IFDThumbnailYcbcrPositioning       = $5039;
  IFDThumbnailRefBlackWhite          = $503A;
  IFDThumbnailCopyright              = $503B;
  IFDLuminanceTable                  = $5090;
  IFDChrominanceTable                = $5091;
  IFDFrameDelay                      = $5100;
  IFDLoopCount                       = $5101;
  IFDPixelUnit                       = $5110;
  IFDPixelPerUnitX                   = $5111;
  IFDPixelPerUnitY                   = $5112;
  IFDPaletteHistogram                = $5113;
  IFDImageID                         = $800D;
  IFDMatteing                        = $80E3;
  IFDDataType                        = $80E4;
  IFDImageDepth                      = $80E5;
  IFDTileDepth                       = $80E6;
  IFDCFARepeatPatternDim             = $828D;
  IFDCFAPattern1                     = $828E;
  IFDBatteryLevel                    = $828F;
  IFDCopyright                       = $8298;
  IFDExposureTime                    = $829A;
  IFDFNumber                         = $829D;
  IFDIptc_naa                        = $83BB;
  IFDIt8RasterPadding                = $84E3;
  IFDIt8ColorTable                   = $84E5;
  IFDImageresourceinformation        = $8649;
  IFDExifOffset                      = $8769;
  IFDInterColorProfile               = $8773;
  IFDExposureProgram                 = $8822;
  IFDSpectralSensitivity             = $8824;
  IFDGPSInfo                         = $8825;
  IFDISOSpeedRatings                 = $8827;
  IFDOECF                            = $8828;
  IFDInterlace                       = $8829;
  IFDTimeZoneOffset                  = $882A;
  IFDSelfTimerMode                   = $882B;
  IFDExifVersion                     = $9000;
  IFDDateTimeOriginal                = $9003;
  IFDDateTimeDigitized               = $9004;
  IFDComponentConfiguration          = $9101;
  IFDCompressedBitsPerPixel          = $9102;
  IFDShutterSpeedValue               = $9201;
  IFDApertureValue                   = $9202;
  IFDBrightnessValue                 = $9203;
  IFDExposureBiasValue               = $9204;
  IFDMaxApertureValue                = $9205;
  IFDSubjectDistance                 = $9206;
  IFDMeteringMode                    = $9207;
  IFDLightSource                     = $9208;
  IFDFlash                           = $9209;
  IFDFocalLength                     = $920A;
  IFDFlashEnergy1                    = $920B;
  IFDSpatialFrequencyResponse1       = $920C;
  IFDNoise                           = $920D;
  IFDFocalPlaneXResolution1          = $920E;
  IFDFocalPlaneYResolution1          = $920F;
  IFDFocalPlaneResolutionUnit1       = $9210;
  IFDImageNumber1                    = $9211;
  IFDSecurityClassification1         = $9212;
  IFDImageHistory1                   = $9213;
  IFDSubjectLocation1                = $9214;
  IFDExposureIndex1                  = $9215;
  IFDTIFF_EPStandardID1              = $9216;
  IFDSensingMethod1                  = $9217;
  IFDStonits                         = $923F;
  IFDMakerNote                       = $927C;
  IFDUserComment                     = $9286;
  IFDSubSecTime                      = $9290;
  IFDSubSecTimeOriginal              = $9291;
  IFDSubSecTimeDigitized             = $9292;
  IFDImageSourceData                 = $953C;
  IFDTitle                           = $9C9B;
  IFDComments                        = $9C9C;
  IFDAuthor                          = $9C9D;
  IFDKeywords                        = $9C9E;
  IFDSubject                         = $9C9F;
  IFDFlashPixVersion                 = $A000;
  IFDColorSpace                      = $A001;
  IFDExifImageWidth                  = $A002;
  IFDExifImageHeight                 = $A003;
  IFDRelatedSoundFile                = $A004;
  IFDExifInteroperabilityOffset      = $A005;
  IFDFlashEnergy2                    = $A20B;
  IFDSpatialFrequencyResponse2       = $A20C;
  IFDFocalPlaneXResolution2          = $A20E;
  IFDFocalPlaneYResolution2          = $A20F;
  IFDFocalPlaneResolutionUnit2       = $A210;
  IFDImageNumber2                    = $A211;
  IFDSecurityClassification2         = $A212;
  IFDImageHistory2                   = $A213;
  IFDSubjectLocation2                = $A214;
  IFDExposureIndex2                  = $A215;
  IFDTIFF_EPStandardID2              = $A216;
  IFDSensingMethod2                  = $A217;
  IFDFileSource                      = $A300;
  IFDSceneType                       = $A301;
  IFDCFAPattern2                     = $A302;
  IFDCustomRendered                  = $A401;
  IFDExposureMode                    = $A402;
  IFDWhiteBalance                    = $A403;
  IFDDigitalZoomRatio                = $A404;
  IFDFocalLengthIn35mmFilm           = $A405;
  IFDSceneCaptureType                = $A406;
  IFDGainControl                     = $A407;
  IFDContrast                        = $A408;
  IFDSaturation                      = $A409;
  IFDSharpness                       = $A40A;
  IFDDeviceSettingDescription        = $A40B;
  IFDSubjectDistanceRange            = $A40C;
  IFDImageUniqueID                   = $A420;

  IFDUnitsNone                       = 1;
  IFDUnitsInch                       = 2;
  IFDUnitsCM                         = 3;

  IFDOrientationNormal               = 1;
  IFDOrientationUpsideDown           = 3;
  IFDOrientationRotateRight          = 6;
  IFDOrientationRotateLeft           = 8;
  IFDOrientationUndefined            = 9;

type
  tJfifHeader                        = packed record
    SOIMarker                        : Byte;
    SOI                              : Byte;
    App0Marker                       : Byte;
    App0                             : Byte;
    App0Length                       : Word; { SwapWord ntig }
    JFIF                             : Array[0 .. 4] of AnsiChar;
    RevMajor                         : Byte;
    RevMinor                         : Byte;
    Units                            : Byte; { JpegUnits* }
    xDensity                         : Word; { SwapWord ntig }
    yDensity                         : Word; { SwapWord ntig }
    ThumbNailWidth                   : Byte;
    ThumbNailHeight                  : Byte;
  end;

  tExifHeader                        = packed record
    SOIMarker                        : Byte;
    SOI                              : Byte;
    App1Marker                       : Byte;
    App1                             : Byte;
    App1Length                       : Word;     { SwapWord ntig }
    Exif                             : Array[0 .. 4] of AnsiChar;
    Null                             : Byte;     { 0 }
    ByteAlign                        : Array[0 .. 1] of AnsiChar;
    TagMark                          : Word;     { evtl SwapWord ntig
                                                   je nach ByteAlign }
    IFD0Offset                       : LongWord; { evtl SwapLongWord ntig
                                                   je nach ByteAlign }
  end;

  tIFDEntry                          = packed record
    TagNumber                        : Word;     { evtl SwapWord ntig
                                                   je nach ByteAlign }
    Format                           : Word;     { evtl SwapWord ntig
                                                   je nach ByteAlign }
    ComponentCount                   : LongWord; { evtl SwapLongWord ntig
                                                   je nach ByteAlign }
    Data                             : Array[0 .. 3] of Byte;
  end;

  aIFDEntry                          = Array[0 .. 0] of tIFDEntry;
  pIFDEntry                          = ^aIFDEntry;

  tByteArray                         = Array[0 .. 0] of Byte;
  pByteArray                         = ^tByteArray;

  tSByte                             = ShortInt;
  tUByte                             = Byte;
  tSShort                            = SmallInt;
  tUShort                            = Word;
  tSLong                             = LongInt;
  tULong                             = LongWord;
  tSFloat                            = Single;
  tDFloat                            = Double;
  tSRational                         = record
                                         Numerator,
                                         Denominator : tSLong;
                                       end;
  tURational                         = record
                                         Numerator,
                                         Denominator : tULong;
                                       end;

procedure Swap(p : pByteArray; Size : Integer);
procedure SwapWord(var w : Word);
procedure SwapLongWord(var l : LongWord);
function  ReadJfifHeader(var Datei : File; var Header : tJfifHeader) : Boolean;
function  ReadExif(var Datei : File; var Header : tExifHeader) : Boolean;
function  ReadIFD(         var Datei         : File;
                           Motorola          : Boolean;
                           var Offset        : LongWord;
                           var IFDEntryCount : Word;
                           var IFDEntry      : pIFDEntry;
                           var NextIFD       : LongWord;
                           var IFDData       : pByteArray;
                           var IFDDataSize   : LongWord)   : Boolean;
function  WriteIFD(var Datei     : File;
                   FilePos       : LongWord;
                   Motorola      : Boolean;
                   IFDEntryCount : Word;
                   IFDEntry      : pIFDEntry;
                   NextIFD       : LongWord;
                   IFDData       : pByteArray;
                   IFDDataSize   : LongWord) : Boolean;
function SkipBytes(        var Datei         : File;
                           Count             : LongWord)   : Boolean;
function GetIFDDataAdr(    IFDEntry          : tIFDEntry;
                           Offset            : LongWord;
                           Motorola          : Boolean)    : LongWord;
function GetIFDDataSize(   Format            : Word) : Word;
function GetIFDDataP(      IFDEntry          : tIFDEntry;
                           Offset            : LongWord;
                           Motorola          : Boolean;
                           IFDData           : pByteArray;
                           IFDDataSize       : LongWord)   : pByteArray;
function GetIFDUShort(     IFDEntry          : tIFDEntry;
                           Motorola          : Boolean;
                           var UShort        : tUShort)    : Boolean;
function SetIFDUShort(     var IFDEntry      : tIFDEntry;
                           Motorola          : Boolean;
                           UShort            : tUShort)    : Boolean;
function GetIFDURational(  IFDEntry          : tIFDEntry;
                           Offset            : LongWord;
                           Motorola          : Boolean;
                           IFDData           : pByteArray;
                           IFDDataSize       : LongWord;
                           var URational     : tURational) : Boolean;
function GetIFDAscii(      IFDEntry          : tIFDEntry;
                           Offset            : LongWord;
                           Motorola          : Boolean;
                           IFDData           : pByteArray;
                           IFDDataSize       : LongWord;
                           var s             : String)     : Boolean;
procedure GetIFDResolution(IFDEntryCount     : Word;
                           IFDEntry          : pIFDEntry;
                           IFDData           : pByteArray;
                           IFDDataSize       : LongWord;
                           Motorola          : Boolean;
                           Offset            : LongWord;
                           var Units         : tUShort;
                           var x, y          : tURational);
procedure GetJpegDPI(FileName : String; var xDPI, yDPI : Integer);
procedure SetJpegDPI(FileName : String; xDPI, yDPI : Integer);

implementation

procedure Swap(p : pByteArray; Size : Integer);
  var     i, j,
          Mid : Integer;
          b     : Byte;
  begin
    j   := Size - 1;
    Mid := Size div 2 - 1;
    for i := 0 to Mid do
      begin
        {$R-}
          b     := p^[i];
          p^[i] := p^[j];
          p^[j] := b;
          dec(j);
        {$R+}
      end;
  end;

procedure SwapWord(var w : Word);
  begin
    Swap(@w, sizeof(Word));
  end;

procedure SwapLongWord(var l : LongWord);
  begin
    Swap(@l, sizeof(LongWord));
  end;

function ReadJfifHeader(var Datei : File; var Header : tJfifHeader) : Boolean;
  var    Count : Integer;
  begin
    ReadJfifHeader   := False;
    Header.SOIMarker := 0;
    Count            := 0;
    FillChar(Header, sizeof(Header), 0);
    BlockRead(Datei, Header, sizeof(Header), Count);
    if Count = sizeof(Header) then
      begin
        SwapWord(Header.App0Length);
        SwapWord(Header.xDensity);
        SwapWord(Header.yDensity);
        if    (Header.SOIMarker  = JPEGMarker)
          and (Header.SOI        = JPEGSOI)
          and (Header.App0Marker = JPEGMarker)
          and (Header.App0       = JPEGApp0)
          and (Header.App0Length >= 16)
          and (AnsiStrLComp(Header.JFIF, PAnsiChar(JpegTypeJFIF),
                            length(JpegTypeJFIF)) = 0)
          and (Header.JFIF[sizeof(Header.JFIF) - 1] = #0)
        then
          ReadJfifHeader := True;
      end;
  end;

function ReadExif(var Datei : File; var Header : tExifHeader) : Boolean;
  var    Count : Integer;
         {$ifndef fpc}
           adr : Integer;
         {$endif}
         Align : (AlignUnknown, AlignIntel, AlignMotorola);
  begin
    ReadExif         := False;
    Header.SOIMarker := 0;
    FillChar(Header, sizeof(Header), 0);
    Count            := 0;
    Align            := AlignUnknown;
    BlockRead(Datei, Header, sizeof(Header), Count);
    if Count = sizeof(Header) then
      begin
        SwapWord(Header.App1Length);
        if (AnsiStrLComp(Header.ByteAlign, TIFFAlignIntel,
                         StrLen(TIFFAlignIntel)) = 0)
        then
          Align := AlignIntel
        else
          if (AnsiStrLComp(Header.ByteAlign, TIFFAlignMotorola,
                       length(TIFFAlignMotorola)) = 0)
          then
            Align := AlignMotorola;
        if    (Header.SOIMarker  = JPEGMarker)
          and (Header.SOI        = JPEGSOI)
          and (Header.App1Marker = JPEGMarker)
          and (Header.App1       = JPEGApp1)
          and (Header.App1Length >= 16)
          and (AnsiStrLComp(Header.Exif, JpegTypeExif,
                            StrLen(JpegTypeExif)) = 0)
          and (Header.Exif[sizeof(Header.Exif) - 1] = #0)
          and (Header.Null = 0)
          and (Align <> AlignUnknown)
        then
          begin
            if Align = AlignMotorola then
              begin
                SwapWord(Header.TagMark);
                SwapLongWord(Header.IFD0Offset);
              end;
            if    (Header.TagMark    =  TiffTagMark)
              and (Header.IFD0Offset >= TiffIFD0Offset)
            then
              ReadExif := True;
          end;
      end;
  end;

function GetIFDDataAdr(IFDEntry : tIFDEntry; Offset : LongWord;
                       Motorola : Boolean) : LongWord;
  var    Adr : LongWord;
  begin
    Adr := LongWord(IFDEntry.Data);
    if Motorola then
      SwapLongWord(Adr);
    GetIFDDataAdr := Adr - Offset;
  end;

function GetIFDDataSize(Format : Word) : Word;
  begin
    case Format of
      IFDFormatUByte     : GetIFDDataSize := sizeof(tUByte);
      IFDFormatSByte     : GetIFDDataSize := sizeof(tSByte);
      IFDFormatASCII     : GetIFDDataSize := 1;
      IFDFormatUndefined : GetIFDDataSize := 1;
      IFDFormatUShort    : GetIFDDataSize := sizeof(tUShort);
      IFDFormatSShort    : GetIFDDataSize := sizeof(tSShort);
      IFDFormatUlong     : GetIFDDataSize := sizeof(tULong);
      IFDFormatSLong     : GetIFDDataSize := sizeof(tSLong);
      IFDFormatSFloat    : GetIFDDataSize := sizeof(tSFloat);
      IFDFormatURational : GetIFDDataSize := sizeof(tURational);
      IFDFormatSRational : GetIFDDataSize := sizeof(tSRational);
      IFDFormatDFloat    : GetIFDDataSize := sizeof(tDFloat);
    else
      GetIFDDataSize := 0;
    end;
  end;

function GetIFDDataP(IFDEntry    : tIFDEntry;
                     Offset      : LongWord;
                     Motorola    : Boolean;
                     IFDData     : pByteArray;
                     IFDDataSize : LongWord) : pByteArray;
  var    Size, Adr : LongWord;
  begin
    Size := GetIFDDataSize(IFDEntry.Format) * IFDEntry.ComponentCount;
    if Size <= sizeof(IFDEntry.Data) then
      GetIFDDataP := @IFDEntry.Data
    else
      begin
        Adr := GetIFDDataAdr(IFDEntry, Offset, Motorola);
        if Adr + Size > IFDDataSize then
          GetIFDDataP := Nil
        else
          {$R-}
            GetIFDDataP := @IFDData^[Adr];
          {$R+}
      end;
  end;

function GetIFDUShort(IFDEntry   : tIFDEntry;
                      Motorola   : Boolean;
                      var UShort : tUShort) : Boolean;
  type pUShort = ^tUShort;
  begin
    GetIFDUShort := False;
    if    (IFDEntry.Format         = IFDFormatUShort)
      and (IFDEntry.ComponentCount = 1 )
    then
      begin
        UShort := pUShort(@IFDEntry.Data)^;
        if Motorola then
          SwapWord(UShort);
        GetIFDUShort := True;
      end;
  end;

function SetIFDUShort(var IFDEntry : tIFDEntry;
                      Motorola     : Boolean;
                      UShort       : tUShort) : Boolean;
  type pUShort = ^tUShort;
  begin
    SetIFDUShort := False;
    if    (IFDEntry.Format         = IFDFormatUShort)
      and (IFDEntry.ComponentCount = 1 )
    then
      begin
        pUShort(@IFDEntry.Data)^ := UShort;
        if Motorola then
          SwapWord(pUShort(@IFDEntry.Data)^);
        SetIFDUShort := True;
      end;
  end;

function GetIFDURational(IFDEntry      : tIFDEntry;
                         Offset        : LongWord;
                         Motorola      : Boolean;
                         IFDData       : pByteArray;
                         IFDDataSize   : LongWord;
                         var URational : tURational) : Boolean;
  type pURational = ^tURational;
  var  p          : pByteArray;
  begin
    GetIFDURational := False;
    if    (IFDEntry.Format         = IFDFormatURational)
      and (IFDEntry.ComponentCount = 1 )
    then
      begin
        p := GetIFDDataP(IFDEntry, Offset, Motorola, IFDData, IFDDataSize);
        if p <> Nil then
          begin
            URational := pURational(p)^;
            if Motorola then
              begin
                SwapLongWord(URational.Numerator);
                SwapLongWord(URational.Denominator);
              end;
            GetIFDURational := True;
          end;
      end;
  end;

function GetIFDAscii(IFDEntry      : tIFDEntry;
                     Offset        : LongWord;
                     Motorola      : Boolean;
                     IFDData       : pByteArray;
                     IFDDataSize   : LongWord;
                     var s         : String) : Boolean;
  var Data : pByteArray;
      j    : LongWord;
  begin
    GetIFDAscii := False;
    if IFDEntry.Format = IFDFormatASCII then
      begin
        Data := GetIFDDataP(IFDEntry, Offset, Motorola, IFDData, IFDDataSize);
        if Data <> Nil then
          begin
            s := '';
            j := 0;
            while j < IFDEntry.ComponentCount do
              {$R-}
                if Data^[j] >= ord(' ') then
                  begin
                    s := s + chr(Data^[j]);
                    inc(j);
                  end
                else
                  j := IFDEntry.ComponentCount;
              {$R+}
            GetIFDAscii := True;
          end;
      end;
  end;

function ReadIFD(var Datei         : File;
                 Motorola          : Boolean;
                 var Offset        : LongWord;
                 var IFDEntryCount : Word;
                 var IFDEntry      : pIFDEntry;
                 var NextIFD       : LongWord;
                 var IFDData       : pByteArray;
                 var IFDDataSize   : LongWord) : Boolean;
  var ok      : Boolean;
      Size,
      i, j, l : LongWord;
  begin
    ok            := False;
    IFDEntryCount := 0;
    IFDEntry      := Nil;
    NextIFD       := 0;
    IFDData       := Nil;
    IFDDataSize   := 0;
    i             := 0;
    j             := 0;
    BlockRead(Datei, IFDEntryCount, sizeof(IFDEntryCount), i);
    if i = sizeof(IFDEntryCount) then
      begin
        if Motorola then
          SwapWord(IFDEntryCount);
        i := IFDEntryCount * sizeof(tIFDEntry);
        inc(Offset, LongWord(sizeof(IFDEntryCount)) + i
                    + LongWord(sizeof(NextIFD)));
        GetMem(IFDEntry, i);
        if IFDEntry <> Nil then
          begin
            BlockRead(Datei, IFDEntry^, i, j);
            if j = i then
              begin
                BlockRead(Datei, NextIFD, sizeof(NextIFD), i);
                if i = sizeof(NextIFD) then
                  begin
                    if Motorola then
                      SwapLongWord(NextIFD);
                    for i := 0 to IFDEntryCount - 1 do
                      begin
                        {$R-}
                          if Motorola then
                            begin
                              SwapWord(IFDEntry^[i].TagNumber);
                              SwapWord(IFDEntry^[i].Format);
                              SwapLongWord(IFDEntry^[i].ComponentCount);
                            end;
                          Size := GetIFDDataSize(IFDEntry^[i].Format)
                                  * IFDEntry^[i].ComponentCount;
                          if Size > sizeof(IFDEntry^[i].Data) then
                            begin
                              l := GetIFDDataAdr(IFDEntry^[i], Offset, Motorola)
                                   + Size;
                              if l > IFDDataSize then
                                IFDDataSize := l;
                            end;
                        {$R+}
                      end;
                    if IFDDataSize > 0 then
                      begin
                        GetMem(IFDData, IFDDataSize);
                        if IFDData <> Nil then
                          begin
                            BlockRead(Datei, IFDData^, IFDDataSize, i);
                            if i = IFDDataSize then
                              ok := True;
                          end;
                      end;
                  end
              end;
          end;
      end;
    if not ok then
      begin
        if IFDEntry <> Nil then
          FreeMem(IFDEntry, IFDEntryCount * sizeof(tIFDEntry));
        if IFDData <> Nil then
          FreeMem(IFDData, IFDDataSize);
        IFDEntryCount := 0;
        IFDEntry      := Nil;
        NextIFD       := 0;
        IFDData       := Nil;
        IFDDataSize   := 0;
      end;
    ReadIFD := ok;
  end;

function WriteIFD(var Datei     : File;
                  FilePos       : LongWord;
                  Motorola      : Boolean;
                  IFDEntryCount : Word;
                  IFDEntry      : pIFDEntry;
                  NextIFD       : LongWord;
                  IFDData       : pByteArray;
                  IFDDataSize   : LongWord) : Boolean;
  var i, j : LongWord;
  begin
    WriteIFD := False;
    Seek(Datei, FilePos);
    if Motorola then
      SwapWord(IFDEntryCount);
    i := 0;
    BlockWrite(Datei, IFDEntryCount, sizeof(IFDEntryCount), i);
    if Motorola then
      SwapWord(IFDEntryCount);
    if i = sizeof(IFDEntryCount) then
      begin
        for i := 0 to IFDEntryCount - 1 do
          {$R-}
            if Motorola then
              begin
                SwapWord(IFDEntry^[i].TagNumber);
                SwapWord(IFDEntry^[i].Format);
                SwapLongWord(IFDEntry^[i].ComponentCount);
              end;
          {$R+}
        i := IFDEntryCount * sizeof(tIFDEntry);
        j := 0;
        BlockWrite(Datei, IFDEntry^, i, j);
        if j = i then
          begin
            if Motorola then
              SwapLongWord(NextIFD);
            BlockWrite(Datei, NextIFD, sizeof(NextIFD), i);
            if Motorola then
              SwapLongWord(NextIFD);
            if i = sizeof(NextIFD) then
              if IFDDataSize > 0 then
                if IFDData <> Nil then
                  begin
                    BlockWrite(Datei, IFDData^, IFDDataSize, i);
                    if i = IFDDataSize then
                      WriteIFD := True;
                  end;
          end;
        for i := 0 to IFDEntryCount - 1 do
          {$R-}
            if Motorola then
              begin
                SwapWord(IFDEntry^[i].TagNumber);
                SwapWord(IFDEntry^[i].Format);
                SwapLongWord(IFDEntry^[i].ComponentCount);
              end;
          {$R+}
      end;
  end;

function SkipBytes(var Datei : File; Count : LongWord) : Boolean;
  var    p    : Pointer;
         Read : LongWord;
  begin
    SkipBytes := True;
    Read      := 0;
    if Count > 0 then
      begin
        GetMem(p, Count);
        if p = Nil then
          SkipBytes := False
        else
          begin
            BlockRead(Datei, p^, Count, Read);
            if Read <> Count then
              SkipBytes := False;
            FreeMem(p, Count);
          end;
      end;
  end;

procedure GetIFDResolution(IFDEntryCount : Word;
                           IFDEntry      : pIFDEntry;
                           IFDData       : pByteArray;
                           IFDDataSize   : LongWord;
                           Motorola      : Boolean;
                           Offset        : LongWord;
                           var Units     : tUShort;
                           var x, y      : tURational);
  var i : LongWord;
  begin
    Units         := IFDUnitsNone;
    x.Numerator   := 0;
    x.Denominator := 0;
    y.Numerator   := 0;
    y.Denominator := 0;
    for i := 0 to IFDEntryCount - 1 do
      {$R-}
        case IFDEntry^[i].TagNumber of
          IFDResolutionUnit :
            GetIFDUShort(IFDEntry^[i], Motorola, Units);
          IFDXResolution :
            GetIFDURational(IFDEntry^[i], Offset, Motorola, IFDData,
                            IFDDataSize, x);
          IFDYResolution :
            GetIFDURational(IFDEntry^[i], Offset, Motorola, IFDData,
                            IFDDataSize, y);
        end;
      {$R+}
  end;

procedure GetJpegDPI(FileName : String; var xDPI, yDPI : Integer);
  var     JfifHeader    : tJfifHeader;
          ExifHeader    : tExifHeader;
          IFDEntry      : pIFDEntry;
          IFDEntryCount : Word;
          IFDData       : pByteArray;
          Offset,
          NextIFD,
          IFDDataSize   : LongWord;
          Datei         : File;
          Motorola,
          b             : Boolean;
          Units         : tUShort;
          xRes, yRes    : tURational;
  begin
    xDPI                 := 0;
    yDPI                 := 0;
    Units                := IFDUnitsNone;
    IFDEntry             := Nil;
    IFDEntryCount        := 0;
    NextIFD              := 0;
    IFDData              := Nil;
    IFDDataSize          := 0;
    JfifHeader.SOIMarker := 0;
    ExifHeader.SOIMarker := 0;
    xRes.Numerator       := 0;
    xRes.Denominator     := 0;
    yRes.Numerator       := 0;
    yRes.Denominator     := 0;
    b                    := True;
    AssignFile(Datei, FileName);
    try
      Reset(Datei, 1);
    except
      b := False;
    end;
    if b then
      begin
        if ReadJfifHeader(Datei, JfifHeader) then
          case JfifHeader.Units of
            JFIFUnitsDPI :
              begin
                xDPI := JfifHeader.xDensity;
                yDPI := JfifHeader.yDensity;
              end;
            JFIFUnitsDPCM :
              begin
                xDPI := (LongWord(JfifHeader.xDensity) * 254 + 50) div 100;
                yDPI := (LongWord(JfifHeader.yDensity) * 254 + 50) div 100;
              end;
          end
        else
          begin
            Seek(Datei, 0);
            if ReadExif(Datei, ExifHeader) then
              begin
                Offset   := ExifHeader.IFD0Offset;
                Motorola := AnsiStrLComp(ExifHeader.ByteAlign, TIFFAlignMotorola,
                                         StrLen(TIFFAlignMotorola)) = 0;
                b        := SkipBytes(Datei, ExifHeader.IFD0Offset
                                             - TiffIFD0Offset);
                while b do
                  begin
                    if not ReadIFD(Datei, Motorola, Offset, IFDEntryCount,
                                   IFDEntry, NextIFD, IFDData, IFDDataSize)
                    then
                      b := False
                    else
                      begin
                        GetIFDResolution(IFDEntryCount, IFDEntry, IFDData,
                                         IFDDataSize, Motorola, Offset, Units,
                                         xRes, yRes);
                        inc(Offset, IFDDataSize);
                        if xRes.Denominator > 0 then
                          case Units of
                            IFDUnitsInch :
                              xDPI := xRes.Numerator div xRes.Denominator;
                            IFDUnitsCM :
                              xDPI := (xRes.Numerator * 100)
                                      div (xRes.Denominator * 254);
                          end;
                        if yRes.Denominator > 0 then
                          case Units of
                            IFDUnitsInch :
                              yDPI := yRes.Numerator div yRes.Denominator;
                            IFDUnitsCM :
                              yDPI := (yRes.Numerator * 100)
                                      div (yRes.Denominator * 254);
                          end;
                        if (xDPI > 0) and (yDPI > 0) then
                          b := False;
                      end;
                    if IFDEntry <> Nil then
                      FreeMem(IFDEntry, IFDEntryCount * sizeof(tIFDEntry));
                    if IFDData <> Nil then
                      FreeMem(IFDData, IFDDataSize);
                  end;
              end;
          end;
        CloseFile(Datei);
      end;
  end;

procedure SetJpegDPI(FileName : String; xDPI, yDPI : Integer);
  var     Header : tJfifHeader;
          Datei  : File;
          ok     : Boolean;
          Count  : Integer;
  begin
    Header.SOIMarker := 0;
    Count := 0;
    ok    := True;
    AssignFile(Datei, FileName);
    try
      Reset(Datei, 1);
    except
      ok := False;
    end;
    if ok then
      begin
        if ReadJfifHeader(Datei, Header) then
          begin
            Header.Units    := JFIFUnitsDPI;
            Header.xDensity := xDPI;
            Header.yDensity := yDPI;
            SwapWord(Header.App0Length);
            SwapWord(Header.xDensity);
            SwapWord(Header.yDensity);
            Seek(Datei, 0);
            BlockWrite(Datei, Header, sizeof(Header), Count);
          end;
        CloseFile(Datei);
      end;
  end;

end.

