unit extpng;

{ http://www.wotsit.org/
  png.zip   70.140 Bytes
  Draft-bo 239.580 Bytes September 8th 1996
}

{$ifdef fpc}
  {$mode objfpc}{$H+}
{$endif}

interface

uses
  {$ifndef fpc}
    Windows,
  {$endif}
  SysUtils,
  WolfUtil;

const
  PngNonASCII         = #$89;
  PngID               = 'PNG';
  PngChunkIHDR        = 'IHDR';
  PngChunkIEND        = 'IEND';
  PngChunkIDAT        = 'IDAT';
  PngChunkpHYs        = 'pHYs';

  PngColorBitPalette  = 1;
  PngColorBitColor    = 2;
  PngColorBitAlpha    = 4;

  PngInterlaceNone    = 0;
  PngInterlaceAdam7   = 1;

  PngUnitsAspectRatio = 0;
  PngUnitsPixPerMeter = 1;

type
  tChunkType          = Array[0 .. 3] of AnsiChar;

  tPngHeader          = packed record
    NonASCII          : AnsiChar;
    PNG               : Array[0 .. 2] of AnsiChar;
    CR                : AnsiChar;
    LF1               : AnsiChar;
    CtrlZ             : AnsiChar;
    LF2               : AnsiChar;
  end;

  tChunkHeader        = packed Record
    Len               : LongWord;
    ChunkType         : tChunkType;
  end;

  tIHDR               = packed Record
    Width             : LongWord;
    Height            : LongWord;
    BitDepth          : Byte;
    ColorType         : Byte;
    CompressionMethod : Byte;
    FilterMethod      : Byte;
    InterlaceMethod   : Byte;
  end;

  tpHYs               = packed Record
    x, y              : LongWord;
    Units             : Byte;
  end;

procedure SwapLongWord(var l : LongWord);
function  WritePngHeader(var Datei : File; Header : tPngHeader) : Boolean;
function  ReadPngHeader(var Datei : File; var Header : tPngHeader) : Boolean;
function  WritePngChunk(var Datei : File; Header : tChunkHeader;
                        Data  : Pointer) : Boolean;
function  ReadPngChunk(var Datei : File; var Header : tChunkHeader;
                       var Data  : Pointer) : Boolean;
procedure GetPngPixPerMeter(FileName : String; var x, y : Integer);
procedure SetPngPixPerMeter(FileName : String; x, y : Integer);

implementation

var
  crc_table : Array[0 .. $FF] of LongWord;

procedure SwapLongWord(var l : LongWord);
  type    t = array[0 .. 3] of Byte;
  var     p : ^t;
          a : t;
  begin
    p    := @l;
    a[0] := p^[3];
    a[1] := p^[2];
    a[2] := p^[1];
    a[3] := p^[0];
    l    := LongWord(a);
  end;

function WritePngHeader(var Datei : File; Header : tPngHeader) : Boolean;
  var    Count : Integer;
  begin
    WritePngHeader := False;
    Count          := 0;
    BlockWrite(Datei, Header, sizeof(Header), Count);
    if sizeof(Header) = Count then
      WritePngHeader := True;
  end;

function ReadPngHeader(var Datei : File; var Header : tPngHeader) : Boolean;
  var    Count : Integer;
  begin
    ReadPngHeader := False;
    Count         := 0;
    BlockRead(Datei, Header, sizeof(Header), Count);
    if sizeof(Header) = Count then
      begin
        if    (Header.NonASCII = PngNonASCII)
          and (StrLComp(Header.PNG, PngID, sizeof(Header.PNG)) = 0)
          and (Header.CR    = #$0D)
          and (Header.LF1   = #$0A)
          and (Header.CtrlZ = chr(ord('Z') and $1F))
          and (Header.LF2   = #$0A)
        then
          ReadPngHeader := True;
      end;
  end;

function update_crc(crc : LongWord; p : Pointer; len : LongWord) : LongWord;
  type   t    = Array[0 .. 0] of Byte;
  var    c, n : LongWord;
         a    : ^t;
  begin
    c := crc;
    a := p;
    for n := 0 to Len - 1 do
      {$R-}
        c := crc_table[(c xor a^[n]) and $ff] xor (c shr 8);
      {$R+}
    update_crc := c;
  end;

function WritePngChunk(var Datei : File; Header : tChunkHeader;
                       Data : Pointer) : Boolean;
  var    Count,
         CRC,
         Len    : LongWord;
         b      : Boolean;
  begin
    b   := False;
    Len := Header.Len;
    SwapLongWord(Header.Len);
    Count := 0;
    BlockWrite(Datei, Header, sizeof(Header), Count);
    if sizeof(Header) = Count then
      begin
        if Len = 0 then
          b := True
        else
          if Data <> Nil then
            begin
              BlockWrite(Datei, Data^, Len, Count);
              if Count = Len then
                b := True;
            end;
        if b then
          begin
            CRC := update_crc($FFFFFFFF, @Header.ChunkType,
                              sizeof(Header.ChunkType));
            if len > 0 then
              CRC := update_crc(CRC, Data, Len);
            CRC := CRC xor $FFFFFFFF;
            SwapLongWord(CRC);
            BlockWrite(Datei, CRC, sizeof(CRC), Count);
            if sizeof(CRC) <> Count then
              b := False;
          end;
      end;
    WritePngChunk := b;
  end;

function ReadPngChunk(var Datei : File; var Header : tChunkHeader;
                      var Data  : Pointer) : Boolean;
  var    Count,
         CRC,
         GoodCRC : LongWord;
         b       : Boolean;
  begin
    Header.Len := 0;
    FillChar(Header, sizeof(Header), 0);
    Data  := Nil;
    Count := 0;
    CRC   := 0;
    b     := False;
    BlockRead(Datei, Header, sizeof(Header), Count);
    if sizeof(Header) = Count then
      begin
        SwapLongWord(Header.Len);
        if Header.Len = 0 then
          b := True
        else
          begin
            GetMem(Data, Header.Len);
            if Data <> Nil then
              begin
                BlockRead(Datei, Data^, Header.Len, Count);
                if Count = Header.Len then
                  b := True
                else
                  begin
                    FreeMem(Data, Header.Len);
                    Data := Nil;
                  end;
              end;
          end;
        if b then
          begin
            BlockRead(Datei, CRC, sizeof(CRC), Count);
            if sizeof(CRC) <> Count then
              b := False
            else
              begin
                SwapLongWord(CRC);
                GoodCRC := update_crc($FFFFFFFF, @Header.ChunkType,
                                      sizeof(Header.ChunkType));
                if Header.Len > 0 then
                  GoodCRC := update_crc(GoodCRC, Data, Header.Len);
                GoodCRC := GoodCRC xor $FFFFFFFF;
                if CRC <> GoodCRC then
                  b := False;
              end;
          end;
        if not b and (Data <> Nil) then
          begin
            FreeMem(Data, Header.Len);
            Data := Nil;
          end;
      end;
    ReadPngChunk := b;
  end;

procedure GetPngPixPerMeter(FileName : String; var x, y : Integer);
  var     PngHeader   : tPngHeader;
          ChunkHeader : tChunkHeader;
          pHYs        : ^tpHYs;
          Datei       : File;
          b           : Boolean;
          p           : Pointer;
  begin
    x               := 0;
    y               := 0;
    PngHeader.CR    := #$0D;
    ChunkHeader.Len := 0;
    p               := Nil;
    b               := True;
    AssignFile(Datei, FileName);
    try
      Reset(Datei, 1);
    except
      b := False;
    end;
    if b then
      begin
        if ReadPngHeader(Datei, PngHeader) then
          while b do
            begin
              b := ReadPngChunk(Datei, ChunkHeader, p);
              if b then
                begin
                  if StrLComp(ChunkHeader.ChunkType, PngChunkIEND,
                              sizeof(ChunkHeader.ChunkType)) = 0
                  then
                    b := False
                  else
                    if    (StrLComp(ChunkHeader.ChunkType, PngChunkpHYs,
                                    sizeof(ChunkHeader.ChunkType)) = 0)
                      and (sizeof(tpHYs) <= ChunkHeader.Len)
                    then
                      begin
                        pHYs := p;
                        SwapLongWord(pHYs^.x);
                        SwapLongWord(pHYs^.y);
                        if PngUnitsPixPerMeter = pHYs^.Units then
                          begin
                            x := pHYs^.x;
                            y := pHYs^.y;
                          end;
                      end;
                end;
            end;
        CloseFile(Datei);
      end;
  end;

procedure SetPngPixPerMeter(FileName : String; x, y : Integer);
  var     PngHeader       : tPngHeader;
          pHYsChunkHeader,
          ChunkHeader     : tChunkHeader;
          rDatei,
          wDatei          : File;
          b, ok,
          Replace         : Boolean;
          p               : Pointer;
          pHYs            : ^tpHYs;
          tmpFileName     : packed Array[0 .. MAX_PATH] of char;
  begin
    tmpFileName[0]  := #0;
    ok              := False;
    Replace         := False;
    PngHeader.CR    := #$0D;
    ChunkHeader.Len := 0;
    p               := Nil;
    b               := True;
    AssignFile(rDatei, FileName);
    try
      Reset(rDatei, 1);
    except
      b := False;
    end;
    if b then
      begin
        if GetTempFileName(pChar(ExtractFilePath(FileName)),
                           'PNG', 0, tmpFileName) <> 0
        then
          begin
            {$ifdef fpc}
              AssignFile(wDatei, PChar(tmpFileName));
            {$else}
              AssignFile(wDatei, tmpFileName);
            {$endif}
            try
              Rewrite(wDatei, 1);
            except
              b := False;
            end;
            if b then
              begin
                if ReadPngHeader(rDatei, PngHeader) then
                  if WritePngHeader(wDatei, PngHeader) then
                    begin
                      ok := True;
                      while b do
                        begin
                          b := ReadPngChunk(rDatei, ChunkHeader, p);
                          if not b then
                            ok := False
                          else
                            begin
                              if StrLComp(ChunkHeader.ChunkType, PngChunkIEND,
                                          sizeof(ChunkHeader.ChunkType)) = 0
                              then
                                b := False
                              else if (StrLComp(ChunkHeader.ChunkType,
                                                PngChunkpHYs,
                                                sizeof(ChunkHeader.ChunkType))
                                       = 0)
                              then
                                begin
                                  if   (sizeof(tpHYs) > ChunkHeader.Len)
                                    or (p = Nil)
                                  then
                                    ok := False
                                  else
                                    begin
                                      Replace     := True;
                                      pHYs        := p;
                                      pHYs^.Units := PngUnitsPixPerMeter;
                                      pHYs^.x     := x;
                                      pHYs^.y     := y;
                                      SwapLongWord(pHYs^.x);
                                      SwapLongWord(pHYs^.y);
                                    end;
                                end
                              else if (StrLComp(ChunkHeader.ChunkType,
                                                PngChunkIDAT,
                                                sizeof(ChunkHeader.ChunkType))
                                       = 0)
                              then
                                if not Replace then
                                  begin
                                    Replace := True;
                                    GetMem(pHYs, sizeof(tpHYs));
                                    if pHYs = Nil then
                                      ok := False
                                    else
                                      begin
                                        pHYsChunkHeader.Len := sizeof(tpHYs);
                                        Move(PngChunkpHYs,
                                             pHYsChunkHeader.ChunkType,
                                             sizeof(pHYsChunkHeader.ChunkType));
                                        pHYs^.Units := PngUnitsPixPerMeter;
                                        pHYs^.x     := x;
                                        pHYs^.y     := y;
                                        SwapLongWord(pHYs^.x);
                                        SwapLongWord(pHYs^.y);
                                        if not WritePngChunk(wDatei,
                                          pHYsChunkHeader, pHYs)
                                        then
                                          ok := False;
                                        FreeMem(pHYs);
                                      end;
                                  end;
                              if not WritePngChunk(wDatei, ChunkHeader, p) then
                                ok := False;
                            end;
                        end;
                    end;
                CloseFile(wDatei);
              end;
          end;
        CloseFile(rDatei);
      end;
    if ok then
      begin
        DeleteFile(FileName);
        RenameFile(tmpFileName, FileName);
      end
    else
      if tmpFileName > '' then
        DeleteFile(tmpFileName);
  end;

procedure make_crc_table;
  var     c, n, k : LongWord;
  begin
    for n := low(crc_table) to high(crc_table) do
      begin
        c := n;
        for k := 0 to 7 do
          if (c and 1) <> 0 then
            c := $edb88320 xor (c shr 1)
          else
            c := c shr 1;
        crc_table[n] := c;
      end;
  end;

initialization
  make_crc_table;

end.

