unit layout;

{$ifdef fpc}
  {$mode objfpc}{$H+}
{$endif}

{$ifdef MSWINDOWS}
  {$ifndef fpc}
    { Delphi 2005 and newer }
    {$define DelphiPng}
    // https://sourceforge.net/projects/pngdelphi2/
  {$endif}
{$endif}

{$ifdef Ver130}
  { Delphi 5 }
  {$define DelphiPng}
{$endif}

{$ifdef Ver120}
  { Delphi 4 }
  {$define DelphiPng}
{$endif}

{$ifdef DelphiPng}
  {$define HavePng}
{$endif}

{$ifdef fpc}
  {$define HavePng}
{$endif}

interface

uses
  {$ifndef Unix}
    Windows,
  {$endif}
  {$ifdef fpc}
    LCLType,
    LConvEncoding,
    LResources,
  {$else}
    JPeg,
    GifImage, // http://www.tolderlund.eu/delphi/
    PcxImage,
    {$ifdef Ver330}
      UITypes,
    {$endif}
  {$endif}
  {$ifdef DelphiPng}
    PngImage,
  {$endif}
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  IniFiles,
  Printers,
  Grids,
  ExtDlgs,
  ComCtrls,
  Buttons,
  Common,
  Zusatz,
  WolfUtil,
  Copyrite;

Type
  tOpenIndex            = (OpenIndexAll, 
                           {$ifndef fpc}
                             OpenIndexPcx,
                             OpenIndexEmf,
                             OpenIndexWmf,
                           {$endif}
                           OpenIndexBmp,
                           OpenIndexJpg,
                           OpenIndexJpeg,
                           OpenIndexGif,
                           {$ifdef HavePng}
                             OpenIndexPng,
                           {$endif}
                           OpenIndexIco);

  { TLayoutForm }

  TLayoutForm           = class(TForm)
    LabelLayoutName     : TLabel;
    Layout              : TEdit;
    Elements            : TGroupBox;
    Element             : TRadioGroup;
    IsMergedDate        : TCheckBox;
    BtnAdjust           : TButton;
    CheckSizes          : TCheckBox;
    Weeks               : TCheckBox;
    ColumnsEdit         : TEdit;
    Columns             : TUpDown;
    LabelColumns        : TLabel;
    Sizes               : TGroupBox;
    Paper               : TGroupBox;
    LabelHeight         : TLabel;
    PaperHeight         : TEdit;
    UpDownHeight        : TUpDown;
    LabelWidth          : TLabel;
    PaperWidth          : TEdit;
    UpDownWidth         : TUpDown;
    Margin              : TGroupBox;
    LabelTop            : TLabel;
    MarginTop           : TEdit;
    UpDownTop           : TUpDown;
    LabelLeft           : TLabel;
    MarginLeft          : TEdit;
    UpDownLeft          : TUpDown;
    LabelRight          : TLabel;
    MarginRight         : TEdit;
    UpDownRight         : TUpDown;
    LabelBottom         : TLabel;
    MarginBottom        : TEdit;
    UpDownBottom        : TUpDown;
    Orientation         : TRadioGroup;
    Punching            : TGroupBox;
    PunchesEdit         : TEdit;
    Punches             : TUpDown;
    LabelPunches        : TLabel;
    LabelMid            : TLabel;
    LabelPunch1         : TLabel;
    Punch1              : TEdit;
    UpDownPunch1        : TUpDown;
    LabelPunch2         : TLabel;
    Punch2              : TEdit;
    UpDownPunch2        : TUpDown;
    LabelPunch3         : TLabel;
    Punch3              : TEdit;
    UpDownPunch3        : TUpDown;
    Headline            : TCheckBox;
    HolidayList         : TCheckBox;
    DocFrame            : TCheckBox;
    MonthFrame          : TCheckBox;
    PictureGroup        : TGroupBox;
    Pictures            : TStringGrid;
    BtnAuswahl          : TButton;
    BtnDelete           : TButton;
    LabelFooter         : TLabel;
    Footer              : TEdit;
    BtnTest             : TButton;
    FontDialog          : TFontDialog;
    OpenDialog          : TOpenPictureDialog;
    BtnOk               : TButton;
    BtnCancel           : TButton;
    FormBackground      : TGroupBox;
    EditFormBkGnd       : TEdit;
    cbFormBackground    : TCheckBox;
    BtnFormBkgnd        : TBitBtn;
    rbStretch           : TRadioButton;
    rbTile              : TRadioButton;
    DayBackground       : TGroupBox;
    EditDayBkGnd        : TEdit;
    cbDayBackground     : TCheckBox;
    BtnDayBkGnd         : TBitBtn;
    DayMonthGroupBox    : TGroupBox;
    DayMonthFormat      : TEdit;
    rbLeft              : TRadioButton;
    rbRight             : TRadioButton;
    procedure             BtnOkClick(Sender: TObject);
    procedure             BtnAdjustClick(Sender: TObject);
    procedure             FormCreate(Sender: TObject);
    procedure             BtnCancelClick(Sender: TObject);
    procedure             FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure             BtnTestClick(Sender: TObject);
    procedure             ElementClick(Sender: TObject);
    procedure             PunchesChange(Sender: TObject);
    procedure             BtnAuswahlClick(Sender: TObject);
    {$ifdef fpc}
      procedure           FontDialogApply(Sender: TObject);
    {$else}
      procedure           FontDialogApply(Sender: TObject; Wnd: HWND);
    {$endif}
    procedure             BtnDeleteClick(Sender: TObject);
    procedure             ColumnsEditExit(Sender: TObject);
    procedure             UpDownHeightClick(Sender: TObject; Button: TUDBtnType);
    procedure             UpDownWidthClick(Sender: TObject; Button: TUDBtnType);
    procedure             UpDownTopClick(Sender: TObject; Button: TUDBtnType);
    procedure             UpDownLeftClick(Sender: TObject; Button: TUDBtnType);
    procedure             UpDownRightClick(Sender: TObject; Button: TUDBtnType);
    procedure             UpDownBottomClick(Sender: TObject; Button: TUDBtnType);
    procedure             UpDownPunch1Click(Sender: TObject; Button: TUDBtnType);
    procedure             UpDownPunch2Click(Sender: TObject; Button: TUDBtnType);
    procedure             UpDownPunch3Click(Sender: TObject; Button: TUDBtnType);
    procedure             PaperHeightChange(Sender: TObject);
    procedure             PunchExit(Sender: TObject);
    procedure             MarginTopChange(Sender: TObject);
    procedure             MarginBottomChange(Sender: TObject);
    procedure             PaperWidthChange(Sender: TObject);
    procedure             MarginLeftChange(Sender: TObject);
    procedure             MarginRightChange(Sender: TObject);
    procedure             PicturesSelectCell(Sender: TObject;
                                             ACol, ARow: Integer;
                                             var CanSelect: Boolean);
    procedure             PicturesClick(Sender: TObject);
    procedure             BtnFormBkgndClick(Sender: TObject);
    procedure             cbFormBackgroundClick(Sender: TObject);
    procedure             OrientationClick(Sender: TObject);
    procedure             PicturesExit(Sender: TObject);
    procedure             cbDayBackgroundClick(Sender: TObject);
    procedure             BtnDayBkGndClick(Sender: TObject);
    procedure             FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure             SetDirty(Sender: TObject);
    procedure             PicturesKeyPress(Sender: TObject; var Key: Char);
    procedure             PicturesMouseUp(Sender: TObject; Button: TMouseButton;
                                          Shift: TShiftState; X, Y: Integer);
  private
    IsDirty             : Boolean;
    LastMonat           : Integer;
    procedure             EnableDayMonth(MakeEnabled : Boolean);
    procedure             FontDialogApplyClick;
    procedure             NumberCheck(Number : tEdit; max : Real; var r : Real);
    procedure             UpDownClick(Edit: TEdit; Button: TUDBtnType);
    procedure             MaxVal3(Edit, Max, Delta : tEdit);
    procedure             MaxVal4(Edit, Max : tEdit; Divisor, Delta : Real);
    procedure             MaxVal2(Edit : tEdit; Max : Real);
    procedure             GetMaxSizes;
    procedure             LayoutCheck;
    procedure             PunchesCheck;
    procedure             SaveEineSeite;
    procedure             SaveRingbuch;
    procedure             SaveMonat;
    procedure             SaveBilder;
    procedure             LoadEineSeite;
    procedure             LoadMonat;
    procedure             LoadRingbuch;
    procedure             LoadBilder;
    function              GetElement  : tFontElement;
    procedure             LoadError;
    procedure             SaveError;
    function              ValidLayout : Boolean;
    procedure             LoadFormBkGnd(IniFile : tIniFile; Formular : tFormular);
    procedure             EnablePictureGroup(MakeEnabled  : Boolean);
    procedure             EnableColumns(MakeEnabled       : Boolean);
    procedure             EnablePaperHeight(MakeEnabled, MustShowHint : Boolean);
    procedure             EnablePaperWidth(MakeEnabled, MustShowHint  : Boolean);
    procedure             EnableOrientation(MakeEnabled   : Boolean);
    procedure             EnablePunching(MakeEnabled      : Boolean);
    procedure             EnableMarginTop(MakeEnabled     : Boolean);
    procedure             EnableMarginLeft(MakeEnabled    : Boolean);
    procedure             EnableMarginRight(MakeEnabled   : Boolean);
    procedure             EnableMarginBottom(MakeEnabled  : Boolean);
    procedure             EnableFooter(MakeEnabled        : Boolean);
    procedure             EnableDayBackground(MakeEnabled : Boolean);
    procedure             EnableUpDown(Lbl : tLabel; Edit : tEdit;
                                       UpDown     : tUpDown;
                                       MakeEnabled, MustShowHint,
                                       DoPunching : Boolean);
    procedure             SaveBkGndMode(IniFile : tIniFile;
                                        Formular : tFormular);
    procedure             EnableFormBkgnd;
    procedure             SelectPicture(var FileName : String);
  public
    FontData            : Array[eJahr .. eTermin, Boolean] of
                            Record
                              Name    : String;
                              Charset : tFontCharSet;
                              Size    : Integer;
                              Color   : tColor;
                              Style   : tFontStyles;
                            end;
    Bilder              : Array[1 .. 12] of String;
    BlattHintergrund,
    TagHintergrund      : String;
    PrinterLogX,
    PrinterLogY,
    PrinterResX,
    PrinterResY         : Integer;
    HoeheMax,
    BreiteMax           : Real;
    procedure             Load;
  end;

var
  LayoutForm  : TLayoutForm;
  ElementName : Array[tFontElement] of String;

implementation

{$R *.dfm}

uses main,
     Preview;

procedure TLayoutForm.SaveError;
  begin
    case SysLocale.PriLangID of
      Lang_German : MsgBox('kann Layout nicht speichern', Caption, mb_IconHand);
    else
      MsgBox('cannot save layout', Caption, mb_IconHand);
    end;
  end;

procedure TLayoutForm.LoadError;
  begin
    case SysLocale.PriLangID of
      Lang_German : MsgBox('kann Layout nicht laden', Caption, mb_IconHand);
    else
      MsgBox('cannot load layout', Caption, mb_IconHand);
    end;
  end;

function TLayoutForm.ValidLayout : Boolean;
  var    i       : Integer;
         Error   : Boolean;
         s1, s2  : String;
         IniFile : tIniFile;
  begin
    Error := False;
    if not Layout.ReadOnly Then
      begin
        try
          IniFile := tIniFile.Create(IniName);
        except
          IniFile := Nil;
        end;
        if IniFile <> Nil then
          try
            if Layout.Text = '' Then
              begin
                Error := True;
                case SysLocale.PriLangID of
                  Lang_German:
                    MsgBox('Bitte geben Sie eine Bezeichnung ein!',
                           Caption, mb_IconExclamation);
                else
                  MsgBox('Please enter a description!',
                         Caption, mb_IconExclamation);
                end;
              end
            else
              begin
                i := 0;
                Repeat
                  s1 := IntToStr(i);
                  s2 := IniFile.ReadString(IniLayouts, s1, '');
                  if s2 = Layout.Text then
                    begin
                      Error := True;
                      s2    := '';
                    end;
                  inc(i);
                until s2 = '';
                if Error then
                  case SysLocale.PriLangID of
                    Lang_German:
                      MsgBox('Diese Bezeichnung ist bereits vergeben',
                             Caption, mb_IconExclamation);
                  else
                    MsgBox('This description is already in use',
                           Caption, mb_IconExclamation);
                  end
                else
                  begin
                    Error := not IsIni(Layout.Text);
                    if Error then
                      MsgBox(Layout.Hint, Caption, mb_IconExclamation);
                  end;
              end;
          finally
           IniFile.Free;
          end;
      end;
    If Error then
      Layout.SetFocus;
    ValidLayout := Not Error;
  end;

procedure TLayoutForm.BtnOkClick(Sender: TObject);
  begin
    if ValidLayout then
      begin
        case tFormular(MainForm.Formular.ItemIndex) of
          FormularEineSeite:
            SaveEineSeite;
          FormularMonat:
            SaveMonat;
          FormularRingbuch:
            SaveRingbuch;
          FormularBilder:
            SaveBilder;
        end;
        IsDirty := False;
        if not Layout.ReadOnly Then
          MainForm.PublicSaveLayouts;
        Close;
      end;
  end;

procedure TLayoutForm.SaveRingbuch;
  var     e       : tFontElement;
          IniFile : tIniFile;
  begin
    try
      IniFile := tIniFile.Create(IniName);
    except
      IniFile := Nil;
    end;
    if IniFile = Nil then
      SaveError
    else
      begin
        try
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniFontName,
                               FontData[eArbeitsTag, False].Name);
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniFontCharset,
                               FontData[eArbeitsTag, False].Charset);
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniFontElement[eTermin] + ' ' + IniFontName,
                               FontData[eTermin, False].Name);
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniFontElement[eTermin] + ' ' + IniFontCharset,
                               FontData[eTermin, False].Charset);
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniFontSize,
                               FontData[eArbeitsTag, False].Size);
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniFontElement[eTermin] + ' ' + IniFontSize,
                               FontData[eTermin, False].Size);
          for e := eArbeitsTag to eTermin do
            begin
              IniFile.WriteString(IniLayout + ' ' + Layout.Text,
                                  FormularName(FormularRingbuch, Lang_English)
                                    + ' ' + IniFontElement[e] + ' ' + IniFontColor,
                                  '0x' + IntToHex(FontData[e, False].Color, 6));
              IniFile.WriteString(IniLayout + ' ' + Layout.Text,
                                  FormularName(FormularRingbuch, Lang_English)
                                    + ' ' + IniFontElement[e] + ' ' + IniFontStyle,
                                  FontStyleString(FontData[e, False].Style));
            end;
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniCheckSizes,
                               ord(CheckSizes.Checked));
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniHeight,
                               PaperHeight.Text);
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniWidth,
                               PaperWidth.Text);
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniMarginLeft,
                               MarginLeft.Text);
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniPunches,
                               Punches.Position);
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniPunch1,
                               Punch1.Text);
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniPunch2,
                               Punch2.Text);
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniPunch3,
                               Punch3.Text);
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniFormBackground, BlattHintergrund);
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniDayMonthFormat, DayMonthFormat.Text);
          if rbLeft.Checked then
            IniFile.WriteString(IniLayout + ' ' + Layout.Text,
                                FormularName(FormularRingbuch, Lang_English)
                                  + ' ' + IniDayMonthAlign, IniLeft)
          else
            IniFile.WriteString(IniLayout + ' ' + Layout.Text,
                                FormularName(FormularRingbuch, Lang_English)
                                  + ' ' + IniDayMonthAlign, IniRight);
          SaveBkGndMode(IniFile, FormularRingbuch);
        except
          SaveError;
        end;
        IniFile.Free;
      end;
  end;

procedure TLayoutForm.Load;
  begin
    Caption := FormularName(tFormular(MainForm.Formular.ItemIndex),
                            SysLocale.PriLangID) + ' ' + 'Layout';
    PaperHeight.Text := '';
    PaperWidth.Text  := '';
    if Layout.Text = '' Then
      begin
        Layout.Color    := clWindow;
        Layout.ReadOnly := False;
        Layout.Tabstop  := True;
        Layout.ShowHint := True;
        ActiveControl   := Layout;
      end
    else
      begin
        Layout.Color    := clBtnFace;
        Layout.ReadOnly := True;
        Layout.Tabstop  := False;
        Layout.ShowHint := False;
        ActiveControl   := Element;
      end;
    case tFormular(MainForm.Formular.ItemIndex) of
      FormularEineSeite:
        LoadEineSeite;
      FormularMonat:
        LoadMonat;
      FormularRingbuch:
        LoadRingbuch;
      FormularBilder:
        LoadBilder;
    end;
    LayoutCheck;
    IsDirty := False;
  end;

procedure TLayoutForm.EnableColumns(MakeEnabled : Boolean);
  begin
    EnableUpDown(LabelColumns, ColumnsEdit, Columns, MakeEnabled, MakeEnabled,
                 False);
  end;

procedure TLayoutForm.EnablePunching(MakeEnabled : Boolean);
  var c : tColor;
      b : Boolean;
  begin
    EnableUpDown(LabelPunches, PunchesEdit, Punches,
                 MakeEnabled, MakeEnabled, false);
    b := MakeEnabled and (Punches.Position = 2);
    EnableUpDown(LabelPunch1,  Punch1,      UpDownPunch1,
                 b,           b,            true);
    b := MakeEnabled and (Punches.Position = 4);
    EnableUpDown(LabelPunch2,  Punch2,      UpDownPunch2,
                 b,           b,            true);
    b := MakeEnabled and (Punches.Position = 6);
    EnableUpDown(LabelPunch3,  Punch3,      UpDownPunch3,
                 b,           b,            true);
    if MakeEnabled then
      c := clWindowText
    else
      c := clGrayText;
    LabelMid.Font.Color := c;
    Punching.Font.Color := c;
  end;

procedure TLayoutForm.EnableDayMonth(MakeEnabled : Boolean);
  var c : tColor;
  begin
    rbLeft.enabled  := MakeEnabled;
    rbRight.enabled := MakeEnabled;
    if MakeEnabled then
      c := clWindowText
    else
      c := clGrayText;
    DayMonthGroupBox.Font.Color := c;
  end;

procedure TLayoutForm.SaveBkGndMode(IniFile : tIniFile; Formular : tFormular);
  begin
    if rbStretch.checked then
      IniFile.WriteString(IniLayout + ' ' + Layout.Text,
                          FormularName(Formular, Lang_English)
                            + ' ' + IniBackgroundMode,
                          IniBkGndStretch)
    else if rbTile.checked then
      IniFile.WriteString(IniLayout + ' ' + Layout.Text,
                          FormularName(Formular, Lang_English)
                            + ' ' + IniBackgroundMode,
                          IniBkGndTile);
  end;

procedure TLayoutForm.EnableDayBackground(MakeEnabled : Boolean);
  begin
    DayBackground.enabled := MakeEnabled;
    if MakeEnabled then
      if TagHintergrund > '' then
        begin
          EditDayBkGnd.Text       := ExtractFileName(TagHintergrund);
          cbDayBackground.checked := True;
        end
      else
        begin
          EditDayBkGnd.Text       := '';
          cbDayBackground.checked := False;
        end
    else
      begin
        TagHintergrund          := '';
        EditDayBkGnd.Text       := '';
        cbDayBackground.checked := False;
      end;
  end;

function DeleteFormat(var Format: String; SubFormat : String) : Boolean;
  var    p : Integer;
  begin
    p := Pos(SubFormat, UpperCase(Format));
    if p <= 0 then
      DeleteFormat   := False
    else
      begin
        DeleteFormat := True;
        Format       := Copy(Format, 1, p - 1)
                        + Copy(Format, p + length(SubFormat), length(Format));
      end;
  end;

procedure TLayoutForm.LoadRingbuch;
  var     e       : tFontElement;
          t       : String;
          b       : Boolean;
          IniFile : tIniFile;
  begin
    Element.Items.Clear;
    Element.Items.Add(ElementName[eFont]);
    Element.Items.Add(ElementName[eArbeitsTag]);
    Element.Items.Add(ElementName[eSamstag]);
    Element.Items.Add(ElementName[eFeiertag]);
    Element.Items.Add(ElementName[eTermin]);
    Element.ItemIndex                     := 0;
    Weeks.enabled                         := False;
    EnablePictureGroup(False);
    HeadLine.enabled                      := False;
    HolidayList.enabled                   := False;
    IsMergedDate.enabled                  := False;
    IsMergedDate.Checked                  := False;
    EnableOrientation(False);
    DocFrame.enabled                      := False;
    MonthFrame.enabled                    := False;
    EnableColumns(False);
    EnableFooter(False);
    EnablePaperHeight(True, False);
    EnablePaperWidth(True,  False);
    Paper.Font.Color := clWindowText;
    EnableMarginTop(False);
    EnableMarginLeft(True);
    EnableMarginRight(False);
    EnableMarginBottom(False);
    EnableDayBackground(False);
    Margin.Font.Color                     := clWindowText;
    Sizes.Font.Color                      := clWindowText;
    EnablePunching(True);
    CheckSizes.Checked                    := True;
    Orientation.ItemIndex                 := ord(poLandscape);
    FontData[eArbeitsTag, False].Name     := 'Arial';
    FontData[eArbeitsTag, False].Charset  := DEFAULT_CHARSET;
    FontData[eArbeitsTag, False].Size     := 10;
    FontData[eArbeitsTag, False].Color    := clBlack;
    FontData[eArbeitsTag, False].Style    := [];
    FontData[eSamstag,  False].Color      := clAqua;
    FontData[eSamstag,  False].Style      := [];
    FontData[eFeiertag, False].Color      := clRed;
    FontData[eFeiertag, False].Style      := [fsItalic];
    FontData[eTermin, False].Name         := 'Arial';
    FontData[eTermin, False].Charset      := DEFAULT_CHARSET;
    FontData[eTermin, False].Size         := 10;
    FontData[eTermin, False].Color        := clAqua;
    FontData[eTermin, False].Style        := [fsItalic];
    PaperHeight.Text                      := '148.5';
    PaperWidth.Text                       := '105.0';
    str(MMTextRand / 1. : 0 : 1, t);
    MarginLeft.Text                       := t;
    Punches.Position                      :=    4;
    Punch1.Text                           :=  '23.5';
    Punch2.Text                           :=  '50.0';
    Punch3.Text                           :=  '';
    EnableDayMonth(True);
    t                                     := GetShortDateFormat;
    repeat
      b := True;
      if DeleteFormat(t, 'DDDD') then
        b := False
      else
        if DeleteFormat(t, 'DDD') then
          b := False
        else
          if DeleteFormat(t, 'YYYY') then
            b := False
          else
            if DeleteFormat(t, 'YY') then
              b := False;
    until b;
    if t > '' then
      if t[length(t)] <> '.' then
        t := Copy(t, 1, length(t) - 1);
    DayMonthFormat.Text := t;
    rbLeft.Checked      := true;
    GetMaxSizes;
    try
      IniFile := tIniFile.Create(IniName);
    except
      IniFile := Nil;
    end;
    if IniFile = Nil then
      LoadError
    else
      begin
        try
          FontData[eArbeitsTag, False].Name :=
            IniFile.ReadString(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniFontName,
                               FontData[eArbeitsTag, False].Name);
          FontData[eArbeitsTag, False].CharSet :=
            IniFile.ReadInteger(IniLayout + ' ' + Layout.Text,
                                FormularName(FormularRingbuch, Lang_English)
                                  + ' ' + IniFontCharSet,
                                FontData[eArbeitsTag, False].Charset);
          FontData[eTermin, False].Name :=
            IniFile.ReadString(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniFontElement[eTermin] + ' ' + IniFontName,
                               FontData[eTermin, False].Name);
          FontData[eTermin, False].Charset :=
            IniFile.ReadInteger(IniLayout + ' ' + Layout.Text,
                                FormularName(FormularRingbuch, Lang_English)
                                  + ' ' + IniFontElement[eTermin]
                                  + ' ' + IniFontCharset,
                                FontData[eTermin, False].Charset);
          FontData[eArbeitsTag, False].Size :=
            IniFile.ReadInteger(IniLayout + ' ' + Layout.Text,
                                FormularName(FormularRingbuch, Lang_English)
                                  + ' ' + IniFontSize,
                                FontData[eArbeitsTag, False].Size);
          FontData[eTermin, False].Size :=
            IniFile.ReadInteger(IniLayout + ' ' + Layout.Text,
                                FormularName(FormularRingbuch, Lang_English)
                                  + ' ' + IniFontElement[eTermin]
                                  + ' ' + IniFontSize,
                                FontData[eTermin, False].Size);
          for e := eArbeitsTag to eTermin do
            begin
              FontData[e, False].Color :=
                IniFile.ReadInteger(IniLayout + ' ' + Layout.Text,
                                    FormularName(FormularRingbuch, Lang_English)
                                      + ' ' + IniFontElement[e]
                                      + ' ' + IniFontColor,
                                    FontData[e, False].Color);
              t := IniFile.ReadString(IniLayout + ' ' + Layout.Text,
                                      FormularName(FormularRingbuch, Lang_English)
                                        + ' ' + IniFontElement[e]
                                        + ' ' + IniFontStyle,
                                      IniNotFound);
              if t <> IniNotFound then
                FontData[e, False].Style := FontStyleSet(t);
            end;
          CheckSizes.Checked :=
            Boolean(IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularRingbuch, Lang_English) + ' ' + IniCheckSizes,
              ord(CheckSizes.Checked)));
          PaperHeight.Text :=
            IniFile.ReadString(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniHeight,
                               PaperHeight.Text);
          PaperWidth.Text :=
            IniFile.ReadString(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniWidth,
                               PaperWidth.Text);
          MarginLeft.Text :=
            IniFile.ReadString(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniMarginLeft,
                               MarginLeft.Text);
          Punches.Position :=
            IniFile.ReadInteger(IniLayout + ' ' + Layout.Text,
                                FormularName(FormularRingbuch, Lang_English)
                                  + ' ' + IniPunches,
                                Punches.Position);
          Punch1.Text :=
            IniFile.ReadString(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniPunch1,
                               Punch1.Text);
          Punch2.Text :=
            IniFile.ReadString(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniPunch2,
                               Punch2.Text);
          Punch3.Text :=
            IniFile.ReadString(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniPunch3,
                               Punch3.Text);
          LoadFormBkGnd(IniFile, FormularRingbuch);
          DayMonthFormat.Text :=
            IniFile.ReadString(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularRingbuch, Lang_English)
                                 + ' ' + IniDayMonthFormat,
                               DayMonthFormat.Text);
          t := IniFile.ReadString(IniLayout + ' ' + Layout.Text,
                                  FormularName(FormularRingbuch, Lang_English)
                                    + ' ' + IniDayMonthAlign,
                                  IniLeft);
          if t = IniLeft then
            rbLeft.Checked  := True
          else
            rbRight.Checked := True;
        except
          LoadError;
        end;
        IniFile.Free;
      end;
  end;

function TLayoutForm.GetElement : tFontElement;
  var    e     : tFontElement;
         Found : Boolean;
  begin
    Found      := False;
    GetElement := eFont;
    e          := low(tFontElement);
    while not found do
      if ElementName[e] = Element.Items[Element.ItemIndex] then
        begin
          GetElement := e;
          Found      := True;
        end
      else
        if e < High(tFontElement) then
          inc(e)
        else
          Found := True;
  end;

procedure TLayoutForm.BtnAdjustClick(Sender: TObject);
  Var     OldFont : tFont;
  begin
    OldFont := tFont.Create;
    if OldFont <> Nil then
      begin
        with FontDialog do
          begin
            Font.Name       := FontData[eArbeitsTag, False].Name;
            Font.Charset    := FontData[eArbeitsTag, False].Charset;
            Font.Size       := FontData[eArbeitsTag, False].Size;
            case GetElement of
              eFont:
                begin
                  Font.Color := FontData[eArbeitsTag, false].Color;
                  Font.Style := FontData[eArbeitsTag, false].Style;
                  Options    := Options - [fdEffects, fdLimitSize];
                end;
              eJahr:
                begin
                  Font.Size  := FontData[eJahr, false].Size;
                  Font.Color := FontData[eJahr, false].Color;
                  Font.Style := FontData[eJahr, false].Style;
                  Options    := Options + [fdEffects] - [fdLimitSize];
                end;
              eArbeitsTag:
                begin
                  Font.Color := FontData[eArbeitsTag, IsMergedDate.Checked].Color;
                  Font.Style := FontData[eArbeitsTag, IsMergedDate.Checked].Style;
                  Options    := Options + [fdEffects, fdLimitSize];
                end;
              eSamstag:
                begin
                  Font.Color := FontData[eSamstag, IsMergedDate.Checked].Color;
                  Font.Style := FontData[eSamstag, IsMergedDate.Checked].Style;
                  Options    := Options + [fdEffects, fdLimitSize];
                end;
              eFeiertag:
                begin
                  Font.Color := FontData[eFeiertag, IsMergedDate.Checked].Color;
                  Font.Style := FontData[eFeiertag, IsMergedDate.Checked].Style;
                  Options    := Options + [fdEffects, fdLimitSize];
                end;
              eFeiertagListe:
                begin
                  Font.Color := FontData[eFeiertagListe, false].Color;
                  Font.Style := FontData[eFeiertagListe, false].Style;
                  Font.Size  := FontData[eFeiertagListe, false].Size;
                  Options    := Options + [fdEffects] - [fdLimitSize];
                end;
              eTermin:
                begin
                  Font.Name    := FontData[eTermin, False].Name;
                  Font.Charset := FontData[eTermin, False].Charset;
                  Font.Size    := FontData[eTermin, False].Size;
                  Font.Color   := FontData[eTermin, False].Color;
                  Font.Style   := FontData[eTermin, False].Style;
                  Options      := Options + [fdEffects] - [fdLimitSize];
                end;
            end;
            if fdLimitSize in Options then
              begin
                MinFontSize := Font.Size;
                MaxFontSize := Font.Size;
              end
            else
              begin
                MinFontSize := 0;
                MaxFontSize := 0;
              end;
            OldFont.Name    := Font.Name;
            OldFont.Charset := Font.Charset;
            OldFont.Size    := Font.Size;
            OldFont.Color   := Font.Color;
            OldFont.Style   := Font.Style;
            if Execute then
              begin
                case GetElement of
                  eFont:
                    begin
                      FontData[eArbeitsTag, False].Name    := Font.Name;
                      FontData[eArbeitsTag, False].Charset := Font.Charset;
                      FontData[eArbeitsTag, False].Size    := Font.Size;
                    end;
                  eJahr:
                    begin
                      FontData[eJahr, false].Size  := Font.Size;
                      FontData[eJahr, false].Color := Font.Color;
                      FontData[eJahr, false].Style := Font.Style;
                    end;
                  eArbeitsTag:
                    begin
                      FontData[eArbeitsTag, IsMergedDate.Checked].Color :=
                        Font.Color;
                      FontData[eArbeitsTag, IsMergedDate.Checked].Style :=
                        Font.Style;
                    end;
                  eSamstag:
                    begin
                      FontData[eSamstag, IsMergedDate.Checked].Color :=
                        Font.Color;
                      FontData[eSamstag, IsMergedDate.Checked].Style :=
                        Font.Style;
                    end;
                  eFeiertag:
                    begin
                      FontData[eFeiertag, IsMergedDate.Checked].Color := Font.Color;
                      FontData[eFeiertag, IsMergedDate.Checked].Style := Font.Style;
                    end;
                  eFeiertagListe:
                    begin
                      FontData[eFeiertagListe, false].Color := Font.Color;
                      FontData[eFeiertagListe, false].Size  := Font.Size;
                      FontData[eFeiertagListe, false].Style := Font.Style;
                    end;
                  eTermin:
                    begin
                      FontData[eTermin, False].Name    := Font.Name;
                      FontData[eTermin, False].Charset := Font.Charset;
                      FontData[eTermin, False].Size    := Font.Size;
                      FontData[eTermin, False].Color   := Font.Color;
                      FontData[eTermin, False].Style   := Font.Style;
                    end;
                end;
                IsDirty := True;
              end
            else
              Begin
                Font.Name    := OldFont.Name;
                Font.Charset := OldFont.Charset;
                Font.Size    := OldFont.Size;
                Font.Color   := OldFont.Color;
                Font.Style   := OldFont.Style;
                {$ifdef fpc}
                  FontDialogApply(Sender);
                {$else}
                  FontDialogApply(Sender, Handle);
                {$endif}
                MainForm.PaintPreview(True);
              End;
          end;
        OldFont.Free;
      end;
  end;

function OpenFilter(EmfDatei, WmfDatei, AlleBilder, Bild : String) : String;
  var    s, t : String;
         o, p : tOpenIndex;
  begin
    t := '';
    for o := Low(tOpenIndex) to High(tOpenIndex) do
      case o of
        OpenIndexBmp :
            t := t + Trenn
                   + UpperCase(ExtGraphicExtension(tBitmap,    1))
                   + Bild
                   + LowerCase(ExtGraphicExtension(tBitmap,    1));
        OpenIndexJpg :
            t := t + Trenn
                   + UpperCase(ExtGraphicExtension(tJpegImage, 1))
                   + Bild
                   + LowerCase(ExtGraphicExtension(tJpegImage, 1));
        OpenIndexJpeg :
            t := t + Trenn
                   + UpperCase(ExtGraphicExtension(tJpegImage, 2))
                   + Bild
                   + LowerCase(ExtGraphicExtension(tJpegImage, 2));
        OpenIndexGif :
            t := t + Trenn
                   + UpperCase(ExtGraphicExtension(tGifImage,   1))
                   + Bild
                   + LowerCase(ExtGraphicExtension(tGifImage,   1));
        OpenIndexIco :
            t := t + Trenn + 'Icon' + Trenn + Stern
                   + LowerCase(ExtGraphicExtension(tIcon,      1));
        {$ifdef fpc}
          OpenIndexPng :
            t := t + Trenn
                   + UpperCase(ExtGraphicExtension(tPortableNetworkGraphic, 1))
                   + Bild
                   + LowerCase(ExtGraphicExtension(tPortableNetworkGraphic, 1));
        {$else}
          OpenIndexPcx :
            t := t + Trenn
                   + UpperCase(ExtGraphicExtension(tsgPcxImage, 1))
                   + Bild
                   + LowerCase(ExtGraphicExtension(tsgPcxImage, 1));
          OpenIndexEmf :
            t := t + Trenn + EmfDatei + Trenn + Stern
                   + LowerCase(ExtGraphicExtension(tMetaFile,   1));
          OpenIndexWmf :
            t := t + Trenn + WmfDatei + Trenn + Stern
                   + LowerCase(ExtGraphicExtension(tMetaFile,   2));
        {$endif}
        {$ifdef DelphiPng}
          OpenIndexPng :
            t := t + Trenn
                   + UpperCase(ExtGraphicExtension(tPngObject,  1))
                   + Bild
                   + LowerCase(ExtGraphicExtension(tPngObject,  1));
        {$endif}
      else
        begin
          t := t + Trenn + AlleBilder + Trenn;
          s := '';
          for p := Succ(Low(tOpenIndex)) to High(tOpenIndex) do
            case p of
              OpenIndexBmp   : s := s + AllStern
                  + LowerCase(ExtGraphicExtension(tBitmap,     1));
              OpenIndexJpg   : s := s + AllStern
                  + LowerCase(ExtGraphicExtension(tJpegImage,  1));
              OpenIndexJpeg  : s := s + AllStern
                  + LowerCase(ExtGraphicExtension(tJpegImage,  2));
              OpenIndexGif   : s := s + AllStern
                  + LowerCase(ExtGraphicExtension(tGifImage,   1));
              OpenIndexIco   : s := s + AllStern
                  + LowerCase(ExtGraphicExtension(tIcon,       1));
              {$ifdef fpc}
                OpenIndexPng : s := s + AllStern
                  + LowerCase(ExtGraphicExtension(tPortableNetworkGraphic, 1));
              {$else}
                OpenIndexPcx : s := s + AllStern
                  + LowerCase(ExtGraphicExtension(tsgPcxImage, 1));
                OpenIndexEmf : s := s + AllStern
                  + LowerCase(ExtGraphicExtension(tMetaFile,   1));
                OpenIndexWmf : s := s + AllStern
                  + LowerCase(ExtGraphicExtension(tMetaFile,   2));
              {$endif}
              {$ifdef DelphiPng}
                OpenIndexPng : s := s + AllStern
                  + LowerCase(ExtGraphicExtension(tPngObject,  1));
              {$endif}
            end;
          t := t + Copy(s, 2, length(s));
        end;
      end;
    {$ifdef fpc}
      if EmfDatei = '' then;
      if WmfDatei = '' then;
    {$endif}
    OpenFilter := Copy(t, 2, length(t));
  end;

procedure TLayoutForm.FormCreate(Sender: TObject);
  var     i, j, w : Integer;
  begin
    IsDirty := False;
    LoadFormFromIni(Self);
    case SysLocale.PriLangID of
      Lang_German:
        begin
          ElementName[eFont]          := CP1252toSystem('Font, &Gre');
          ElementName[eJahr]          :=
            CP1252toSystem('&Jahr: Farbe, Stil, Gre');
          ElementName[eFeiertagListe] := 'Feiertag &Liste: F., S., Gr.';
          ElementName[eArbeitsTag]    := 'Arbeitstag: Farbe, Stil';
          ElementName[eSamstag]       := 'Samstag: Farbe, Stil';
          ElementName[eFeiertag]      := '&Feiertag: Farbe, Stil';
          ElementName[eTermin]        := 'Ter&min';
          Pictures.Cells[0,0]         := 'Monat';
          Pictures.Cells[1,0]         := 'Bild';
          Pictures.Cells[2,0]         := 'Kommentar';
          DayMonthFormat.Hint := 'd = Tag, m = Monat, d = 1..31, '
                                 + 'dd = 01..31, m = 1..12, mm = 01..12, '
                                 + 'mmm = ' + GetShortMonthName(1) + '..'
                                 + GetShortMonthName(12) + ', mmmm = '
                                 + GetLongMonthName(1) + '..'
                                 + GetLongMonthName(12);
          OpenDialog.Filter :=
            OpenFilter('Erweiterter Metafile', 'Metafile', 'Alle Bilder', Bild);
        end;
    else
      begin
        ElementName[eFont]                  := 'Font, &size';
        ElementName[eJahr]                  := 'Year: Color, style, si&ze';
        ElementName[eFeiertagListe]         := 'Holiday &list: Col., st., size';
        ElementName[eArbeitsTag]            := 'Working day: Color, style';
        ElementName[eSamstag]               := 'Saturday: Color, style';
        ElementName[eFeiertag]              := '&Holiday: Color, style';
        ElementName[eTermin]                := 'Personal &Date';
        PictureGroup.Caption                := '&Pictures';
        Pictures.Cells[0,0]                 := 'Month';
        Pictures.Cells[1,0]                 := 'Picture';
        Pictures.Cells[2,0]                 := 'Comment';
        BtnAuswahl.Caption                  := 's&elect picture';
        BtnDelete.Caption                   := '&delete picture';
        LabelLayoutName.Caption             := 'Layout name';
        Layout.hint                         := 'only letters, digits, blanks '
                                               + 'and _ allowed';
        Elements.Caption                    := 'Print elements';
        IsMergedDate.Caption                := 'Da&y is merged date';
        BtnAdjust.Caption                   := '&Adjust print element';
        Headline.Caption                    := 'H&eadline';
        HolidayList.Caption                 := 'Hol&iday list';
        CheckSizes.Caption                  := 'Check font sizes';
        Weeks.Caption                       := 'Print wee&ks';
        Orientation.Caption                 := 'Orientation';
        Orientation.Items[ord(poLandscape)] := 'Lan&dscape';
        Orientation.Items[ord(poPortrait)]  := 'Po&rtrait';
        DocFrame.Caption                    := '&Border of document';
        MonthFrame.Caption                  := 'Frame each &month';
        LabelColumns.Caption                := 'Columns';
        LabelFooter.Caption                 := 'Footer';
        Sizes.Caption                       := 'Sizes in mm';
        Paper.Caption                       := '&Form';
        LabelHeight.Caption                 := 'Height';
        PaperHeight.Hint                    := 'contains margin!';
        LabelWidth.Caption                  := 'Width';
        PaperWidth.Hint                     := PaperHeight.Hint;
        Margin.Caption                      := 'Mar&gin';
        LabelTop.Caption                    := 'Top';
        MarginTop.Hint                      := 'reduces useable form size!';
        LabelLeft.Caption                   := 'Left';
        MarginLeft.Hint                     := MarginTop.Hint;
        LabelRight.Caption                  := 'Right';
        MarginRight.Hint                    := MarginTop.Hint;
        LabelBottom.Caption                 := 'Bottom';
        MarginBottom.Hint                   := MarginTop.Hint;
        Punching.Caption                    := 'P&unching';
        LabelPunches.Caption                := 'punches';
        LabelMid.Caption                    := 'measured from the middle';
        LabelPunch1.Caption                 := '1st punch';
        LabelPunch2.Caption                 := '2nd punch';
        LabelPunch3.Caption                 := '3rd punch';
        BtnTest.Caption                     := 'Pre&view';
        BtnCancel.Caption                   := '&Cancel';
        FormBackground.Caption              := 'Form Background';
        DayBackground.Caption               := 'Day Background';
        rbStretch.Caption                   := 'Stretch';
        rbTile.Caption                      := 'Tile';
        DayMonthGroupBox.Caption            := 'Day/month format';
        rbLeft.Caption                      := 'left align';
        rbRight.Caption                     := 'right align';
        DayMonthFormat.Hint := 'd = Day, m = Month, d = 1..31, '
                               + 'dd = 01..31, m = 1..12, mm = 01..12, '
                               + 'mmm = ' + GetShortMonthName(1) + '..'
                               + GetShortMonthName(12) + ', mmmm = '
                               + GetLongMonthName(1) + '..'
                               + GetLongMonthName(12);
        OpenDialog.Filter                   :=
          OpenFilter('Enhanced Metafile', 'Metafile', 'All Pictures', Picture);
      end;
    end;
    w := Canvas.TextWidth(Pictures.Cells[0, 0]);
    for i := 1 to 12 do
      begin
        Pictures.Cells[0, i] := GetLongMonthName(i);
        j := Canvas.TextWidth(Pictures.Cells[0, i]);
        if j > w then
          w := j;
      end;
    inc(w, Pictures.GridLineWidth + 6);
    j := w - Pictures.ColWidths[0];
    Pictures.ColWidths[0] := w;
    Pictures.ColWidths[2] := Pictures.Width
                             - Pictures.ColWidths[0]
                             - Pictures.ColWidths[1]
                             - 3 * Pictures.GridLineWidth - 3;
    { The non-associated UpDowns look different in Linux
      without the following lines }
    UpDownHeight.Height := Columns.Height;
    UpDownWidth.Height  := Columns.Height;
    UpDownTop.Height    := Columns.Height;
    UpDownLeft.Height   := Columns.Height;
    UpDownRight.Height  := Columns.Height;
    UpDownBottom.Height := Columns.Height;
    UpDownPunch1.Height := Columns.Height;
    UpDownPunch2.Height := Columns.Height;
    UpDownPunch3.Height := Columns.Height;
  end;

procedure TLayoutForm.BtnCancelClick(Sender: TObject);
  begin
    IsDirty := False;
    Close;
  end;

procedure TLayoutForm.FormClose(Sender: TObject; var CloseAction: TCloseAction);
  begin
    if CloseAction = caFree then;
    SaveFormToIni(Self);
    MainForm.ClosePreview;
  end;

procedure TLayoutForm.NumberCheck(Number : tEdit; max : Real; var r : Real);
  var     Error : Integer;
          s     : String;
  begin
    Val(Number.Text, r, Error);
    if Error <> 0 then
      r := 0
    else
      if r < 0 then
        r := 0
      else
        if r > Max then
          r := Max;
    str(r : 0 : 1, s);
    Number.Text := s;
  end;

procedure TLayoutForm.GetMaxSizes;
  var i : Integer;
  begin
    Printer.Orientation := tPrinterOrientation(Orientation.ItemIndex);
    {$ifdef fpc}
      PrinterResX       := Printer.PageWidth;
      PrinterResY       := Printer.PageHeight;
      PrinterLogX       := Printer.XDPI;
      PrinterLogY       := Printer.YDPI;
    {$else}
      PrinterResX       := GetDeviceCaps(Printer.Handle, HorzRes);
      PrinterResY       := GetDeviceCaps(Printer.Handle, VertRes);
      PrinterLogX       := GetDeviceCaps(Printer.Handle, LogPixelsX);
      PrinterLogY       := GetDeviceCaps(Printer.Handle, LogPixelsY);
    {$endif}
    if Orientation.ItemIndex = ord(poPortrait) then
      begin
        if PrinterResX > PrinterResY then
          begin
            i           := PrinterResX;
            PrinterResX := PrinterResY;
            PrinterResy := i;
            i           := PrinterLogX;
            PrinterLogX := PrinterLogY;
            PrinterLogy := i;
          end
      end
    else
      if PrinterResX < PrinterResY then
        begin
          i           := PrinterResX;
          PrinterResX := PrinterResY;
          PrinterResY := i;
        end
      else
        begin
          i           := PrinterLogX;
          PrinterLogX := PrinterLogY;
          PrinterLogy := i;
        end;
    HoeheMax  := int(PrinterResY / PrinterLogY * 254) / 10;
    BreiteMax := int(PrinterResX / PrinterLogX * 254) / 10;
  end;

procedure TLayoutForm.LayoutCheck;
  var     Ok      : Boolean;
          i       : Integer;
          s1      : String;
          h, w, r : Real;
  begin
    Screen.Cursor := crHourGlass;
    Ok            := True;
    h             := 0;
    w             := 0;
    r             := 0;
    val(PaperHeight.Text, h, i);
    if h = 0 then
      begin
        str(HoeheMax : 1 : 1, s1);
        PaperHeight.Text := s1;
      end;
    val(PaperWidth.Text, w, i);
    if w = 0 then
      begin
        str(BreiteMax : 1 : 1, s1);
        PaperWidth.Text := s1;
      end;
    i := i;
    MainForm.ResizePreview;
    case tFormular(MainForm.Formular.ItemIndex) of
      FormularEineSeite,
      FormularRingbuch:
        begin
          NumberCheck(PaperHeight, HoeheMax,  h);
          NumberCheck(PaperWidth,  BreiteMax, w);
        end;
    end;
    case tFormular(MainForm.Formular.ItemIndex) of
      FormularEineSeite:
        begin
          NumberCheck(MarginTop,   h, r);
          NumberCheck(MarginLeft,  w, r);
          NumberCheck(MarginRight, w, r);
        end;
      FormularBilder:
        begin
          NumberCheck(MarginTop,    HoeheMax,      h);
          NumberCheck(MarginLeft,   BreiteMax,     w);
          NumberCheck(MarginRight,  BreiteMax - w, r);
          NumberCheck(MarginBottom, HoeheMax  - h, r);
        end;
      FormularRingbuch:
        begin
          NumberCheck(MarginLeft,   BreiteMax,     w);
          r := (h - MMLoch) / 2;
          if r < 0 then
            r := 0;
          PunchesCheck;
        end;
    end;
    MainForm.ShowPreview;
    BtnOk.enabled := Ok;
    Screen.Cursor := crDefault;
  end;

procedure TLayoutForm.BtnTestClick(Sender: TObject);
  begin
    PreviewForm.Show;
    LayoutCheck;
  end;

procedure TLayoutForm.SaveEineSeite;
  var     e       : tFontElement;
          b       : Boolean;
          IniFile : tIniFile;
  begin
    try
      IniFile := tIniFile.Create(IniName);
    except
      IniFile := Nil;
    end;
    if IniFile = Nil then
      SaveError
    else
      begin
        try
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniFontName,
                               FontData[eArbeitsTag, False].Name);
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniFontCharset,
                               FontData[eArbeitsTag, False].Charset);
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniFontSize,
                               FontData[eArbeitsTag, False].Size);
          with FontData[eJahr, False] do
            begin
              IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                                   FormularName(FormularEineSeite, Lang_English)
                                     + ' ' + IniFontElement[eJahr]
                                     + ' ' + IniFontSize,
                                   Size);
              IniFile.WriteString(IniLayout + ' ' + Layout.Text,
                                  FormularName(FormularEineSeite, Lang_English)
                                    + ' ' + IniFontElement[eJahr]
                                    + ' ' + IniFontColor,
                                  '0x' + IntToHex(Color, 6));
              IniFile.WriteString(IniLayout + ' ' + Layout.Text,
                                  FormularName(FormularEineSeite, Lang_English)
                                    + ' ' + IniFontElement[eJahr]
                                    + ' ' + IniFontStyle,
                                  FontStyleString(Style));
            end;
          with FontData[eFeiertagListe, False] do
            begin
              IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                                   FormularName(FormularEineSeite, Lang_English)
                                     + ' ' + IniFontElement[eFeiertagListe]
                                     + ' ' + IniFontSize,
                                   Size);
              IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                                   FormularName(FormularEineSeite, Lang_English)
                                     + ' ' + IniFontElement[eFeiertagListe]
                                     + ' ' + IniFontColor,
                                   '0x' + IntToHex(Color, 6));
              IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                                   FormularName(FormularEineSeite, Lang_English)
                                     + ' ' + IniFontElement[eFeiertagListe]
                                     + ' ' + IniFontStyle,
                                   FontStyleString(Style));
            end;
          for e := eArbeitsTag to eFeiertag do
            for b := False to True do
              with FontData[e, b] do
                begin
                  IniFile.WriteString(IniLayout + ' ' + Layout.Text,
                                      FormularName(FormularEineSeite, Lang_English)
                                        + ' ' + IniFontElement[e] + IniIsDate[b]
                                        + ' ' + IniFontColor,
                                      '0x' + IntToHex(Color, 6));
                  IniFile.WriteString(IniLayout + ' ' + Layout.Text,
                                      FormularName(FormularEineSeite, Lang_English)
                                        + ' ' + IniFontElement[e] + IniIsDate[b]
                                        + ' ' + IniFontStyle,
                                      FontStyleString(Style));
                end;
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniCheckSizes,
                               ord(CheckSizes.Checked));
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniWeeks,
                               ord(Weeks.Checked));
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniHeadLine,
                               ord(HeadLine.Checked));
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniHolidayList,
                               ord(HolidayList.Checked));
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniOrientation,
                               Orientation.ItemIndex);
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniDocFrame,
                               ord(DocFrame.Checked));
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniMonthFrame,
                               ord(MonthFrame.Checked));
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniColumns,
                               Columns.Position);
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniFooter,
                               '"' + Footer.Text + '"');
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniHeight,
                               PaperHeight.Text);
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniWidth,
                               PaperWidth.Text);
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniMarginTop,
                               MarginTop.Text);
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniMarginLeft,
                               MarginLeft.Text);
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniMarginRight,
                               MarginRight.Text);
          IniFile.WriteString(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniFormBackground, BlattHintergrund);
          SaveBkGndMode(IniFile, FormularEineSeite);
        except
          SaveError;
        end;
        IniFile.Free;
      end;
  end;

procedure TLayoutForm.SaveMonat;
  var     e       : tFontElement;
          b       : Boolean;
          IniFile : tIniFile;
  begin
    try
      IniFile := tIniFile.Create(IniName);
    except
      IniFile := Nil;
    end;
    if IniFile = Nil then
      SaveError
    else
      begin
        try
          IniFile.WriteString( IniLayout + ' ' + Layout.Text,
                               FormularName(FormularMonat, Lang_English)
                                 + ' ' + IniFontName,
                               FontData[eArbeitsTag, False].Name);
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularMonat, Lang_English)
                                 + ' ' + IniFontCharset,
                               FontData[eArbeitsTag, False].Charset);
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularMonat, Lang_English)
                                 + ' ' + IniFontSize,
                               FontData[eArbeitsTag, False].Size);
          for e := eArbeitsTag to eFeiertag do
            for b := False to True do
              with FontData[e, b] do
                begin
                  IniFile.WriteString(IniLayout + ' ' + Layout.Text,
                                      FormularName(FormularMonat, Lang_English)
                                        + ' ' + IniFontElement[e] + IniIsDate[b]
                                        + ' ' + IniFontColor,
                                      '0x' + IntToHex(Color, 6));
                  IniFile.WriteString(IniLayout + ' ' + Layout.Text,
                                      FormularName(FormularMonat, Lang_English)
                                        + ' ' + IniFontElement[e] + IniIsDate[b]
                                        + ' ' + IniFontStyle,
                                      FontStyleString(Style));
                end;
          IniFile.WriteInteger(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularMonat, Lang_English)
                                 + ' ' + IniCheckSizes,
                               ord(CheckSizes.Checked));
          IniFile.WriteString(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularMonat, Lang_English)
                                 + ' ' + IniFormBackground, BlattHintergrund);
          SaveBkGndMode(IniFile, FormularMonat);
        except
          SaveError;
        end;
        IniFile.Free;
      end;
  end;

procedure TLayoutForm.LoadEineSeite;
  var     t       : String;
          b       : Boolean;
          e       : tFontElement;
          IniFile : tIniFile;
  begin
    EnablePictureGroup(False);
    Weeks.enabled                         := True;
    HeadLine.enabled                      := True;
    HolidayList.enabled                   := True;
    IsMergedDate.enabled                  := True;
    IsMergedDate.Checked                  := False;
    EnableOrientation(True);
    DocFrame.enabled                      := True;
    MonthFrame.enabled                    := True;
    EnableColumns(True);
    EnableFooter(True);
    case SysLocale.PriLangID of
      Lang_German:
        Footer.Text := 'erstellt mit ' + CalPrint + ' ' + CopyrightInternet;
    else
      Footer.Text   := 'created with ' + CalPrint + ' ' + CopyrightInternet;
    end;
    EnablePaperHeight(True, True);
    EnablePaperWidth(True,  True);
    Paper.Font.Color := clWindowText;
    EnableMarginTop(True);
    EnableMarginLeft(True);
    EnableMarginRight(True);
    EnableMarginBottom(False);
    EnablePunching(False);
    EnableDayBackground(False);
    Margin.Font.Color                     := clWindowText;
    Sizes.Font.Color                      := clWindowText;
    CheckSizes.Checked                    := True;
    Weeks.Checked                         := True;
    Headline.Checked                      := True;
    HolidayList.Checked                   := True;
    DocFrame.Checked                      := False;
    MonthFrame.Checked                    := False;
    Columns.Position                      := 3;
    Orientation.ItemIndex                 := ord(poPortrait);
    MarginTop.Text                        :=  '0.0';
    MarginLeft.Text                       := '15.0';
    MarginRight.Text                      :=  '0.0';
    DayMonthFormat.Text                   := '';
    EnableDayMonth(False);
    FontData[eArbeitsTag, False].Name     := 'Arial';
    FontData[eArbeitsTag, False].Charset  := Default_Charset;
    FontData[eArbeitsTag, False].Size     := 10;
    FontData[eArbeitsTag, False].Color    := clBlack;
    FontData[eArbeitsTag, False].Style    := [];
    FontData[eArbeitsTag, True].Color     := clBlack;
    FontData[eArbeitsTag, True].Style     := [fsUnderline];
    FontData[eSamstag, False].Color       := clAqua;
    FontData[eSamstag, False].Style       := [];
    FontData[eSamstag, True].Color        := clAqua;
    FontData[eSamstag, True].Style        := [fsUnderline];
    FontData[eFeiertag, False].Color      := clRed;
    FontData[eFeiertag, False].Style      := [fsItalic];
    FontData[eFeiertag, True].Color       := clRed;
    FontData[eFeiertag, True].Style       := [fsItalic, fsUnderline];
    FontData[eJahr, False].Size           := 96;
    FontData[eJahr, False].Color          := clAqua;
    FontData[eJahr, False].Style          := [];
    FontData[eFeiertagListe, False].Size  := 10;
    FontData[eFeiertagListe, False].Color := clLime;
    FontData[eFeiertagListe, False].Style := [];
    Element.Items.Clear;
    Element.Items.Add(ElementName[eFont]);
    Element.Items.Add(ElementName[eArbeitsTag]);
    Element.Items.Add(ElementName[eSamstag]);
    Element.Items.Add(ElementName[eFeiertag]);
    Element.Items.Add(ElementName[eJahr]);
    Element.Items.Add(ElementName[eFeiertagListe]);
    Element.ItemIndex := 0;
    try
      IniFile := tIniFile.Create(IniName);
    except
      IniFile := Nil;
    end;
    if IniFile = Nil then
      LoadError
    else
      begin
        try
          Orientation.ItemIndex :=
            IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularEineSeite, Lang_English) + ' ' + IniOrientation,
              Orientation.ItemIndex);
          GetMaxSizes;
          FontData[eArbeitsTag, False].Name :=
            IniFile.ReadString(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularEineSeite, Lang_English)
                                 + ' ' + IniFontName,
                               FontData[eArbeitsTag, False].Name);
          FontData[eArbeitsTag, False].Charset :=
            IniFile.ReadInteger(IniLayout + ' ' + Layout.Text,
                                FormularName(FormularEineSeite, Lang_English)
                                  + ' ' + IniFontCharset,
                                FontData[eArbeitsTag, False].Charset);
          FontData[eArbeitsTag, False].Size :=
            IniFile.ReadInteger(IniLayout + ' ' + Layout.Text,
                                FormularName(FormularEineSeite, Lang_English)
                                  + ' ' + IniFontSize,
                                FontData[eArbeitsTag, False].Size);
          with FontData[eJahr, False] do
            begin
              Size  := IniFile.ReadInteger(
                         IniLayout + ' ' + Layout.Text,
                         FormularName(FormularEineSeite, Lang_English)
                           + ' ' + IniFontElement[eJahr] + ' ' + IniFontSize,
                         Size);
              Color := IniFile.ReadInteger(
                         IniLayout + ' ' + Layout.Text,
                         FormularName(FormularEineSeite, Lang_English)
                           + ' ' + IniFontElement[eJahr] + ' ' + IniFontColor,
                         Color);
              t := IniFile.ReadString(IniLayout + ' ' + Layout.Text,
                                      FormularName(FormularEineSeite, Lang_English)
                                        + ' ' + IniFontElement[eJahr]
                                        + ' ' + IniFontStyle,
                                      IniNotFound);
              if t <> IniNotFound then
                Style := FontStyleSet(t);
            end;
          with FontData[eFeiertagListe, False] do
            begin
              Size  := IniFile.ReadInteger(
                         IniLayout + ' ' + Layout.Text,
                         FormularName(FormularEineSeite, Lang_English)
                           + ' ' + IniFontElement[eFeiertagListe]
                           + ' ' + IniFontSize,
                         Size);
              Color := IniFile.ReadInteger(IniLayout + ' ' + Layout.Text,
                         FormularName(FormularEineSeite, Lang_English)
                           + ' ' + IniFontElement[eFeiertagListe]
                           + ' ' + IniFontColor,
                         Color);
              t := IniFile.ReadString(IniLayout + ' ' + Layout.Text,
                                      FormularName(FormularEineSeite, Lang_English)
                                        + ' ' + IniFontElement[eFeiertagListe]
                                        + ' ' + IniFontStyle,
                                      IniNotFound);
              if t <> IniNotFound then
                Style := FontStyleSet(t);
            end;
          for e := eArbeitsTag to eFeiertag do
            for b := False to True do
              with FontData[e, b] do
                begin
                  Color := IniFile.ReadInteger(
                             IniLayout + ' ' + Layout.Text,
                             FormularName(FormularEineSeite, Lang_English)
                               + ' ' + IniFontElement[e] + IniIsDate[b]
                               + ' ' + IniFontColor,
                             Color);
                  t := IniFile.ReadString(
                         IniLayout + ' ' + Layout.Text,
                         FormularName(FormularEineSeite, Lang_English)
                           + ' ' + IniFontElement[e] + IniIsDate[b]
                           + ' ' + IniFontStyle,
                         IniNotFound);
                  if t <> IniNotFound then
                    Style := FontStyleSet(t);
                end;
          CheckSizes.Checked :=
            Boolean(IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularEineSeite, Lang_English) + ' ' + IniCheckSizes,
              ord(CheckSizes.Checked)));
          Weeks.Checked :=
            Boolean(IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularEineSeite, Lang_English) + ' ' + IniWeeks,
              ord(Weeks.Checked)));
          DocFrame.Checked :=
            IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularEineSeite, Lang_English) + ' ' + IniDocFrame,
              ord(DocFrame.Checked)) = ord(True);
          MonthFrame.Checked :=
            IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularEineSeite, Lang_English) + ' ' + IniMonthFrame,
              ord(MonthFrame.Checked)) = ord(True);
          HeadLine.Checked :=
            IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularEineSeite, Lang_English) + ' ' + IniHeadLine,
              ord(HeadLine.Checked)) = ord(True);
          HolidayList.Checked :=
            IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularEineSeite, Lang_English) + ' ' + IniHolidayList,
              ord(HolidayList.Checked)) =  ord(True);
          Columns.Position :=
            IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularEineSeite, Lang_English) + ' ' + IniColumns,
              Columns.Position);
          t :=
            IniFile.ReadString(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularEineSeite, Lang_English) + ' ' + IniFooter,
              Footer.Text);
          if t > '' then
            if t[1] = '"' then
              t := Copy(t, 2, length(t));
          if t > '' then
            if t[length(t)] = '"' then
              t := Copy(t, 1, length(t) - 1);
          Footer.Text := t;
          PaperHeight.Text :=
            IniFile.ReadString(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularEineSeite, Lang_English) + ' ' + IniHeight,
              PaperHeight.Text);
          PaperWidth.Text :=
            IniFile.ReadString(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularEineSeite, Lang_English) + ' ' + IniWidth,
              PaperWidth.Text);
          MarginTop.Text :=
            IniFile.ReadString(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularEineSeite, Lang_English) + ' ' + IniMarginTop,
              MarginTop.Text);
          MarginLeft.Text :=
            IniFile.ReadString(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularEineSeite, Lang_English) + ' ' + IniMarginLeft,
              MarginLeft.Text);
          MarginRight.Text :=
            IniFile.ReadString(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularEineSeite, Lang_English) + ' ' + IniMarginRight,
              MarginRight.Text);
          LoadFormBkGnd(IniFile, FormularEineSeite);
        except
          LoadError;
        end;
        IniFile.Free;
      end;
  end;

procedure TLayoutForm.LoadMonat;
  var     t       : String;
          b       : Boolean;
          e       : tFontElement;
          IniFile : tIniFile;
  begin
    EnablePictureGroup(False);
    Weeks.enabled                         := False;
    HeadLine.enabled                      := False;
    HolidayList.enabled                   := False;
    IsMergedDate.enabled                  := True;
    IsMergedDate.Checked                  := False;
    EnableOrientation(False);
    DocFrame.enabled                      := False;
    MonthFrame.enabled                    := False;
    EnableColumns(False);
    EnableFooter(False);
    EnablePaperHeight(False, False);
    EnablePaperWidth(False,  False);
    Paper.Font.Color := clGrayText;
    EnableMarginTop(False);
    EnableMarginLeft(False);
    EnableMarginRight(False);
    EnableMarginBottom(False);
    EnableDayBackground(False);
    Margin.Font.Color                     := clGrayText;
    Sizes.Font.Color                      := clGrayText;
    EnablePunching(False);
    Orientation.ItemIndex                 := ord(poLandscape);
    CheckSizes.Checked                    := True;
    DayMonthFormat.Text                   := '';
    EnableDayMonth(False);
    FontData[eArbeitsTag, False].Name     := 'Arial';
    FontData[eArbeitsTag, False].Charset  := Default_Charset;
    FontData[eArbeitsTag, False].Size     := 12;
    FontData[eArbeitsTag, False].Color    := clBlack;
    FontData[eArbeitsTag, False].Style    := [];
    FontData[eArbeitsTag, True].Color     := clBlack;
    FontData[eArbeitsTag, True].Style     := [fsUnderline];
    FontData[eSamstag, False].Color       := clAqua;
    FontData[eSamstag, False].Style       := [];
    FontData[eSamstag, True].Color        := clAqua;
    FontData[eSamstag, True].Style        := [fsUnderline];
    FontData[eFeiertag, False].Color      := clRed;
    FontData[eFeiertag, False].Style      := [fsItalic];
    FontData[eFeiertag, True].Color       := clRed;
    FontData[eFeiertag, True].Style       := [fsItalic, fsUnderline];
    Element.Items.Clear;
    Element.Items.Add(ElementName[eFont]);
    Element.Items.Add(ElementName[eArbeitstag]);
    Element.Items.Add(ElementName[eSamstag]);
    Element.Items.Add(ElementName[eFeiertag]);
    Element.ItemIndex := 0;
    GetMaxSizes;
    try
      IniFile := tIniFile.Create(IniName);
    except
      IniFile := Nil;
    end;
    if IniFile = Nil then
      LoadError
    else
      begin
        try
          FontData[eArbeitsTag, False].Name :=
            IniFile.ReadString(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularMonat, Lang_English) + ' ' + IniFontName,
              FontData[eArbeitsTag, False].Name);
          FontData[eArbeitsTag, False].Charset :=
            IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularMonat, Lang_English) + ' ' + IniFontCharset,
              FontData[eArbeitsTag, False].Charset);
          FontData[eArbeitsTag, False].Size :=
            IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularMonat, Lang_English) + ' ' + IniFontSize,
              FontData[eArbeitsTag, False].Size);
          for e := eArbeitsTag to eFeiertag do
            for b := False to True do
              with FontData[e, b] do
                begin
                  Color := IniFile.ReadInteger(
                             IniLayout + ' ' + Layout.Text,
                             FormularName(FormularMonat, Lang_English)
                               + ' ' + IniFontElement[e] + IniIsDate[b]
                               + ' ' + IniFontColor,
                             Color);
                  t := IniFile.ReadString(
                         IniLayout + ' ' + Layout.Text,
                         FormularName(FormularMonat, Lang_English)
                           + ' ' + IniFontElement[e] + IniIsDate[b]
                           + ' ' + IniFontStyle,
                         IniNotFound);
                  if t <> IniNotFound then
                    Style := FontStyleSet(t);
                end;
          CheckSizes.Checked :=
            Boolean(IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularMonat, Lang_English) + ' ' + IniCheckSizes,
              ord(CheckSizes.Checked)));
          LoadFormBkGnd(IniFile, FormularMonat);
        except
          LoadError;
        end;
        IniFile.Free;
      end;
  end;

procedure tLayoutForm.LoadFormBkGnd(IniFile : tIniFile; Formular : tFormular);
  var t : String;
  begin
    BlattHintergrund :=
      IniFile.ReadString(IniLayout + ' ' + Layout.Text,
                         FormularName(Formular, Lang_English)
                           + ' ' + IniFormBackground, '');
    EnableFormBkgnd;
    t := IniFile.ReadString(IniLayout + ' ' + Layout.Text,
                            FormularName(Formular, Lang_English)
                              + ' ' + IniBackgroundMode,
                            IniBkGndStretch);
    rbStretch.checked := False;
    rbTile.checked    := False;
    if t = IniBkGndStretch then
      rbStretch.Checked := True
    else if t = IniBkGndTile then
      rbTile.Checked    := True;
  end;

procedure TLayoutForm.EnablePictureGroup(MakeEnabled : Boolean);
  var i : Integer;
  begin
    BtnAuswahl.Enabled := MakeEnabled;
    BtnDelete.enabled  := MakeEnabled;
    if MakeEnabled then
      begin
        PictureGroup.Font.Color := clWindowText;
        Pictures.Font.Color     := clWindowText;
      end
    else
      begin
        PictureGroup.Font.Color := clGrayText;
        Pictures.Font.Color     := clGrayText;
        for i := 1 to Pictures.RowCount - 1 do
          Pictures.Cells[1, i] := '';
      end;
    Pictures.enabled := MakeEnabled;
  end;

procedure TLayoutForm.EnableUpDown(Lbl : tLabel; Edit : tEdit;
                                   UpDown     : tUpDown;
                                   MakeEnabled, MustShowHint,
                                   DoPunching : Boolean);
  begin
    if MakeEnabled then
      Lbl.Font.Color := clWindowText
    else
      begin
        Lbl.Font.Color := clGrayText;
        if DoPunching then
          Edit.Text := '';
      end;
    Edit.enabled   := MakeEnabled;
    Edit.ShowHint  := MustShowHint and (Edit.Hint > '');
    UpDown.enabled := MakeEnabled;
  end;

procedure TLayoutForm.EnablePaperHeight(MakeEnabled, MustShowHint : Boolean);
  begin
    EnableUpDown(LabelHeight, PaperHeight, UpDownHeight,
                 MakeEnabled, MustShowHint, false);
  end;

procedure TLayoutForm.EnablePaperWidth(MakeEnabled, MustShowHint : Boolean);
  begin
    EnableUpDown(LabelWidth, PaperWidth, UpDownWidth,
                 MakeEnabled, MustShowHint, false);
  end;

procedure TLayoutForm.EnableMarginTop(MakeEnabled : Boolean);
  begin
    EnableUpDown(LabelTop, MarginTop, UpDownTop,
                 MakeEnabled, MakeEnabled, false);
  end;

procedure TLayoutForm.EnableMarginLeft(MakeEnabled : Boolean);
  begin
    EnableUpDown(LabelLeft, MarginLeft, UpDownLeft,
                 MakeEnabled, MakeEnabled, false);
  end;

procedure TLayoutForm.EnableMarginRight(MakeEnabled : Boolean);
  begin
    EnableUpDown(LabelRight, MarginRight, UpDownRight,
                 MakeEnabled, MakeEnabled, false);
  end;

procedure TLayoutForm.EnableMarginBottom(MakeEnabled : Boolean);
  begin
    EnableUpDown(LabelBottom, MarginBottom, UpDownBottom,
                 MakeEnabled, MakeEnabled, false);
  end;

procedure TLayoutForm.EnableOrientation(MakeEnabled : Boolean);
  begin
    Orientation.enabled := MakeEnabled;
    if MakeEnabled then
      Orientation.Font.Color := clWindowText
    else
      Orientation.Font.Color := clGrayText;
  end;

procedure TLayoutForm.EnableFooter(MakeEnabled : Boolean);
  begin
    Footer.enabled := MakeEnabled;
    if MakeEnabled then
      LabelFooter.Font.Color := clWindowText
    else
      begin
        LabelFooter.Font.Color := clGrayText;
        Footer.Text := '';
      end;
  end;

procedure TLayoutForm.LoadBilder;
  var     e       : tFontElement;
          b       : Boolean;
          t       : String;
          i       : Integer;
          IniFile : tIniFile;
  begin
    Element.Items.Clear;
    Element.Items.Add(ElementName[eFont]);
    Element.Items.Add(ElementName[eArbeitsTag]);
    Element.Items.Add(ElementName[eSamstag]);
    Element.Items.Add(ElementName[eFeiertag]);
    Element.Items.Add(ElementName[eFeiertagListe]);
    LastMonat                               := 0;
    Element.ItemIndex                       := 0;
    EnablePictureGroup(True);
    Weeks.enabled                           := True;
    HeadLine.enabled                        := False;
    Headline.Checked                        := False;
    HolidayList.enabled                     := False;
    IsMergedDate.enabled                    := False;
    IsMergedDate.Checked                    := False;
    EnableOrientation(False);
    DocFrame.enabled                        := False;
    MonthFrame.enabled                      := False;
    EnableColumns(False);
    EnableFooter(False);
    EnablePaperHeight(False, False);
    EnablePaperWidth(False,  False);
    Paper.Font.Color := clGrayText;
    EnableMarginTop(True);
    EnableMarginLeft(True);
    EnableMarginRight(True);
    EnableMarginBottom(True);
    Margin.Font.Color                       := clWindowText;
    Sizes.Font.Color                        := clWindowText;
    EnablePunching(False);
    Orientation.ItemIndex                   := ord(poPortrait);
    Weeks.Checked                           := True;
    CheckSizes.Checked                      := True;
    DayMonthFormat.Text                     := '';
    EnableDayMonth(False);
    FontData[eArbeitsTag,    False].Name    := 'Arial';
    FontData[eArbeitsTag,    False].Charset := Default_Charset;
    FontData[eArbeitsTag,    False].Size    := 24;
    FontData[eArbeitsTag,    False].Color   := clBlack;
    FontData[eArbeitsTag,    False].Style   := [];
    FontData[eArbeitsTag,    True].Color    := clBlack;
    FontData[eArbeitsTag,    True].Style    := [fsUnderline];
    FontData[eSamstag,       False].Color   := clAqua;
    FontData[eSamstag,       False].Style   := [];
    FontData[eSamstag,       True].Color    := clAqua;
    FontData[eSamstag,       True].Style    := [fsUnderline];
    FontData[eFeierTag,      False].Color   := clRed;
    FontData[eFeierTag,      False].Style   := [fsItalic];
    FontData[eFeierTag,      True].Color    := clRed;
    FontData[eFeierTag,      True].Style    := [fsItalic, fsUnderline];
    FontData[eFeiertagListe, False].Color   := clBlack;
    FontData[eFeiertagListe, False].Style   := [];
    FontData[eFeiertagListe, False].Size    := 12;
    MarginTop.Text                          := '0.0';
    MarginLeft.Text                         := '10.0';
    MarginRight.Text                        := '10.0';
    MarginBottom.Text                       := '10.0';
    GetMaxSizes;
    try
      IniFile := tIniFile.Create(IniName);
    except
      IniFile := Nil;
    end;
    if IniFile = Nil then
      LoadError
    else
      begin
        try
          FontData[eArbeitsTag, False].Name :=
            IniFile.ReadString(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularBilder, Lang_English) + ' ' + IniFontName,
              FontData[eArbeitsTag, False].Name);
          FontData[eArbeitsTag, False].Charset :=
            IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularBilder, Lang_English) + ' ' + IniFontCharset,
              FontData[eArbeitsTag, False].Charset);
          FontData[eArbeitsTag, False].Size :=
            IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularBilder, Lang_English) + ' ' + IniFontSize,
              FontData[eArbeitsTag, False].Size);
          FontData[eFeiertagListe, False].Size :=
            IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularBilder, Lang_English)
                + ' ' + IniFontElement[eFeiertagListe] + ' ' + IniFontSize,
              FontData[eFeiertagListe, False].Size);
          FontData[eFeiertagListe, False].Color :=
            IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularBilder, Lang_English)
                + ' ' + IniFontElement[eFeiertagListe] + ' ' + IniFontColor,
              FontData[eFeiertagListe, False].Color);
          t := IniFile.ReadString(
                 IniLayout + ' ' + Layout.Text,
                 FormularName(FormularBilder, Lang_English)
                   + ' ' + IniFontElement[eFeiertagListe] + ' ' + IniFontStyle,
                 IniNotFound);
          if t <> IniNotFound then
            FontData[eFeiertagListe, False].Style := FontStyleSet(t);
          for e := eArbeitsTag to eFeiertag do
            for b := False to True do
              with FontData[e, b] do
                begin
                  Color :=
                    IniFile.ReadInteger(
                      IniLayout + ' ' + Layout.Text,
                      FormularName(FormularBilder, Lang_English) + ' '
                        + IniFontElement[e] + IniIsDate[b] + ' ' + IniFontColor,
                      Color);
                  t := IniFile.ReadString(
                         IniLayout + ' ' + Layout.Text,
                         FormularName(FormularBilder, Lang_English) + ' '
                           + IniFontElement[e] + IniIsDate[b] + ' ' + IniFontStyle,
                         IniNotFound);
                  if t <> IniNotFound then
                    Style := FontStyleSet(t);
                end;
          CheckSizes.Checked :=
            Boolean(IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularBilder, Lang_English) + ' ' + IniCheckSizes,
              ord(CheckSizes.Checked)));
          Weeks.Checked :=
            Boolean(IniFile.ReadInteger(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularBilder, Lang_English) + ' ' + IniCheckSizes,
              ord(Weeks.Checked)));
          MarginTop.Text :=
            IniFile.ReadString(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularBilder, Lang_English) + ' ' + IniMarginTop,
              MarginTop.Text);
          MarginLeft.Text :=
            IniFile.ReadString(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularBilder, Lang_English) + ' ' + IniMarginLeft,
              MarginLeft.Text);
          MarginRight.Text :=
            IniFile.ReadString(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularBilder, Lang_English) + ' ' + IniMarginRight,
              MarginRight.Text);
          MarginBottom.Text :=
            IniFile.ReadString(
              IniLayout + ' ' + Layout.Text,
              FormularName(FormularBilder, Lang_English) + ' ' + IniMarginBottom,
              MarginBottom.Text);
          for i := 1 to 12 do
            begin
              Bilder[i] :=
                IniFile.ReadString(
                  IniLayout + ' ' + Layout.Text,
                  FormularName(FormularBilder, Lang_English) + ' ' + IniPicture
                    + ' ' + IntToDez(i, 2, False),
                  '');
              Pictures.Cells[1, i] := ExtractFileName(Bilder[i]);
              Pictures.Cells[2, i] :=
                IniFile.ReadString(
                  IniLayout + ' ' + Layout.Text,
                  FormularName(FormularBilder, Lang_English) + ' ' + IniComment
                    + ' ' + IntToDez(i, 2, False),
                  '');
            end;
          LoadFormBkGnd(IniFile, FormularBilder);
          TagHintergrund :=
            IniFile.ReadString(IniLayout + ' ' + Layout.Text,
                               FormularName(FormularBilder, Lang_English)
                                 + ' ' + IniDayBackground, '');
          EnableDayBackground(True);
        except
          LoadError;
        end;
        IniFile.Free;
      end;
  end;

procedure TLayoutForm.ElementClick(Sender: TObject);
  begin
    case tFormular(MainForm.Formular.ItemIndex) of
      FormularEineSeite,
      FormularMonat,
      FormularBilder:
        case GetElement of
          eArbeitsTag .. eFeiertag:
            IsMergedDate.enabled := True;
        else
          IsMergedDate.enabled := False;
        end;
    end;
  end;

procedure TLayoutForm.PunchesChange(Sender: TObject);
  begin
    if Punches.enabled then
      begin
        if Punches.Position = 6 then
          begin
            EnableUpDown(LabelPunch3, Punch3, UpDownPunch3, true, true, true);
            EnableUpDown(LabelPunch2, Punch2, UpDownPunch2, true, true, true);
            EnableUpDown(LabelPunch1, Punch1, UpDownPunch3, true, true, true);
          end
        else
          if Punches.Position = 4 then
            begin
              EnableUpDown(LabelPunch3, Punch3, UpDownPunch3, false, false, true);
              EnableUpDown(LabelPunch2, Punch2, UpDownPunch2, true,  true,  true);
              EnableUpDown(LabelPunch1, Punch1, UpDownPunch1, true,  true,  true);
            end
          else
            begin
              EnableUpDown(LabelPunch3, Punch3, UpDownPunch3, false, false, true);
              EnableUpDown(LabelPunch3, Punch2, UpDownPunch2, false, false, true);
              EnableUpDown(LabelPunch1, Punch1, UpDownPunch1, true,  true,  true);
            end;
        PunchesCheck;
        IsDirty := True;
      end;
  end;

procedure TLayoutForm.SaveBilder;
  var     e       : tFontElement;
          b       : Boolean;
          i       : Integer;
          IniFile : tIniFile;
  begin
    try
      IniFile := tIniFile.Create(IniName);
    except
      IniFile := Nil;
    end;
    if IniFile = Nil then
      SaveError
    else
      begin
        try
          IniFile.WriteString(
            IniLayout + ' ' + Layout.Text,
            FormularName(FormularBilder, Lang_English) + ' ' + IniFontName,
            FontData[eArbeitsTag, False].Name);
          IniFile.WriteInteger(
            IniLayout + ' ' + Layout.Text,
            FormularName(FormularBilder, Lang_English) + ' ' + IniFontCharset,
            FontData[eArbeitsTag, False].Charset);
          IniFile.WriteInteger(
            IniLayout + ' ' + Layout.Text,
            FormularName(FormularBilder, Lang_English) + ' ' + IniFontSize,
            FontData[eArbeitsTag, False].Size);
          IniFile.WriteInteger(
            IniLayout + ' ' + Layout.Text,
            FormularName(FormularBilder, Lang_English)
              + ' ' + IniFontElement[eFeiertagListe] + ' ' + IniFontSize,
            FontData[eFeiertagListe, False].Size);
          IniFile.WriteString(
            IniLayout + ' ' + Layout.Text,
            FormularName(FormularBilder, Lang_English)
              + ' ' + IniFontElement[eFeiertagListe] + ' ' + IniFontColor,
            '0x' + IntToHex(FontData[eFeiertagListe, False].Color, 6));
          IniFile.WriteString(
            IniLayout + ' ' + Layout.Text,
            FormularName(FormularBilder, Lang_English)
              + ' ' + IniFontElement[eFeiertagListe] + ' ' + IniFontStyle,
            FontStyleString(FontData[eFeiertagListe, False].Style));
          for e := eArbeitsTag to eFeiertag do
            for b := False to True do
              with FontData[e, b] do
                begin
                  IniFile.WriteString(
                    IniLayout + ' ' + Layout.Text,
                    FormularName(FormularBilder, Lang_English)
                      + ' ' + IniFontElement[e] + IniIsDate[b] + ' ' + IniFontColor,
                    '0x' + IntToHex(Color, 6));
                  IniFile.WriteString(
                    IniLayout + ' ' + Layout.Text,
                    FormularName(FormularBilder, Lang_English)
                      + ' ' + IniFontElement[e] + IniIsDate[b] + ' ' + IniFontStyle,
                    FontStyleString(Style));
                end;
          IniFile.WriteInteger(
            IniLayout + ' ' + Layout.Text,
            FormularName(FormularBilder, Lang_English) + ' ' + IniCheckSizes,
            ord(CheckSizes.Checked));
          IniFile.WriteInteger(
            IniLayout + ' ' + Layout.Text,
            FormularName(FormularBilder, Lang_English) + ' ' + IniWeeks,
            ord(Weeks.Checked));
          IniFile.WriteString(
            IniLayout + ' ' + Layout.Text,
            FormularName(FormularBilder, Lang_English) + ' ' + IniMarginTop,
            MarginTop.Text);
          IniFile.WriteString(
            IniLayout + ' ' + Layout.Text,
            FormularName(FormularBilder, Lang_English) + ' ' + IniMarginLeft,
            MarginLeft.Text);
          IniFile.WriteString(
            IniLayout + ' ' + Layout.Text,
            FormularName(FormularBilder, Lang_English) + ' ' + IniMarginRight,
            MarginRight.Text);
          IniFile.WriteString(
            IniLayout + ' ' + Layout.Text,
            FormularName(FormularBilder, Lang_English) + ' ' + IniMarginBottom,
            MarginBottom.Text);
          for i := 1 to 12 do
            begin
              IniFile.WriteString(
                IniLayout + ' ' + Layout.Text,
                FormularName(FormularBilder, Lang_English) + ' ' + IniPicture + ' '
                  + IntToDez(i, 2, False),
                Bilder[i]);
              IniFile.WriteString(
                IniLayout + ' ' + Layout.Text,
                FormularName(FormularBilder, Lang_English) + ' ' + IniComment + ' '
                  + IntToDez(i, 2, False),
                Pictures.Cells[2,i]);
            end;
          IniFile.WriteString(IniLayout + ' ' + Layout.Text,
                              FormularName(FormularBilder, Lang_English)
                                + ' ' + IniFormBackground, BlattHintergrund);
          IniFile.WriteString(IniLayout + ' ' + Layout.Text,
                              FormularName(FormularBilder, Lang_English)
                                + ' ' + IniDayBackground, TagHintergrund);
          SaveBkGndMode(IniFile, FormularBilder);
        except
          SaveError;
        end;
        IniFile.Free;
      end;
  end;

procedure TLayoutForm.SelectPicture(var FileName : String);
  var     IniFile : tIniFile;
  begin
    Opendialog.FileName := FileName;
    IniFile := tIniFile.Create(IniName);
    if IniFile <> Nil then
      begin
        Opendialog.FilterIndex := IniFile.ReadInteger(
          IniFileTypes, IniGraphicImport, 0);
        IniFile.Free;
      end;
    If OpenDialog.Execute then
      begin
        IniFile := tIniFile.Create(IniName);
        if IniFile <> Nil then
          begin
            IniFile.WriteInteger(
              IniFileTypes, IniGraphicImport, Opendialog.FilterIndex);
            IniFile.Free;
          end;
        FileName := Opendialog.FileName;
        IsDirty  := True;
      end;
  end;

procedure TLayoutForm.BtnAuswahlClick(Sender: TObject);
  begin
    SelectPicture(Bilder[Pictures.Row]);
    Pictures.Cells[1, Pictures.Row] := ExtractFileName(Bilder[Pictures.Row]);
    LayoutCheck;
  end;

{$ifdef fpc}
  procedure TLayoutForm.FontDialogApply(Sender: TObject);
    begin
      FontDialogApplyClick;
    end;
{$else}
  procedure TLayoutForm.FontDialogApply(Sender: TObject; Wnd: HWND);
    begin
      FontDialogApplyClick;
    end;
{$endif}

procedure TLayoutForm.FontDialogApplyClick;
  begin
    with FontDialog do
      case GetElement of
        eFont:
          begin
            FontData[eArbeitsTag, False].Name    := Font.Name;
            FontData[eArbeitsTag, False].Charset := Font.Charset;
            FontData[eArbeitsTag, False].Size    := Font.Size;
          end;
        eJahr:
          begin
            FontData[eJahr, false].Size  := Font.Size;
            FontData[eJahr, false].Color := Font.Color;
            FontData[eJahr, false].Style := Font.Style;
          end;
        eArbeitsTag:
          begin
            FontData[eArbeitsTag, IsMergedDate.Checked].Color := Font.Color;
            FontData[eArbeitsTag, IsMergedDate.Checked].Style := Font.Style;
          end;
        eSamstag:
          begin
            FontData[eSamstag, IsMergedDate.Checked].Color := Font.Color;
            FontData[eSamstag, IsMergedDate.Checked].Style := Font.Style;
          end;
        eFeiertag:
          begin
            FontData[eFeiertag, IsMergedDate.Checked].Color := Font.Color;
            FontData[eFeiertag, IsMergedDate.Checked].Style := Font.Style;
          end;
        eFeiertagListe:
          begin
            FontData[eFeiertagListe, false].Color := Font.Color;
            FontData[eFeiertagListe, false].Size  := Font.Size;
            FontData[eFeiertagListe, false].Style := Font.Style;
          end;
        eTermin:
          begin
            FontData[eTermin, False].Name    := Font.Name;
            FontData[eTermin, False].Charset := Font.Charset;
            FontData[eTermin, False].Size    := Font.Size;
            FontData[eTermin, False].Color   := Font.Color;
            FontData[eTermin, False].Style   := Font.Style;
          end;
      end;
    LayoutCheck;
    IsDirty := True;
  end;

procedure TLayoutForm.BtnDeleteClick(Sender: TObject);
  begin
    Bilder[Pictures.Row]            := '';
    Pictures.Cells[1, Pictures.Row] := '';
    IsDirty                         := True;
  end;

procedure TLayoutForm.ColumnsEditExit(Sender: TObject);
  var     s : String;
  begin
    s := IntToStr(Columns.Position);
    ColumnsEdit.Text := s;
    IsDirty := True;
  end;

procedure TLayoutForm.UpDownClick(Edit: TEdit; Button: TUDBtnType);
  var     r     : Real;
          Error : Integer;
          s     : String;
  begin
    val(Edit.Text, r, Error);
    if Error <> 0 then
      r := 0;
    if Button = btNext then
      r := r + 0.1
    else
      r := r - 0.1;
    if r < 0 then
      r := 0;
    str(r : 0 : 1, s);
    Edit.Text := s;
  end;

procedure TLayoutForm.UpDownHeightClick(Sender: TObject; Button: TUDBtnType);
  begin
    UpDownClick(PaperHeight, Button);
    PaperHeightChange(Sender);
  end;

procedure TLayoutForm.UpDownWidthClick(Sender: TObject; Button: TUDBtnType);
  begin
    UpDownClick(PaperWidth, Button);
    PaperWidthChange(Sender);
  end;

procedure TLayoutForm.UpDownTopClick(Sender: TObject; Button: TUDBtnType);
  begin
    UpDownClick(MarginTop, Button);
    MarginTopChange(Sender);
  end;

procedure TLayoutForm.UpDownLeftClick(Sender: TObject; Button: TUDBtnType);
  begin
    UpDownClick(MarginLeft, Button);
    MarginLeftChange(Sender);
  end;

procedure TLayoutForm.UpDownRightClick(Sender: TObject; Button: TUDBtnType);
  begin
    UpDownClick(MarginRight, Button);
    MarginRightChange(Sender);
  end;

procedure TLayoutForm.UpDownBottomClick(Sender: TObject; Button: TUDBtnType);
  begin
    UpDownClick(MarginBottom, Button);
    MarginBottomChange(Sender);
  end;

procedure TLayoutForm.UpDownPunch1Click(Sender: TObject; Button: TUDBtnType);
  begin
    UpDownClick(Punch1, Button);
    PunchExit(Sender);
  end;

procedure TLayoutForm.UpDownPunch2Click(Sender: TObject; Button: TUDBtnType);
  begin
    UpDownClick(Punch2, Button);
    PunchExit(Sender);
  end;

procedure TLayoutForm.UpDownPunch3Click(Sender: TObject; Button: TUDBtnType);
  begin
    UpDownClick(Punch3, Button);
    PunchExit(Sender);
  end;

procedure TLayoutForm.MaxVal3(Edit, Max, Delta : tEdit);
  var     r     : Real;
          Error : Integer;
  begin
    val(Delta.Text, r, Error);
    if Error <> 0 then
      r := 0;
    MaxVal4(Edit, Max, 1, r);
  end;

procedure TLayoutForm.MaxVal4(Edit, Max : tEdit; Divisor, Delta : Real);
  var     r     : Real;
          Error : Integer;
  begin
    val(Max.Text, r, Error);
    if Error <> 0 then
      r := 0;
    r := r / Divisor - Delta;
    if r < 0 then
      r := 0;
    MaxVal2(Edit, r);
  end;

procedure TLayoutForm.MaxVal2(Edit : tEdit; Max : Real);
  var     r     : Real;
          Error : Integer;
          s     : String;
  begin
    val(Edit.Text, r, Error);
    if Error <> 0 then
      r := Max
    else
      if r > Max then
        r := Max;
    str(r : 0 : 1, s);
    Edit.Text := s;
  end;

procedure tLayoutForm.PunchesCheck;
  begin
    if Punches.Position = 6 then
      begin
        MaxVal4(Punch3, PaperHeight, 2, MMLoch / 2);
        MaxVal4(Punch2, Punch3,      1, MMLoch);
        MaxVal4(Punch1, Punch2,      1, MMLoch);
      end
    else
      if Punches.Position = 4 then
        begin
          MaxVal4(Punch2, PaperHeight, 2, MMLoch / 2);
          MaxVal4(Punch1, Punch2,      1, MMLoch);
        end
      else
        MaxVal4(Punch1,   Paperheight, 2, MMLoch / 2);
  end;

procedure TLayoutForm.PaperHeightChange(Sender: TObject);
  begin
    MaxVal2(PaperHeight, HoeheMax);
    PunchesCheck;
    IsDirty := True;
  end;

procedure TLayoutForm.PunchExit(Sender: TObject);
  begin
    PunchesCheck;
    IsDirty := True;
  end;

procedure TLayoutForm.MarginTopChange(Sender: TObject);
  begin
    MaxVal4(MarginTop,    PaperHeight, 1, 0);
    MaxVal3(MarginBottom, PaperHeight, MarginTop);
    IsDirty := True;
  end;

procedure TLayoutForm.MarginBottomChange(Sender: TObject);
  begin
    MaxVal3(MarginBottom, PaperHeight, MarginTop);
    IsDirty := True;
  end;

procedure TLayoutForm.PaperWidthChange(Sender: TObject);
  begin
    MaxVal2(PaperWidth,  BreiteMax);
    MaxVal4(MarginLeft,  PaperWidth, 1, 0);
    MaxVal3(MarginRight, PaperWidth, MarginLeft);
    IsDirty := True;
  end;

procedure TLayoutForm.MarginLeftChange(Sender: TObject);
  begin
    MaxVal4(MarginLeft,  PaperWidth, 1, 0);
    if tFormular(MainForm.Formular.ItemIndex) <> FormularRingbuch then
      MaxVal3(MarginRight, PaperWidth, MarginLeft);
    IsDirty := True;
  end;

procedure TLayoutForm.MarginRightChange(Sender: TObject);
  begin
    MaxVal3(MarginRight, PaperWidth, MarginLeft);
    IsDirty := True;
  end;

procedure TLayoutForm.PicturesSelectCell(
            Sender: TObject; ACol, ARow: Integer;
            var CanSelect: Boolean);
  begin
    if ACol = 0 then;
    if ARow = 0 then;
    CanSelect := True;
    LayoutCheck;
  end;

procedure TLayoutForm.PicturesClick(Sender: TObject);
  begin
    if Pictures.Col = 2 then
      Pictures.Options := Pictures.Options + [goEditing]
    else
      Pictures.Options := Pictures.Options - [goEditing];
    if LastMonat <> Pictures.Row then
      LayoutCheck;
    LastMonat := Pictures.Row;
  end;

procedure TLayoutForm.BtnFormBkgndClick(Sender: TObject);
  begin
    SelectPicture(BlattHintergrund);
    EnableFormBkgnd;
  end;

procedure TLayoutForm.EnableFormBkgnd;
  begin
    if BlattHintergrund = '' then
      begin
        EditFormBkgnd.Text       := '';
        cbFormBackground.checked := False;
      end
    else
      begin
        EditFormBkgnd.Text       := ExtractFileName(BlattHintergrund);
        cbFormBackground.checked := True;
      end;
    rbStretch.enabled     := cbFormBackground.checked;
    rbTile.enabled        := cbFormBackground.checked;
    EditFormBkGnd.enabled := cbFormBackground.checked;
  end;

procedure TLayoutForm.cbFormBackgroundClick(Sender: TObject);
  begin
    if cbFormBackground.checked then
      if BlattHintergrund = '' then
        BtnFormBkgndClick(Sender)
      else
        EnableFormBkgnd
    else
      begin
        BlattHintergrund := '';
        EnableFormBkgnd;
      end;
    IsDirty := True;
  end;

procedure TLayoutForm.OrientationClick(Sender: TObject);
  var     h, b, o, l, r, u : String;
  begin
    h := PaperHeight.Text;
    b := PaperWidth.Text;
    o := MarginTop.Text;
    l := MarginLeft.Text;
    r := MarginRight.Text;
    u := MarginBottom.Text;

    PaperHeight.Text  := b;
    PaperWidth.Text   := h;
    MarginTop.Text    := l;
    MarginLeft.Text   := o;
    MarginRight.Text  := u;
    MarginBottom.Text := r;
    IsDirty           := True;
  end;

procedure TLayoutForm.PicturesExit(Sender: TObject);
  begin
    LastMonat := 0;
  end;

procedure TLayoutForm.cbDayBackgroundClick(Sender: TObject);
  begin
    if cbDayBackground.checked then
      if TagHintergrund = '' then
        BtnDayBkgndClick(Sender)
      else
        EnableDayBackground(True)
    else
      begin
        TagHintergrund := '';
        EnableDayBackground(True);
      end;
    IsDirty := True;
  end;

procedure TLayoutForm.BtnDayBkGndClick(Sender: TObject);
  begin
    SelectPicture(TagHintergrund);
    EnableDayBackground(True);
  end;

procedure TLayoutForm.FormCloseQuery(Sender: TObject;
                                     var CanClose: Boolean);
  var     i : Integer;
  begin
    if IsDirty then
      case SysLocale.PriLangID of
        Lang_German:
          i := MsgBox('Eingaben verwerfen?', Caption,
                      mb_OkCancel or mb_IconQuestion);
      else
        i := MsgBox('Cancel changes?', Caption,
                    mb_OkCancel or mb_IconQuestion);
      end
    else
      i := idOk;
    if i = idOk Then
      CanClose := True
    else
      CanClose := False;
  end;

procedure TLayoutForm.SetDirty(Sender: TObject);
  begin
    IsDirty := True;
  end;

procedure TLayoutForm.PicturesMouseUp(Sender: TObject; Button: TMouseButton;
                                      Shift: TShiftState; X, Y: Integer);
  begin
    IsDirty := True;
  end;

procedure TLayoutForm.PicturesKeyPress(Sender: TObject; var Key: Char);
  begin
    IsDirty := True;
  end;

end.
